{# Import the theme's layout. #}
{% extends "!layout.html" %}

{%- block footer %}
<footer class="footer">

  <div class="container">
    <ul class="list-inline">
      <li><a href="https://github.com/vispy/vispy">GitHub</a></li>
      <li>·</li>
      <li><a href=""><a href="https://github.com/vispy/vispy/blob/master/examples/">Examples</a></li>
      <li>·</li>
      <li><a href="https://github.com/vispy/vispy/issues">Issues</a></li>
      <li>·</li>
      <li><a href="https://github.com/vispy/vispy/releases">Releases</a></li>
      <li class="pull-right"><a href="#">Back to top</a></li>
    </ul>
    <ul class="list-unstyled small">
      <li>{{ copyright }}</li>
      <li>Code licensed under
        <a href="http://opensource.org/licenses/BSD-3-Clause">BSD license</a>,
        documentation under <a href="http://creativecommons.org/licenses/by/3.0/">CC BY 3.0 </a>
      </li>
      <li>
        Made with <a href="http://sphinx-doc.org">sphinx</a> using the excellent
        <a href="https://github.com/ryan-roemer/sphinx-bootstrap-theme">bootstrap theme</a>
      </li>
    </ul>
  </div>
</footer>
{%- endblock %}
