define(["@jupyter-widgets/base"],(function(e){return function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=4)}([function(e,t,n){var r;
/*!
 * jQuery JavaScript Library v3.4.1
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2019-05-01T21:04Z
 */!function(t,n){"use strict";"object"==typeof e.exports?e.exports=t.document?n(t,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return n(e)}:n(t)}("undefined"!=typeof window?window:this,(function(n,i){"use strict";var o=[],s=n.document,a=Object.getPrototypeOf,u=o.slice,l=o.concat,c=o.push,f=o.indexOf,h={},p=h.toString,d=h.hasOwnProperty,g=d.toString,v=g.call(Object),m={},y=function(e){return"function"==typeof e&&"number"!=typeof e.nodeType},_=function(e){return null!=e&&e===e.window},b={type:!0,src:!0,nonce:!0,noModule:!0};function w(e,t,n){var r,i,o=(n=n||s).createElement("script");if(o.text=e,t)for(r in b)(i=t[r]||t.getAttribute&&t.getAttribute(r))&&o.setAttribute(r,i);n.head.appendChild(o).parentNode.removeChild(o)}function x(e){return null==e?e+"":"object"==typeof e||"function"==typeof e?h[p.call(e)]||"object":typeof e}var E=function(e,t){return new E.fn.init(e,t)},T=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;function C(e){var t=!!e&&"length"in e&&e.length,n=x(e);return!y(e)&&!_(e)&&("array"===n||0===t||"number"==typeof t&&t>0&&t-1 in e)}E.fn=E.prototype={jquery:"3.4.1",constructor:E,length:0,toArray:function(){return u.call(this)},get:function(e){return null==e?u.call(this):e<0?this[e+this.length]:this[e]},pushStack:function(e){var t=E.merge(this.constructor(),e);return t.prevObject=this,t},each:function(e){return E.each(this,e)},map:function(e){return this.pushStack(E.map(this,(function(t,n){return e.call(t,n,t)})))},slice:function(){return this.pushStack(u.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(e){var t=this.length,n=+e+(e<0?t:0);return this.pushStack(n>=0&&n<t?[this[n]]:[])},end:function(){return this.prevObject||this.constructor()},push:c,sort:o.sort,splice:o.splice},E.extend=E.fn.extend=function(){var e,t,n,r,i,o,s=arguments[0]||{},a=1,u=arguments.length,l=!1;for("boolean"==typeof s&&(l=s,s=arguments[a]||{},a++),"object"==typeof s||y(s)||(s={}),a===u&&(s=this,a--);a<u;a++)if(null!=(e=arguments[a]))for(t in e)r=e[t],"__proto__"!==t&&s!==r&&(l&&r&&(E.isPlainObject(r)||(i=Array.isArray(r)))?(n=s[t],o=i&&!Array.isArray(n)?[]:i||E.isPlainObject(n)?n:{},i=!1,s[t]=E.extend(l,o,r)):void 0!==r&&(s[t]=r));return s},E.extend({expando:"jQuery"+("3.4.1"+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isPlainObject:function(e){var t,n;return!(!e||"[object Object]"!==p.call(e))&&(!(t=a(e))||"function"==typeof(n=d.call(t,"constructor")&&t.constructor)&&g.call(n)===v)},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},globalEval:function(e,t){w(e,{nonce:t&&t.nonce})},each:function(e,t){var n,r=0;if(C(e))for(n=e.length;r<n&&!1!==t.call(e[r],r,e[r]);r++);else for(r in e)if(!1===t.call(e[r],r,e[r]))break;return e},trim:function(e){return null==e?"":(e+"").replace(T,"")},makeArray:function(e,t){var n=t||[];return null!=e&&(C(Object(e))?E.merge(n,"string"==typeof e?[e]:e):c.call(n,e)),n},inArray:function(e,t,n){return null==t?-1:f.call(t,e,n)},merge:function(e,t){for(var n=+t.length,r=0,i=e.length;r<n;r++)e[i++]=t[r];return e.length=i,e},grep:function(e,t,n){for(var r=[],i=0,o=e.length,s=!n;i<o;i++)!t(e[i],i)!==s&&r.push(e[i]);return r},map:function(e,t,n){var r,i,o=0,s=[];if(C(e))for(r=e.length;o<r;o++)null!=(i=t(e[o],o,n))&&s.push(i);else for(o in e)null!=(i=t(e[o],o,n))&&s.push(i);return l.apply([],s)},guid:1,support:m}),"function"==typeof Symbol&&(E.fn[Symbol.iterator]=o[Symbol.iterator]),E.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),(function(e,t){h["[object "+t+"]"]=t.toLowerCase()}));var S=
/*!
 * Sizzle CSS Selector Engine v2.3.4
 * https://sizzlejs.com/
 *
 * Copyright JS Foundation and other contributors
 * Released under the MIT license
 * https://js.foundation/
 *
 * Date: 2019-04-08
 */
function(e){var t,n,r,i,o,s,a,u,l,c,f,h,p,d,g,v,m,y,_,b="sizzle"+1*new Date,w=e.document,x=0,E=0,T=ue(),C=ue(),S=ue(),A=ue(),D=function(e,t){return e===t&&(f=!0),0},z={}.hasOwnProperty,k=[],R=k.pop,N=k.push,j=k.push,O=k.slice,F=function(e,t){for(var n=0,r=e.length;n<r;n++)if(e[n]===t)return n;return-1},L="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",P="[\\x20\\t\\r\\n\\f]",q="(?:\\\\.|[\\w-]|[^\0-\\xa0])+",M="\\["+P+"*("+q+")(?:"+P+"*([*^$|!~]?=)"+P+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+q+"))|)"+P+"*\\]",I=":("+q+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+M+")*)|.*)\\)|)",B=new RegExp(P+"+","g"),W=new RegExp("^"+P+"+|((?:^|[^\\\\])(?:\\\\.)*)"+P+"+$","g"),H=new RegExp("^"+P+"*,"+P+"*"),U=new RegExp("^"+P+"*([>+~]|"+P+")"+P+"*"),$=new RegExp(P+"|>"),X=new RegExp(I),V=new RegExp("^"+q+"$"),G={ID:new RegExp("^#("+q+")"),CLASS:new RegExp("^\\.("+q+")"),TAG:new RegExp("^("+q+"|[*])"),ATTR:new RegExp("^"+M),PSEUDO:new RegExp("^"+I),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+P+"*(even|odd|(([+-]|)(\\d*)n|)"+P+"*(?:([+-]|)"+P+"*(\\d+)|))"+P+"*\\)|)","i"),bool:new RegExp("^(?:"+L+")$","i"),needsContext:new RegExp("^"+P+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+P+"*((?:-\\d)?\\d*)"+P+"*\\)|)(?=[^-]|$)","i")},K=/HTML$/i,Y=/^(?:input|select|textarea|button)$/i,Z=/^h\d$/i,J=/^[^{]+\{\s*\[native \w/,Q=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,ee=/[+~]/,te=new RegExp("\\\\([\\da-f]{1,6}"+P+"?|("+P+")|.)","ig"),ne=function(e,t,n){var r="0x"+t-65536;return r!=r||n?t:r<0?String.fromCharCode(r+65536):String.fromCharCode(r>>10|55296,1023&r|56320)},re=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,ie=function(e,t){return t?"\0"===e?"�":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},oe=function(){h()},se=be((function(e){return!0===e.disabled&&"fieldset"===e.nodeName.toLowerCase()}),{dir:"parentNode",next:"legend"});try{j.apply(k=O.call(w.childNodes),w.childNodes),k[w.childNodes.length].nodeType}catch(e){j={apply:k.length?function(e,t){N.apply(e,O.call(t))}:function(e,t){for(var n=e.length,r=0;e[n++]=t[r++];);e.length=n-1}}}function ae(e,t,r,i){var o,a,l,c,f,d,m,y=t&&t.ownerDocument,x=t?t.nodeType:9;if(r=r||[],"string"!=typeof e||!e||1!==x&&9!==x&&11!==x)return r;if(!i&&((t?t.ownerDocument||t:w)!==p&&h(t),t=t||p,g)){if(11!==x&&(f=Q.exec(e)))if(o=f[1]){if(9===x){if(!(l=t.getElementById(o)))return r;if(l.id===o)return r.push(l),r}else if(y&&(l=y.getElementById(o))&&_(t,l)&&l.id===o)return r.push(l),r}else{if(f[2])return j.apply(r,t.getElementsByTagName(e)),r;if((o=f[3])&&n.getElementsByClassName&&t.getElementsByClassName)return j.apply(r,t.getElementsByClassName(o)),r}if(n.qsa&&!A[e+" "]&&(!v||!v.test(e))&&(1!==x||"object"!==t.nodeName.toLowerCase())){if(m=e,y=t,1===x&&$.test(e)){for((c=t.getAttribute("id"))?c=c.replace(re,ie):t.setAttribute("id",c=b),a=(d=s(e)).length;a--;)d[a]="#"+c+" "+_e(d[a]);m=d.join(","),y=ee.test(e)&&me(t.parentNode)||t}try{return j.apply(r,y.querySelectorAll(m)),r}catch(t){A(e,!0)}finally{c===b&&t.removeAttribute("id")}}}return u(e.replace(W,"$1"),t,r,i)}function ue(){var e=[];return function t(n,i){return e.push(n+" ")>r.cacheLength&&delete t[e.shift()],t[n+" "]=i}}function le(e){return e[b]=!0,e}function ce(e){var t=p.createElement("fieldset");try{return!!e(t)}catch(e){return!1}finally{t.parentNode&&t.parentNode.removeChild(t),t=null}}function fe(e,t){for(var n=e.split("|"),i=n.length;i--;)r.attrHandle[n[i]]=t}function he(e,t){var n=t&&e,r=n&&1===e.nodeType&&1===t.nodeType&&e.sourceIndex-t.sourceIndex;if(r)return r;if(n)for(;n=n.nextSibling;)if(n===t)return-1;return e?1:-1}function pe(e){return function(t){return"input"===t.nodeName.toLowerCase()&&t.type===e}}function de(e){return function(t){var n=t.nodeName.toLowerCase();return("input"===n||"button"===n)&&t.type===e}}function ge(e){return function(t){return"form"in t?t.parentNode&&!1===t.disabled?"label"in t?"label"in t.parentNode?t.parentNode.disabled===e:t.disabled===e:t.isDisabled===e||t.isDisabled!==!e&&se(t)===e:t.disabled===e:"label"in t&&t.disabled===e}}function ve(e){return le((function(t){return t=+t,le((function(n,r){for(var i,o=e([],n.length,t),s=o.length;s--;)n[i=o[s]]&&(n[i]=!(r[i]=n[i]))}))}))}function me(e){return e&&void 0!==e.getElementsByTagName&&e}for(t in n=ae.support={},o=ae.isXML=function(e){var t=e.namespaceURI,n=(e.ownerDocument||e).documentElement;return!K.test(t||n&&n.nodeName||"HTML")},h=ae.setDocument=function(e){var t,i,s=e?e.ownerDocument||e:w;return s!==p&&9===s.nodeType&&s.documentElement?(d=(p=s).documentElement,g=!o(p),w!==p&&(i=p.defaultView)&&i.top!==i&&(i.addEventListener?i.addEventListener("unload",oe,!1):i.attachEvent&&i.attachEvent("onunload",oe)),n.attributes=ce((function(e){return e.className="i",!e.getAttribute("className")})),n.getElementsByTagName=ce((function(e){return e.appendChild(p.createComment("")),!e.getElementsByTagName("*").length})),n.getElementsByClassName=J.test(p.getElementsByClassName),n.getById=ce((function(e){return d.appendChild(e).id=b,!p.getElementsByName||!p.getElementsByName(b).length})),n.getById?(r.filter.ID=function(e){var t=e.replace(te,ne);return function(e){return e.getAttribute("id")===t}},r.find.ID=function(e,t){if(void 0!==t.getElementById&&g){var n=t.getElementById(e);return n?[n]:[]}}):(r.filter.ID=function(e){var t=e.replace(te,ne);return function(e){var n=void 0!==e.getAttributeNode&&e.getAttributeNode("id");return n&&n.value===t}},r.find.ID=function(e,t){if(void 0!==t.getElementById&&g){var n,r,i,o=t.getElementById(e);if(o){if((n=o.getAttributeNode("id"))&&n.value===e)return[o];for(i=t.getElementsByName(e),r=0;o=i[r++];)if((n=o.getAttributeNode("id"))&&n.value===e)return[o]}return[]}}),r.find.TAG=n.getElementsByTagName?function(e,t){return void 0!==t.getElementsByTagName?t.getElementsByTagName(e):n.qsa?t.querySelectorAll(e):void 0}:function(e,t){var n,r=[],i=0,o=t.getElementsByTagName(e);if("*"===e){for(;n=o[i++];)1===n.nodeType&&r.push(n);return r}return o},r.find.CLASS=n.getElementsByClassName&&function(e,t){if(void 0!==t.getElementsByClassName&&g)return t.getElementsByClassName(e)},m=[],v=[],(n.qsa=J.test(p.querySelectorAll))&&(ce((function(e){d.appendChild(e).innerHTML="<a id='"+b+"'></a><select id='"+b+"-\r\\' msallowcapture=''><option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&v.push("[*^$]="+P+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||v.push("\\["+P+"*(?:value|"+L+")"),e.querySelectorAll("[id~="+b+"-]").length||v.push("~="),e.querySelectorAll(":checked").length||v.push(":checked"),e.querySelectorAll("a#"+b+"+*").length||v.push(".#.+[+~]")})),ce((function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var t=p.createElement("input");t.setAttribute("type","hidden"),e.appendChild(t).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&v.push("name"+P+"*[*^$|!~]?="),2!==e.querySelectorAll(":enabled").length&&v.push(":enabled",":disabled"),d.appendChild(e).disabled=!0,2!==e.querySelectorAll(":disabled").length&&v.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),v.push(",.*:")}))),(n.matchesSelector=J.test(y=d.matches||d.webkitMatchesSelector||d.mozMatchesSelector||d.oMatchesSelector||d.msMatchesSelector))&&ce((function(e){n.disconnectedMatch=y.call(e,"*"),y.call(e,"[s!='']:x"),m.push("!=",I)})),v=v.length&&new RegExp(v.join("|")),m=m.length&&new RegExp(m.join("|")),t=J.test(d.compareDocumentPosition),_=t||J.test(d.contains)?function(e,t){var n=9===e.nodeType?e.documentElement:e,r=t&&t.parentNode;return e===r||!(!r||1!==r.nodeType||!(n.contains?n.contains(r):e.compareDocumentPosition&&16&e.compareDocumentPosition(r)))}:function(e,t){if(t)for(;t=t.parentNode;)if(t===e)return!0;return!1},D=t?function(e,t){if(e===t)return f=!0,0;var r=!e.compareDocumentPosition-!t.compareDocumentPosition;return r||(1&(r=(e.ownerDocument||e)===(t.ownerDocument||t)?e.compareDocumentPosition(t):1)||!n.sortDetached&&t.compareDocumentPosition(e)===r?e===p||e.ownerDocument===w&&_(w,e)?-1:t===p||t.ownerDocument===w&&_(w,t)?1:c?F(c,e)-F(c,t):0:4&r?-1:1)}:function(e,t){if(e===t)return f=!0,0;var n,r=0,i=e.parentNode,o=t.parentNode,s=[e],a=[t];if(!i||!o)return e===p?-1:t===p?1:i?-1:o?1:c?F(c,e)-F(c,t):0;if(i===o)return he(e,t);for(n=e;n=n.parentNode;)s.unshift(n);for(n=t;n=n.parentNode;)a.unshift(n);for(;s[r]===a[r];)r++;return r?he(s[r],a[r]):s[r]===w?-1:a[r]===w?1:0},p):p},ae.matches=function(e,t){return ae(e,null,null,t)},ae.matchesSelector=function(e,t){if((e.ownerDocument||e)!==p&&h(e),n.matchesSelector&&g&&!A[t+" "]&&(!m||!m.test(t))&&(!v||!v.test(t)))try{var r=y.call(e,t);if(r||n.disconnectedMatch||e.document&&11!==e.document.nodeType)return r}catch(e){A(t,!0)}return ae(t,p,null,[e]).length>0},ae.contains=function(e,t){return(e.ownerDocument||e)!==p&&h(e),_(e,t)},ae.attr=function(e,t){(e.ownerDocument||e)!==p&&h(e);var i=r.attrHandle[t.toLowerCase()],o=i&&z.call(r.attrHandle,t.toLowerCase())?i(e,t,!g):void 0;return void 0!==o?o:n.attributes||!g?e.getAttribute(t):(o=e.getAttributeNode(t))&&o.specified?o.value:null},ae.escape=function(e){return(e+"").replace(re,ie)},ae.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},ae.uniqueSort=function(e){var t,r=[],i=0,o=0;if(f=!n.detectDuplicates,c=!n.sortStable&&e.slice(0),e.sort(D),f){for(;t=e[o++];)t===e[o]&&(i=r.push(o));for(;i--;)e.splice(r[i],1)}return c=null,e},i=ae.getText=function(e){var t,n="",r=0,o=e.nodeType;if(o){if(1===o||9===o||11===o){if("string"==typeof e.textContent)return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)n+=i(e)}else if(3===o||4===o)return e.nodeValue}else for(;t=e[r++];)n+=i(t);return n},(r=ae.selectors={cacheLength:50,createPseudo:le,match:G,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(te,ne),e[3]=(e[3]||e[4]||e[5]||"").replace(te,ne),"~="===e[2]&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),"nth"===e[1].slice(0,3)?(e[3]||ae.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*("even"===e[3]||"odd"===e[3])),e[5]=+(e[7]+e[8]||"odd"===e[3])):e[3]&&ae.error(e[0]),e},PSEUDO:function(e){var t,n=!e[6]&&e[2];return G.CHILD.test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":n&&X.test(n)&&(t=s(n,!0))&&(t=n.indexOf(")",n.length-t)-n.length)&&(e[0]=e[0].slice(0,t),e[2]=n.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(te,ne).toLowerCase();return"*"===e?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=T[e+" "];return t||(t=new RegExp("(^|"+P+")"+e+"("+P+"|$)"))&&T(e,(function(e){return t.test("string"==typeof e.className&&e.className||void 0!==e.getAttribute&&e.getAttribute("class")||"")}))},ATTR:function(e,t,n){return function(r){var i=ae.attr(r,e);return null==i?"!="===t:!t||(i+="","="===t?i===n:"!="===t?i!==n:"^="===t?n&&0===i.indexOf(n):"*="===t?n&&i.indexOf(n)>-1:"$="===t?n&&i.slice(-n.length)===n:"~="===t?(" "+i.replace(B," ")+" ").indexOf(n)>-1:"|="===t&&(i===n||i.slice(0,n.length+1)===n+"-"))}},CHILD:function(e,t,n,r,i){var o="nth"!==e.slice(0,3),s="last"!==e.slice(-4),a="of-type"===t;return 1===r&&0===i?function(e){return!!e.parentNode}:function(t,n,u){var l,c,f,h,p,d,g=o!==s?"nextSibling":"previousSibling",v=t.parentNode,m=a&&t.nodeName.toLowerCase(),y=!u&&!a,_=!1;if(v){if(o){for(;g;){for(h=t;h=h[g];)if(a?h.nodeName.toLowerCase()===m:1===h.nodeType)return!1;d=g="only"===e&&!d&&"nextSibling"}return!0}if(d=[s?v.firstChild:v.lastChild],s&&y){for(_=(p=(l=(c=(f=(h=v)[b]||(h[b]={}))[h.uniqueID]||(f[h.uniqueID]={}))[e]||[])[0]===x&&l[1])&&l[2],h=p&&v.childNodes[p];h=++p&&h&&h[g]||(_=p=0)||d.pop();)if(1===h.nodeType&&++_&&h===t){c[e]=[x,p,_];break}}else if(y&&(_=p=(l=(c=(f=(h=t)[b]||(h[b]={}))[h.uniqueID]||(f[h.uniqueID]={}))[e]||[])[0]===x&&l[1]),!1===_)for(;(h=++p&&h&&h[g]||(_=p=0)||d.pop())&&((a?h.nodeName.toLowerCase()!==m:1!==h.nodeType)||!++_||(y&&((c=(f=h[b]||(h[b]={}))[h.uniqueID]||(f[h.uniqueID]={}))[e]=[x,_]),h!==t)););return(_-=i)===r||_%r==0&&_/r>=0}}},PSEUDO:function(e,t){var n,i=r.pseudos[e]||r.setFilters[e.toLowerCase()]||ae.error("unsupported pseudo: "+e);return i[b]?i(t):i.length>1?(n=[e,e,"",t],r.setFilters.hasOwnProperty(e.toLowerCase())?le((function(e,n){for(var r,o=i(e,t),s=o.length;s--;)e[r=F(e,o[s])]=!(n[r]=o[s])})):function(e){return i(e,0,n)}):i}},pseudos:{not:le((function(e){var t=[],n=[],r=a(e.replace(W,"$1"));return r[b]?le((function(e,t,n,i){for(var o,s=r(e,null,i,[]),a=e.length;a--;)(o=s[a])&&(e[a]=!(t[a]=o))})):function(e,i,o){return t[0]=e,r(t,null,o,n),t[0]=null,!n.pop()}})),has:le((function(e){return function(t){return ae(e,t).length>0}})),contains:le((function(e){return e=e.replace(te,ne),function(t){return(t.textContent||i(t)).indexOf(e)>-1}})),lang:le((function(e){return V.test(e||"")||ae.error("unsupported lang: "+e),e=e.replace(te,ne).toLowerCase(),function(t){var n;do{if(n=g?t.lang:t.getAttribute("xml:lang")||t.getAttribute("lang"))return(n=n.toLowerCase())===e||0===n.indexOf(e+"-")}while((t=t.parentNode)&&1===t.nodeType);return!1}})),target:function(t){var n=e.location&&e.location.hash;return n&&n.slice(1)===t.id},root:function(e){return e===d},focus:function(e){return e===p.activeElement&&(!p.hasFocus||p.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:ge(!1),disabled:ge(!0),checked:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&!!e.checked||"option"===t&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,!0===e.selected},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(e){return!r.pseudos.empty(e)},header:function(e){return Z.test(e.nodeName)},input:function(e){return Y.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&"button"===e.type||"button"===t},text:function(e){var t;return"input"===e.nodeName.toLowerCase()&&"text"===e.type&&(null==(t=e.getAttribute("type"))||"text"===t.toLowerCase())},first:ve((function(){return[0]})),last:ve((function(e,t){return[t-1]})),eq:ve((function(e,t,n){return[n<0?n+t:n]})),even:ve((function(e,t){for(var n=0;n<t;n+=2)e.push(n);return e})),odd:ve((function(e,t){for(var n=1;n<t;n+=2)e.push(n);return e})),lt:ve((function(e,t,n){for(var r=n<0?n+t:n>t?t:n;--r>=0;)e.push(r);return e})),gt:ve((function(e,t,n){for(var r=n<0?n+t:n;++r<t;)e.push(r);return e}))}}).pseudos.nth=r.pseudos.eq,{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})r.pseudos[t]=pe(t);for(t in{submit:!0,reset:!0})r.pseudos[t]=de(t);function ye(){}function _e(e){for(var t=0,n=e.length,r="";t<n;t++)r+=e[t].value;return r}function be(e,t,n){var r=t.dir,i=t.next,o=i||r,s=n&&"parentNode"===o,a=E++;return t.first?function(t,n,i){for(;t=t[r];)if(1===t.nodeType||s)return e(t,n,i);return!1}:function(t,n,u){var l,c,f,h=[x,a];if(u){for(;t=t[r];)if((1===t.nodeType||s)&&e(t,n,u))return!0}else for(;t=t[r];)if(1===t.nodeType||s)if(c=(f=t[b]||(t[b]={}))[t.uniqueID]||(f[t.uniqueID]={}),i&&i===t.nodeName.toLowerCase())t=t[r]||t;else{if((l=c[o])&&l[0]===x&&l[1]===a)return h[2]=l[2];if(c[o]=h,h[2]=e(t,n,u))return!0}return!1}}function we(e){return e.length>1?function(t,n,r){for(var i=e.length;i--;)if(!e[i](t,n,r))return!1;return!0}:e[0]}function xe(e,t,n,r,i){for(var o,s=[],a=0,u=e.length,l=null!=t;a<u;a++)(o=e[a])&&(n&&!n(o,r,i)||(s.push(o),l&&t.push(a)));return s}function Ee(e,t,n,r,i,o){return r&&!r[b]&&(r=Ee(r)),i&&!i[b]&&(i=Ee(i,o)),le((function(o,s,a,u){var l,c,f,h=[],p=[],d=s.length,g=o||function(e,t,n){for(var r=0,i=t.length;r<i;r++)ae(e,t[r],n);return n}(t||"*",a.nodeType?[a]:a,[]),v=!e||!o&&t?g:xe(g,h,e,a,u),m=n?i||(o?e:d||r)?[]:s:v;if(n&&n(v,m,a,u),r)for(l=xe(m,p),r(l,[],a,u),c=l.length;c--;)(f=l[c])&&(m[p[c]]=!(v[p[c]]=f));if(o){if(i||e){if(i){for(l=[],c=m.length;c--;)(f=m[c])&&l.push(v[c]=f);i(null,m=[],l,u)}for(c=m.length;c--;)(f=m[c])&&(l=i?F(o,f):h[c])>-1&&(o[l]=!(s[l]=f))}}else m=xe(m===s?m.splice(d,m.length):m),i?i(null,s,m,u):j.apply(s,m)}))}function Te(e){for(var t,n,i,o=e.length,s=r.relative[e[0].type],a=s||r.relative[" "],u=s?1:0,c=be((function(e){return e===t}),a,!0),f=be((function(e){return F(t,e)>-1}),a,!0),h=[function(e,n,r){var i=!s&&(r||n!==l)||((t=n).nodeType?c(e,n,r):f(e,n,r));return t=null,i}];u<o;u++)if(n=r.relative[e[u].type])h=[be(we(h),n)];else{if((n=r.filter[e[u].type].apply(null,e[u].matches))[b]){for(i=++u;i<o&&!r.relative[e[i].type];i++);return Ee(u>1&&we(h),u>1&&_e(e.slice(0,u-1).concat({value:" "===e[u-2].type?"*":""})).replace(W,"$1"),n,u<i&&Te(e.slice(u,i)),i<o&&Te(e=e.slice(i)),i<o&&_e(e))}h.push(n)}return we(h)}return ye.prototype=r.filters=r.pseudos,r.setFilters=new ye,s=ae.tokenize=function(e,t){var n,i,o,s,a,u,l,c=C[e+" "];if(c)return t?0:c.slice(0);for(a=e,u=[],l=r.preFilter;a;){for(s in n&&!(i=H.exec(a))||(i&&(a=a.slice(i[0].length)||a),u.push(o=[])),n=!1,(i=U.exec(a))&&(n=i.shift(),o.push({value:n,type:i[0].replace(W," ")}),a=a.slice(n.length)),r.filter)!(i=G[s].exec(a))||l[s]&&!(i=l[s](i))||(n=i.shift(),o.push({value:n,type:s,matches:i}),a=a.slice(n.length));if(!n)break}return t?a.length:a?ae.error(e):C(e,u).slice(0)},a=ae.compile=function(e,t){var n,i=[],o=[],a=S[e+" "];if(!a){for(t||(t=s(e)),n=t.length;n--;)(a=Te(t[n]))[b]?i.push(a):o.push(a);(a=S(e,function(e,t){var n=t.length>0,i=e.length>0,o=function(o,s,a,u,c){var f,d,v,m=0,y="0",_=o&&[],b=[],w=l,E=o||i&&r.find.TAG("*",c),T=x+=null==w?1:Math.random()||.1,C=E.length;for(c&&(l=s===p||s||c);y!==C&&null!=(f=E[y]);y++){if(i&&f){for(d=0,s||f.ownerDocument===p||(h(f),a=!g);v=e[d++];)if(v(f,s||p,a)){u.push(f);break}c&&(x=T)}n&&((f=!v&&f)&&m--,o&&_.push(f))}if(m+=y,n&&y!==m){for(d=0;v=t[d++];)v(_,b,s,a);if(o){if(m>0)for(;y--;)_[y]||b[y]||(b[y]=R.call(u));b=xe(b)}j.apply(u,b),c&&!o&&b.length>0&&m+t.length>1&&ae.uniqueSort(u)}return c&&(x=T,l=w),_};return n?le(o):o}(o,i))).selector=e}return a},u=ae.select=function(e,t,n,i){var o,u,l,c,f,h="function"==typeof e&&e,p=!i&&s(e=h.selector||e);if(n=n||[],1===p.length){if((u=p[0]=p[0].slice(0)).length>2&&"ID"===(l=u[0]).type&&9===t.nodeType&&g&&r.relative[u[1].type]){if(!(t=(r.find.ID(l.matches[0].replace(te,ne),t)||[])[0]))return n;h&&(t=t.parentNode),e=e.slice(u.shift().value.length)}for(o=G.needsContext.test(e)?0:u.length;o--&&(l=u[o],!r.relative[c=l.type]);)if((f=r.find[c])&&(i=f(l.matches[0].replace(te,ne),ee.test(u[0].type)&&me(t.parentNode)||t))){if(u.splice(o,1),!(e=i.length&&_e(u)))return j.apply(n,i),n;break}}return(h||a(e,p))(i,t,!g,n,!t||ee.test(e)&&me(t.parentNode)||t),n},n.sortStable=b.split("").sort(D).join("")===b,n.detectDuplicates=!!f,h(),n.sortDetached=ce((function(e){return 1&e.compareDocumentPosition(p.createElement("fieldset"))})),ce((function(e){return e.innerHTML="<a href='#'></a>","#"===e.firstChild.getAttribute("href")}))||fe("type|href|height|width",(function(e,t,n){if(!n)return e.getAttribute(t,"type"===t.toLowerCase()?1:2)})),n.attributes&&ce((function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),""===e.firstChild.getAttribute("value")}))||fe("value",(function(e,t,n){if(!n&&"input"===e.nodeName.toLowerCase())return e.defaultValue})),ce((function(e){return null==e.getAttribute("disabled")}))||fe(L,(function(e,t,n){var r;if(!n)return!0===e[t]?t.toLowerCase():(r=e.getAttributeNode(t))&&r.specified?r.value:null})),ae}(n);E.find=S,E.expr=S.selectors,E.expr[":"]=E.expr.pseudos,E.uniqueSort=E.unique=S.uniqueSort,E.text=S.getText,E.isXMLDoc=S.isXML,E.contains=S.contains,E.escapeSelector=S.escape;var A=function(e,t,n){for(var r=[],i=void 0!==n;(e=e[t])&&9!==e.nodeType;)if(1===e.nodeType){if(i&&E(e).is(n))break;r.push(e)}return r},D=function(e,t){for(var n=[];e;e=e.nextSibling)1===e.nodeType&&e!==t&&n.push(e);return n},z=E.expr.match.needsContext;function k(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()}var R=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function N(e,t,n){return y(t)?E.grep(e,(function(e,r){return!!t.call(e,r,e)!==n})):t.nodeType?E.grep(e,(function(e){return e===t!==n})):"string"!=typeof t?E.grep(e,(function(e){return f.call(t,e)>-1!==n})):E.filter(t,e,n)}E.filter=function(e,t,n){var r=t[0];return n&&(e=":not("+e+")"),1===t.length&&1===r.nodeType?E.find.matchesSelector(r,e)?[r]:[]:E.find.matches(e,E.grep(t,(function(e){return 1===e.nodeType})))},E.fn.extend({find:function(e){var t,n,r=this.length,i=this;if("string"!=typeof e)return this.pushStack(E(e).filter((function(){for(t=0;t<r;t++)if(E.contains(i[t],this))return!0})));for(n=this.pushStack([]),t=0;t<r;t++)E.find(e,i[t],n);return r>1?E.uniqueSort(n):n},filter:function(e){return this.pushStack(N(this,e||[],!1))},not:function(e){return this.pushStack(N(this,e||[],!0))},is:function(e){return!!N(this,"string"==typeof e&&z.test(e)?E(e):e||[],!1).length}});var j,O=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/;(E.fn.init=function(e,t,n){var r,i;if(!e)return this;if(n=n||j,"string"==typeof e){if(!(r="<"===e[0]&&">"===e[e.length-1]&&e.length>=3?[null,e,null]:O.exec(e))||!r[1]&&t)return!t||t.jquery?(t||n).find(e):this.constructor(t).find(e);if(r[1]){if(t=t instanceof E?t[0]:t,E.merge(this,E.parseHTML(r[1],t&&t.nodeType?t.ownerDocument||t:s,!0)),R.test(r[1])&&E.isPlainObject(t))for(r in t)y(this[r])?this[r](t[r]):this.attr(r,t[r]);return this}return(i=s.getElementById(r[2]))&&(this[0]=i,this.length=1),this}return e.nodeType?(this[0]=e,this.length=1,this):y(e)?void 0!==n.ready?n.ready(e):e(E):E.makeArray(e,this)}).prototype=E.fn,j=E(s);var F=/^(?:parents|prev(?:Until|All))/,L={children:!0,contents:!0,next:!0,prev:!0};function P(e,t){for(;(e=e[t])&&1!==e.nodeType;);return e}E.fn.extend({has:function(e){var t=E(e,this),n=t.length;return this.filter((function(){for(var e=0;e<n;e++)if(E.contains(this,t[e]))return!0}))},closest:function(e,t){var n,r=0,i=this.length,o=[],s="string"!=typeof e&&E(e);if(!z.test(e))for(;r<i;r++)for(n=this[r];n&&n!==t;n=n.parentNode)if(n.nodeType<11&&(s?s.index(n)>-1:1===n.nodeType&&E.find.matchesSelector(n,e))){o.push(n);break}return this.pushStack(o.length>1?E.uniqueSort(o):o)},index:function(e){return e?"string"==typeof e?f.call(E(e),this[0]):f.call(this,e.jquery?e[0]:e):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){return this.pushStack(E.uniqueSort(E.merge(this.get(),E(e,t))))},addBack:function(e){return this.add(null==e?this.prevObject:this.prevObject.filter(e))}}),E.each({parent:function(e){var t=e.parentNode;return t&&11!==t.nodeType?t:null},parents:function(e){return A(e,"parentNode")},parentsUntil:function(e,t,n){return A(e,"parentNode",n)},next:function(e){return P(e,"nextSibling")},prev:function(e){return P(e,"previousSibling")},nextAll:function(e){return A(e,"nextSibling")},prevAll:function(e){return A(e,"previousSibling")},nextUntil:function(e,t,n){return A(e,"nextSibling",n)},prevUntil:function(e,t,n){return A(e,"previousSibling",n)},siblings:function(e){return D((e.parentNode||{}).firstChild,e)},children:function(e){return D(e.firstChild)},contents:function(e){return void 0!==e.contentDocument?e.contentDocument:(k(e,"template")&&(e=e.content||e),E.merge([],e.childNodes))}},(function(e,t){E.fn[e]=function(n,r){var i=E.map(this,t,n);return"Until"!==e.slice(-5)&&(r=n),r&&"string"==typeof r&&(i=E.filter(r,i)),this.length>1&&(L[e]||E.uniqueSort(i),F.test(e)&&i.reverse()),this.pushStack(i)}}));var q=/[^\x20\t\r\n\f]+/g;function M(e){return e}function I(e){throw e}function B(e,t,n,r){var i;try{e&&y(i=e.promise)?i.call(e).done(t).fail(n):e&&y(i=e.then)?i.call(e,t,n):t.apply(void 0,[e].slice(r))}catch(e){n.apply(void 0,[e])}}E.Callbacks=function(e){e="string"==typeof e?function(e){var t={};return E.each(e.match(q)||[],(function(e,n){t[n]=!0})),t}(e):E.extend({},e);var t,n,r,i,o=[],s=[],a=-1,u=function(){for(i=i||e.once,r=t=!0;s.length;a=-1)for(n=s.shift();++a<o.length;)!1===o[a].apply(n[0],n[1])&&e.stopOnFalse&&(a=o.length,n=!1);e.memory||(n=!1),t=!1,i&&(o=n?[]:"")},l={add:function(){return o&&(n&&!t&&(a=o.length-1,s.push(n)),function t(n){E.each(n,(function(n,r){y(r)?e.unique&&l.has(r)||o.push(r):r&&r.length&&"string"!==x(r)&&t(r)}))}(arguments),n&&!t&&u()),this},remove:function(){return E.each(arguments,(function(e,t){for(var n;(n=E.inArray(t,o,n))>-1;)o.splice(n,1),n<=a&&a--})),this},has:function(e){return e?E.inArray(e,o)>-1:o.length>0},empty:function(){return o&&(o=[]),this},disable:function(){return i=s=[],o=n="",this},disabled:function(){return!o},lock:function(){return i=s=[],n||t||(o=n=""),this},locked:function(){return!!i},fireWith:function(e,n){return i||(n=[e,(n=n||[]).slice?n.slice():n],s.push(n),t||u()),this},fire:function(){return l.fireWith(this,arguments),this},fired:function(){return!!r}};return l},E.extend({Deferred:function(e){var t=[["notify","progress",E.Callbacks("memory"),E.Callbacks("memory"),2],["resolve","done",E.Callbacks("once memory"),E.Callbacks("once memory"),0,"resolved"],["reject","fail",E.Callbacks("once memory"),E.Callbacks("once memory"),1,"rejected"]],r="pending",i={state:function(){return r},always:function(){return o.done(arguments).fail(arguments),this},catch:function(e){return i.then(null,e)},pipe:function(){var e=arguments;return E.Deferred((function(n){E.each(t,(function(t,r){var i=y(e[r[4]])&&e[r[4]];o[r[1]]((function(){var e=i&&i.apply(this,arguments);e&&y(e.promise)?e.promise().progress(n.notify).done(n.resolve).fail(n.reject):n[r[0]+"With"](this,i?[e]:arguments)}))})),e=null})).promise()},then:function(e,r,i){var o=0;function s(e,t,r,i){return function(){var a=this,u=arguments,l=function(){var n,l;if(!(e<o)){if((n=r.apply(a,u))===t.promise())throw new TypeError("Thenable self-resolution");l=n&&("object"==typeof n||"function"==typeof n)&&n.then,y(l)?i?l.call(n,s(o,t,M,i),s(o,t,I,i)):(o++,l.call(n,s(o,t,M,i),s(o,t,I,i),s(o,t,M,t.notifyWith))):(r!==M&&(a=void 0,u=[n]),(i||t.resolveWith)(a,u))}},c=i?l:function(){try{l()}catch(n){E.Deferred.exceptionHook&&E.Deferred.exceptionHook(n,c.stackTrace),e+1>=o&&(r!==I&&(a=void 0,u=[n]),t.rejectWith(a,u))}};e?c():(E.Deferred.getStackHook&&(c.stackTrace=E.Deferred.getStackHook()),n.setTimeout(c))}}return E.Deferred((function(n){t[0][3].add(s(0,n,y(i)?i:M,n.notifyWith)),t[1][3].add(s(0,n,y(e)?e:M)),t[2][3].add(s(0,n,y(r)?r:I))})).promise()},promise:function(e){return null!=e?E.extend(e,i):i}},o={};return E.each(t,(function(e,n){var s=n[2],a=n[5];i[n[1]]=s.add,a&&s.add((function(){r=a}),t[3-e][2].disable,t[3-e][3].disable,t[0][2].lock,t[0][3].lock),s.add(n[3].fire),o[n[0]]=function(){return o[n[0]+"With"](this===o?void 0:this,arguments),this},o[n[0]+"With"]=s.fireWith})),i.promise(o),e&&e.call(o,o),o},when:function(e){var t=arguments.length,n=t,r=Array(n),i=u.call(arguments),o=E.Deferred(),s=function(e){return function(n){r[e]=this,i[e]=arguments.length>1?u.call(arguments):n,--t||o.resolveWith(r,i)}};if(t<=1&&(B(e,o.done(s(n)).resolve,o.reject,!t),"pending"===o.state()||y(i[n]&&i[n].then)))return o.then();for(;n--;)B(i[n],s(n),o.reject);return o.promise()}});var W=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;E.Deferred.exceptionHook=function(e,t){n.console&&n.console.warn&&e&&W.test(e.name)&&n.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},E.readyException=function(e){n.setTimeout((function(){throw e}))};var H=E.Deferred();function U(){s.removeEventListener("DOMContentLoaded",U),n.removeEventListener("load",U),E.ready()}E.fn.ready=function(e){return H.then(e).catch((function(e){E.readyException(e)})),this},E.extend({isReady:!1,readyWait:1,ready:function(e){(!0===e?--E.readyWait:E.isReady)||(E.isReady=!0,!0!==e&&--E.readyWait>0||H.resolveWith(s,[E]))}}),E.ready.then=H.then,"complete"===s.readyState||"loading"!==s.readyState&&!s.documentElement.doScroll?n.setTimeout(E.ready):(s.addEventListener("DOMContentLoaded",U),n.addEventListener("load",U));var $=function(e,t,n,r,i,o,s){var a=0,u=e.length,l=null==n;if("object"===x(n))for(a in i=!0,n)$(e,t,a,n[a],!0,o,s);else if(void 0!==r&&(i=!0,y(r)||(s=!0),l&&(s?(t.call(e,r),t=null):(l=t,t=function(e,t,n){return l.call(E(e),n)})),t))for(;a<u;a++)t(e[a],n,s?r:r.call(e[a],a,t(e[a],n)));return i?e:l?t.call(e):u?t(e[0],n):o},X=/^-ms-/,V=/-([a-z])/g;function G(e,t){return t.toUpperCase()}function K(e){return e.replace(X,"ms-").replace(V,G)}var Y=function(e){return 1===e.nodeType||9===e.nodeType||!+e.nodeType};function Z(){this.expando=E.expando+Z.uid++}Z.uid=1,Z.prototype={cache:function(e){var t=e[this.expando];return t||(t={},Y(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(e,t,n){var r,i=this.cache(e);if("string"==typeof t)i[K(t)]=n;else for(r in t)i[K(r)]=t[r];return i},get:function(e,t){return void 0===t?this.cache(e):e[this.expando]&&e[this.expando][K(t)]},access:function(e,t,n){return void 0===t||t&&"string"==typeof t&&void 0===n?this.get(e,t):(this.set(e,t,n),void 0!==n?n:t)},remove:function(e,t){var n,r=e[this.expando];if(void 0!==r){if(void 0!==t){n=(t=Array.isArray(t)?t.map(K):(t=K(t))in r?[t]:t.match(q)||[]).length;for(;n--;)delete r[t[n]]}(void 0===t||E.isEmptyObject(r))&&(e.nodeType?e[this.expando]=void 0:delete e[this.expando])}},hasData:function(e){var t=e[this.expando];return void 0!==t&&!E.isEmptyObject(t)}};var J=new Z,Q=new Z,ee=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,te=/[A-Z]/g;function ne(e,t,n){var r;if(void 0===n&&1===e.nodeType)if(r="data-"+t.replace(te,"-$&").toLowerCase(),"string"==typeof(n=e.getAttribute(r))){try{n=function(e){return"true"===e||"false"!==e&&("null"===e?null:e===+e+""?+e:ee.test(e)?JSON.parse(e):e)}(n)}catch(e){}Q.set(e,t,n)}else n=void 0;return n}E.extend({hasData:function(e){return Q.hasData(e)||J.hasData(e)},data:function(e,t,n){return Q.access(e,t,n)},removeData:function(e,t){Q.remove(e,t)},_data:function(e,t,n){return J.access(e,t,n)},_removeData:function(e,t){J.remove(e,t)}}),E.fn.extend({data:function(e,t){var n,r,i,o=this[0],s=o&&o.attributes;if(void 0===e){if(this.length&&(i=Q.get(o),1===o.nodeType&&!J.get(o,"hasDataAttrs"))){for(n=s.length;n--;)s[n]&&0===(r=s[n].name).indexOf("data-")&&(r=K(r.slice(5)),ne(o,r,i[r]));J.set(o,"hasDataAttrs",!0)}return i}return"object"==typeof e?this.each((function(){Q.set(this,e)})):$(this,(function(t){var n;if(o&&void 0===t)return void 0!==(n=Q.get(o,e))?n:void 0!==(n=ne(o,e))?n:void 0;this.each((function(){Q.set(this,e,t)}))}),null,t,arguments.length>1,null,!0)},removeData:function(e){return this.each((function(){Q.remove(this,e)}))}}),E.extend({queue:function(e,t,n){var r;if(e)return t=(t||"fx")+"queue",r=J.get(e,t),n&&(!r||Array.isArray(n)?r=J.access(e,t,E.makeArray(n)):r.push(n)),r||[]},dequeue:function(e,t){t=t||"fx";var n=E.queue(e,t),r=n.length,i=n.shift(),o=E._queueHooks(e,t);"inprogress"===i&&(i=n.shift(),r--),i&&("fx"===t&&n.unshift("inprogress"),delete o.stop,i.call(e,(function(){E.dequeue(e,t)}),o)),!r&&o&&o.empty.fire()},_queueHooks:function(e,t){var n=t+"queueHooks";return J.get(e,n)||J.access(e,n,{empty:E.Callbacks("once memory").add((function(){J.remove(e,[t+"queue",n])}))})}}),E.fn.extend({queue:function(e,t){var n=2;return"string"!=typeof e&&(t=e,e="fx",n--),arguments.length<n?E.queue(this[0],e):void 0===t?this:this.each((function(){var n=E.queue(this,e,t);E._queueHooks(this,e),"fx"===e&&"inprogress"!==n[0]&&E.dequeue(this,e)}))},dequeue:function(e){return this.each((function(){E.dequeue(this,e)}))},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var n,r=1,i=E.Deferred(),o=this,s=this.length,a=function(){--r||i.resolveWith(o,[o])};for("string"!=typeof e&&(t=e,e=void 0),e=e||"fx";s--;)(n=J.get(o[s],e+"queueHooks"))&&n.empty&&(r++,n.empty.add(a));return a(),i.promise(t)}});var re=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,ie=new RegExp("^(?:([+-])=|)("+re+")([a-z%]*)$","i"),oe=["Top","Right","Bottom","Left"],se=s.documentElement,ae=function(e){return E.contains(e.ownerDocument,e)},ue={composed:!0};se.getRootNode&&(ae=function(e){return E.contains(e.ownerDocument,e)||e.getRootNode(ue)===e.ownerDocument});var le=function(e,t){return"none"===(e=t||e).style.display||""===e.style.display&&ae(e)&&"none"===E.css(e,"display")},ce=function(e,t,n,r){var i,o,s={};for(o in t)s[o]=e.style[o],e.style[o]=t[o];for(o in i=n.apply(e,r||[]),t)e.style[o]=s[o];return i};function fe(e,t,n,r){var i,o,s=20,a=r?function(){return r.cur()}:function(){return E.css(e,t,"")},u=a(),l=n&&n[3]||(E.cssNumber[t]?"":"px"),c=e.nodeType&&(E.cssNumber[t]||"px"!==l&&+u)&&ie.exec(E.css(e,t));if(c&&c[3]!==l){for(u/=2,l=l||c[3],c=+u||1;s--;)E.style(e,t,c+l),(1-o)*(1-(o=a()/u||.5))<=0&&(s=0),c/=o;c*=2,E.style(e,t,c+l),n=n||[]}return n&&(c=+c||+u||0,i=n[1]?c+(n[1]+1)*n[2]:+n[2],r&&(r.unit=l,r.start=c,r.end=i)),i}var he={};function pe(e){var t,n=e.ownerDocument,r=e.nodeName,i=he[r];return i||(t=n.body.appendChild(n.createElement(r)),i=E.css(t,"display"),t.parentNode.removeChild(t),"none"===i&&(i="block"),he[r]=i,i)}function de(e,t){for(var n,r,i=[],o=0,s=e.length;o<s;o++)(r=e[o]).style&&(n=r.style.display,t?("none"===n&&(i[o]=J.get(r,"display")||null,i[o]||(r.style.display="")),""===r.style.display&&le(r)&&(i[o]=pe(r))):"none"!==n&&(i[o]="none",J.set(r,"display",n)));for(o=0;o<s;o++)null!=i[o]&&(e[o].style.display=i[o]);return e}E.fn.extend({show:function(){return de(this,!0)},hide:function(){return de(this)},toggle:function(e){return"boolean"==typeof e?e?this.show():this.hide():this.each((function(){le(this)?E(this).show():E(this).hide()}))}});var ge=/^(?:checkbox|radio)$/i,ve=/<([a-z][^\/\0>\x20\t\r\n\f]*)/i,me=/^$|^module$|\/(?:java|ecma)script/i,ye={option:[1,"<select multiple='multiple'>","</select>"],thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function _e(e,t){var n;return n=void 0!==e.getElementsByTagName?e.getElementsByTagName(t||"*"):void 0!==e.querySelectorAll?e.querySelectorAll(t||"*"):[],void 0===t||t&&k(e,t)?E.merge([e],n):n}function be(e,t){for(var n=0,r=e.length;n<r;n++)J.set(e[n],"globalEval",!t||J.get(t[n],"globalEval"))}ye.optgroup=ye.option,ye.tbody=ye.tfoot=ye.colgroup=ye.caption=ye.thead,ye.th=ye.td;var we,xe,Ee=/<|&#?\w+;/;function Te(e,t,n,r,i){for(var o,s,a,u,l,c,f=t.createDocumentFragment(),h=[],p=0,d=e.length;p<d;p++)if((o=e[p])||0===o)if("object"===x(o))E.merge(h,o.nodeType?[o]:o);else if(Ee.test(o)){for(s=s||f.appendChild(t.createElement("div")),a=(ve.exec(o)||["",""])[1].toLowerCase(),u=ye[a]||ye._default,s.innerHTML=u[1]+E.htmlPrefilter(o)+u[2],c=u[0];c--;)s=s.lastChild;E.merge(h,s.childNodes),(s=f.firstChild).textContent=""}else h.push(t.createTextNode(o));for(f.textContent="",p=0;o=h[p++];)if(r&&E.inArray(o,r)>-1)i&&i.push(o);else if(l=ae(o),s=_e(f.appendChild(o),"script"),l&&be(s),n)for(c=0;o=s[c++];)me.test(o.type||"")&&n.push(o);return f}we=s.createDocumentFragment().appendChild(s.createElement("div")),(xe=s.createElement("input")).setAttribute("type","radio"),xe.setAttribute("checked","checked"),xe.setAttribute("name","t"),we.appendChild(xe),m.checkClone=we.cloneNode(!0).cloneNode(!0).lastChild.checked,we.innerHTML="<textarea>x</textarea>",m.noCloneChecked=!!we.cloneNode(!0).lastChild.defaultValue;var Ce=/^key/,Se=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,Ae=/^([^.]*)(?:\.(.+)|)/;function De(){return!0}function ze(){return!1}function ke(e,t){return e===function(){try{return s.activeElement}catch(e){}}()==("focus"===t)}function Re(e,t,n,r,i,o){var s,a;if("object"==typeof t){for(a in"string"!=typeof n&&(r=r||n,n=void 0),t)Re(e,a,n,r,t[a],o);return e}if(null==r&&null==i?(i=n,r=n=void 0):null==i&&("string"==typeof n?(i=r,r=void 0):(i=r,r=n,n=void 0)),!1===i)i=ze;else if(!i)return e;return 1===o&&(s=i,(i=function(e){return E().off(e),s.apply(this,arguments)}).guid=s.guid||(s.guid=E.guid++)),e.each((function(){E.event.add(this,t,i,r,n)}))}function Ne(e,t,n){n?(J.set(e,t,!1),E.event.add(e,t,{namespace:!1,handler:function(e){var r,i,o=J.get(this,t);if(1&e.isTrigger&&this[t]){if(o.length)(E.event.special[t]||{}).delegateType&&e.stopPropagation();else if(o=u.call(arguments),J.set(this,t,o),r=n(this,t),this[t](),o!==(i=J.get(this,t))||r?J.set(this,t,!1):i={},o!==i)return e.stopImmediatePropagation(),e.preventDefault(),i.value}else o.length&&(J.set(this,t,{value:E.event.trigger(E.extend(o[0],E.Event.prototype),o.slice(1),this)}),e.stopImmediatePropagation())}})):void 0===J.get(e,t)&&E.event.add(e,t,De)}E.event={global:{},add:function(e,t,n,r,i){var o,s,a,u,l,c,f,h,p,d,g,v=J.get(e);if(v)for(n.handler&&(n=(o=n).handler,i=o.selector),i&&E.find.matchesSelector(se,i),n.guid||(n.guid=E.guid++),(u=v.events)||(u=v.events={}),(s=v.handle)||(s=v.handle=function(t){return void 0!==E&&E.event.triggered!==t.type?E.event.dispatch.apply(e,arguments):void 0}),l=(t=(t||"").match(q)||[""]).length;l--;)p=g=(a=Ae.exec(t[l])||[])[1],d=(a[2]||"").split(".").sort(),p&&(f=E.event.special[p]||{},p=(i?f.delegateType:f.bindType)||p,f=E.event.special[p]||{},c=E.extend({type:p,origType:g,data:r,handler:n,guid:n.guid,selector:i,needsContext:i&&E.expr.match.needsContext.test(i),namespace:d.join(".")},o),(h=u[p])||((h=u[p]=[]).delegateCount=0,f.setup&&!1!==f.setup.call(e,r,d,s)||e.addEventListener&&e.addEventListener(p,s)),f.add&&(f.add.call(e,c),c.handler.guid||(c.handler.guid=n.guid)),i?h.splice(h.delegateCount++,0,c):h.push(c),E.event.global[p]=!0)},remove:function(e,t,n,r,i){var o,s,a,u,l,c,f,h,p,d,g,v=J.hasData(e)&&J.get(e);if(v&&(u=v.events)){for(l=(t=(t||"").match(q)||[""]).length;l--;)if(p=g=(a=Ae.exec(t[l])||[])[1],d=(a[2]||"").split(".").sort(),p){for(f=E.event.special[p]||{},h=u[p=(r?f.delegateType:f.bindType)||p]||[],a=a[2]&&new RegExp("(^|\\.)"+d.join("\\.(?:.*\\.|)")+"(\\.|$)"),s=o=h.length;o--;)c=h[o],!i&&g!==c.origType||n&&n.guid!==c.guid||a&&!a.test(c.namespace)||r&&r!==c.selector&&("**"!==r||!c.selector)||(h.splice(o,1),c.selector&&h.delegateCount--,f.remove&&f.remove.call(e,c));s&&!h.length&&(f.teardown&&!1!==f.teardown.call(e,d,v.handle)||E.removeEvent(e,p,v.handle),delete u[p])}else for(p in u)E.event.remove(e,p+t[l],n,r,!0);E.isEmptyObject(u)&&J.remove(e,"handle events")}},dispatch:function(e){var t,n,r,i,o,s,a=E.event.fix(e),u=new Array(arguments.length),l=(J.get(this,"events")||{})[a.type]||[],c=E.event.special[a.type]||{};for(u[0]=a,t=1;t<arguments.length;t++)u[t]=arguments[t];if(a.delegateTarget=this,!c.preDispatch||!1!==c.preDispatch.call(this,a)){for(s=E.event.handlers.call(this,a,l),t=0;(i=s[t++])&&!a.isPropagationStopped();)for(a.currentTarget=i.elem,n=0;(o=i.handlers[n++])&&!a.isImmediatePropagationStopped();)a.rnamespace&&!1!==o.namespace&&!a.rnamespace.test(o.namespace)||(a.handleObj=o,a.data=o.data,void 0!==(r=((E.event.special[o.origType]||{}).handle||o.handler).apply(i.elem,u))&&!1===(a.result=r)&&(a.preventDefault(),a.stopPropagation()));return c.postDispatch&&c.postDispatch.call(this,a),a.result}},handlers:function(e,t){var n,r,i,o,s,a=[],u=t.delegateCount,l=e.target;if(u&&l.nodeType&&!("click"===e.type&&e.button>=1))for(;l!==this;l=l.parentNode||this)if(1===l.nodeType&&("click"!==e.type||!0!==l.disabled)){for(o=[],s={},n=0;n<u;n++)void 0===s[i=(r=t[n]).selector+" "]&&(s[i]=r.needsContext?E(i,this).index(l)>-1:E.find(i,this,null,[l]).length),s[i]&&o.push(r);o.length&&a.push({elem:l,handlers:o})}return l=this,u<t.length&&a.push({elem:l,handlers:t.slice(u)}),a},addProp:function(e,t){Object.defineProperty(E.Event.prototype,e,{enumerable:!0,configurable:!0,get:y(t)?function(){if(this.originalEvent)return t(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[e]},set:function(t){Object.defineProperty(this,e,{enumerable:!0,configurable:!0,writable:!0,value:t})}})},fix:function(e){return e[E.expando]?e:new E.Event(e)},special:{load:{noBubble:!0},click:{setup:function(e){var t=this||e;return ge.test(t.type)&&t.click&&k(t,"input")&&Ne(t,"click",De),!1},trigger:function(e){var t=this||e;return ge.test(t.type)&&t.click&&k(t,"input")&&Ne(t,"click"),!0},_default:function(e){var t=e.target;return ge.test(t.type)&&t.click&&k(t,"input")&&J.get(t,"click")||k(t,"a")}},beforeunload:{postDispatch:function(e){void 0!==e.result&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}},E.removeEvent=function(e,t,n){e.removeEventListener&&e.removeEventListener(t,n)},E.Event=function(e,t){if(!(this instanceof E.Event))return new E.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||void 0===e.defaultPrevented&&!1===e.returnValue?De:ze,this.target=e.target&&3===e.target.nodeType?e.target.parentNode:e.target,this.currentTarget=e.currentTarget,this.relatedTarget=e.relatedTarget):this.type=e,t&&E.extend(this,t),this.timeStamp=e&&e.timeStamp||Date.now(),this[E.expando]=!0},E.Event.prototype={constructor:E.Event,isDefaultPrevented:ze,isPropagationStopped:ze,isImmediatePropagationStopped:ze,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=De,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=De,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=De,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},E.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(e){var t=e.button;return null==e.which&&Ce.test(e.type)?null!=e.charCode?e.charCode:e.keyCode:!e.which&&void 0!==t&&Se.test(e.type)?1&t?1:2&t?3:4&t?2:0:e.which}},E.event.addProp),E.each({focus:"focusin",blur:"focusout"},(function(e,t){E.event.special[e]={setup:function(){return Ne(this,e,ke),!1},trigger:function(){return Ne(this,e),!0},delegateType:t}})),E.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},(function(e,t){E.event.special[e]={delegateType:t,bindType:t,handle:function(e){var n,r=this,i=e.relatedTarget,o=e.handleObj;return i&&(i===r||E.contains(r,i))||(e.type=o.origType,n=o.handler.apply(this,arguments),e.type=t),n}}})),E.fn.extend({on:function(e,t,n,r){return Re(this,e,t,n,r)},one:function(e,t,n,r){return Re(this,e,t,n,r,1)},off:function(e,t,n){var r,i;if(e&&e.preventDefault&&e.handleObj)return r=e.handleObj,E(e.delegateTarget).off(r.namespace?r.origType+"."+r.namespace:r.origType,r.selector,r.handler),this;if("object"==typeof e){for(i in e)this.off(i,t,e[i]);return this}return!1!==t&&"function"!=typeof t||(n=t,t=void 0),!1===n&&(n=ze),this.each((function(){E.event.remove(this,e,n,t)}))}});var je=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([a-z][^\/\0>\x20\t\r\n\f]*)[^>]*)\/>/gi,Oe=/<script|<style|<link/i,Fe=/checked\s*(?:[^=]|=\s*.checked.)/i,Le=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Pe(e,t){return k(e,"table")&&k(11!==t.nodeType?t:t.firstChild,"tr")&&E(e).children("tbody")[0]||e}function qe(e){return e.type=(null!==e.getAttribute("type"))+"/"+e.type,e}function Me(e){return"true/"===(e.type||"").slice(0,5)?e.type=e.type.slice(5):e.removeAttribute("type"),e}function Ie(e,t){var n,r,i,o,s,a,u,l;if(1===t.nodeType){if(J.hasData(e)&&(o=J.access(e),s=J.set(t,o),l=o.events))for(i in delete s.handle,s.events={},l)for(n=0,r=l[i].length;n<r;n++)E.event.add(t,i,l[i][n]);Q.hasData(e)&&(a=Q.access(e),u=E.extend({},a),Q.set(t,u))}}function Be(e,t){var n=t.nodeName.toLowerCase();"input"===n&&ge.test(e.type)?t.checked=e.checked:"input"!==n&&"textarea"!==n||(t.defaultValue=e.defaultValue)}function We(e,t,n,r){t=l.apply([],t);var i,o,s,a,u,c,f=0,h=e.length,p=h-1,d=t[0],g=y(d);if(g||h>1&&"string"==typeof d&&!m.checkClone&&Fe.test(d))return e.each((function(i){var o=e.eq(i);g&&(t[0]=d.call(this,i,o.html())),We(o,t,n,r)}));if(h&&(o=(i=Te(t,e[0].ownerDocument,!1,e,r)).firstChild,1===i.childNodes.length&&(i=o),o||r)){for(a=(s=E.map(_e(i,"script"),qe)).length;f<h;f++)u=i,f!==p&&(u=E.clone(u,!0,!0),a&&E.merge(s,_e(u,"script"))),n.call(e[f],u,f);if(a)for(c=s[s.length-1].ownerDocument,E.map(s,Me),f=0;f<a;f++)u=s[f],me.test(u.type||"")&&!J.access(u,"globalEval")&&E.contains(c,u)&&(u.src&&"module"!==(u.type||"").toLowerCase()?E._evalUrl&&!u.noModule&&E._evalUrl(u.src,{nonce:u.nonce||u.getAttribute("nonce")}):w(u.textContent.replace(Le,""),u,c))}return e}function He(e,t,n){for(var r,i=t?E.filter(t,e):e,o=0;null!=(r=i[o]);o++)n||1!==r.nodeType||E.cleanData(_e(r)),r.parentNode&&(n&&ae(r)&&be(_e(r,"script")),r.parentNode.removeChild(r));return e}E.extend({htmlPrefilter:function(e){return e.replace(je,"<$1></$2>")},clone:function(e,t,n){var r,i,o,s,a=e.cloneNode(!0),u=ae(e);if(!(m.noCloneChecked||1!==e.nodeType&&11!==e.nodeType||E.isXMLDoc(e)))for(s=_e(a),r=0,i=(o=_e(e)).length;r<i;r++)Be(o[r],s[r]);if(t)if(n)for(o=o||_e(e),s=s||_e(a),r=0,i=o.length;r<i;r++)Ie(o[r],s[r]);else Ie(e,a);return(s=_e(a,"script")).length>0&&be(s,!u&&_e(e,"script")),a},cleanData:function(e){for(var t,n,r,i=E.event.special,o=0;void 0!==(n=e[o]);o++)if(Y(n)){if(t=n[J.expando]){if(t.events)for(r in t.events)i[r]?E.event.remove(n,r):E.removeEvent(n,r,t.handle);n[J.expando]=void 0}n[Q.expando]&&(n[Q.expando]=void 0)}}}),E.fn.extend({detach:function(e){return He(this,e,!0)},remove:function(e){return He(this,e)},text:function(e){return $(this,(function(e){return void 0===e?E.text(this):this.empty().each((function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=e)}))}),null,e,arguments.length)},append:function(){return We(this,arguments,(function(e){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||Pe(this,e).appendChild(e)}))},prepend:function(){return We(this,arguments,(function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Pe(this,e);t.insertBefore(e,t.firstChild)}}))},before:function(){return We(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this)}))},after:function(){return We(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)}))},empty:function(){for(var e,t=0;null!=(e=this[t]);t++)1===e.nodeType&&(E.cleanData(_e(e,!1)),e.textContent="");return this},clone:function(e,t){return e=null!=e&&e,t=null==t?e:t,this.map((function(){return E.clone(this,e,t)}))},html:function(e){return $(this,(function(e){var t=this[0]||{},n=0,r=this.length;if(void 0===e&&1===t.nodeType)return t.innerHTML;if("string"==typeof e&&!Oe.test(e)&&!ye[(ve.exec(e)||["",""])[1].toLowerCase()]){e=E.htmlPrefilter(e);try{for(;n<r;n++)1===(t=this[n]||{}).nodeType&&(E.cleanData(_e(t,!1)),t.innerHTML=e);t=0}catch(e){}}t&&this.empty().append(e)}),null,e,arguments.length)},replaceWith:function(){var e=[];return We(this,arguments,(function(t){var n=this.parentNode;E.inArray(this,e)<0&&(E.cleanData(_e(this)),n&&n.replaceChild(t,this))}),e)}}),E.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},(function(e,t){E.fn[e]=function(e){for(var n,r=[],i=E(e),o=i.length-1,s=0;s<=o;s++)n=s===o?this:this.clone(!0),E(i[s])[t](n),c.apply(r,n.get());return this.pushStack(r)}}));var Ue=new RegExp("^("+re+")(?!px)[a-z%]+$","i"),$e=function(e){var t=e.ownerDocument.defaultView;return t&&t.opener||(t=n),t.getComputedStyle(e)},Xe=new RegExp(oe.join("|"),"i");function Ve(e,t,n){var r,i,o,s,a=e.style;return(n=n||$e(e))&&(""!==(s=n.getPropertyValue(t)||n[t])||ae(e)||(s=E.style(e,t)),!m.pixelBoxStyles()&&Ue.test(s)&&Xe.test(t)&&(r=a.width,i=a.minWidth,o=a.maxWidth,a.minWidth=a.maxWidth=a.width=s,s=n.width,a.width=r,a.minWidth=i,a.maxWidth=o)),void 0!==s?s+"":s}function Ge(e,t){return{get:function(){if(!e())return(this.get=t).apply(this,arguments);delete this.get}}}!function(){function e(){if(c){l.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",c.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",se.appendChild(l).appendChild(c);var e=n.getComputedStyle(c);r="1%"!==e.top,u=12===t(e.marginLeft),c.style.right="60%",a=36===t(e.right),i=36===t(e.width),c.style.position="absolute",o=12===t(c.offsetWidth/3),se.removeChild(l),c=null}}function t(e){return Math.round(parseFloat(e))}var r,i,o,a,u,l=s.createElement("div"),c=s.createElement("div");c.style&&(c.style.backgroundClip="content-box",c.cloneNode(!0).style.backgroundClip="",m.clearCloneStyle="content-box"===c.style.backgroundClip,E.extend(m,{boxSizingReliable:function(){return e(),i},pixelBoxStyles:function(){return e(),a},pixelPosition:function(){return e(),r},reliableMarginLeft:function(){return e(),u},scrollboxSize:function(){return e(),o}}))}();var Ke=["Webkit","Moz","ms"],Ye=s.createElement("div").style,Ze={};function Je(e){var t=E.cssProps[e]||Ze[e];return t||(e in Ye?e:Ze[e]=function(e){for(var t=e[0].toUpperCase()+e.slice(1),n=Ke.length;n--;)if((e=Ke[n]+t)in Ye)return e}(e)||e)}var Qe=/^(none|table(?!-c[ea]).+)/,et=/^--/,tt={position:"absolute",visibility:"hidden",display:"block"},nt={letterSpacing:"0",fontWeight:"400"};function rt(e,t,n){var r=ie.exec(t);return r?Math.max(0,r[2]-(n||0))+(r[3]||"px"):t}function it(e,t,n,r,i,o){var s="width"===t?1:0,a=0,u=0;if(n===(r?"border":"content"))return 0;for(;s<4;s+=2)"margin"===n&&(u+=E.css(e,n+oe[s],!0,i)),r?("content"===n&&(u-=E.css(e,"padding"+oe[s],!0,i)),"margin"!==n&&(u-=E.css(e,"border"+oe[s]+"Width",!0,i))):(u+=E.css(e,"padding"+oe[s],!0,i),"padding"!==n?u+=E.css(e,"border"+oe[s]+"Width",!0,i):a+=E.css(e,"border"+oe[s]+"Width",!0,i));return!r&&o>=0&&(u+=Math.max(0,Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-o-u-a-.5))||0),u}function ot(e,t,n){var r=$e(e),i=(!m.boxSizingReliable()||n)&&"border-box"===E.css(e,"boxSizing",!1,r),o=i,s=Ve(e,t,r),a="offset"+t[0].toUpperCase()+t.slice(1);if(Ue.test(s)){if(!n)return s;s="auto"}return(!m.boxSizingReliable()&&i||"auto"===s||!parseFloat(s)&&"inline"===E.css(e,"display",!1,r))&&e.getClientRects().length&&(i="border-box"===E.css(e,"boxSizing",!1,r),(o=a in e)&&(s=e[a])),(s=parseFloat(s)||0)+it(e,t,n||(i?"border":"content"),o,r,s)+"px"}function st(e,t,n,r,i){return new st.prototype.init(e,t,n,r,i)}E.extend({cssHooks:{opacity:{get:function(e,t){if(t){var n=Ve(e,"opacity");return""===n?"1":n}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{},style:function(e,t,n,r){if(e&&3!==e.nodeType&&8!==e.nodeType&&e.style){var i,o,s,a=K(t),u=et.test(t),l=e.style;if(u||(t=Je(a)),s=E.cssHooks[t]||E.cssHooks[a],void 0===n)return s&&"get"in s&&void 0!==(i=s.get(e,!1,r))?i:l[t];"string"===(o=typeof n)&&(i=ie.exec(n))&&i[1]&&(n=fe(e,t,i),o="number"),null!=n&&n==n&&("number"!==o||u||(n+=i&&i[3]||(E.cssNumber[a]?"":"px")),m.clearCloneStyle||""!==n||0!==t.indexOf("background")||(l[t]="inherit"),s&&"set"in s&&void 0===(n=s.set(e,n,r))||(u?l.setProperty(t,n):l[t]=n))}},css:function(e,t,n,r){var i,o,s,a=K(t);return et.test(t)||(t=Je(a)),(s=E.cssHooks[t]||E.cssHooks[a])&&"get"in s&&(i=s.get(e,!0,n)),void 0===i&&(i=Ve(e,t,r)),"normal"===i&&t in nt&&(i=nt[t]),""===n||n?(o=parseFloat(i),!0===n||isFinite(o)?o||0:i):i}}),E.each(["height","width"],(function(e,t){E.cssHooks[t]={get:function(e,n,r){if(n)return!Qe.test(E.css(e,"display"))||e.getClientRects().length&&e.getBoundingClientRect().width?ot(e,t,r):ce(e,tt,(function(){return ot(e,t,r)}))},set:function(e,n,r){var i,o=$e(e),s=!m.scrollboxSize()&&"absolute"===o.position,a=(s||r)&&"border-box"===E.css(e,"boxSizing",!1,o),u=r?it(e,t,r,a,o):0;return a&&s&&(u-=Math.ceil(e["offset"+t[0].toUpperCase()+t.slice(1)]-parseFloat(o[t])-it(e,t,"border",!1,o)-.5)),u&&(i=ie.exec(n))&&"px"!==(i[3]||"px")&&(e.style[t]=n,n=E.css(e,t)),rt(0,n,u)}}})),E.cssHooks.marginLeft=Ge(m.reliableMarginLeft,(function(e,t){if(t)return(parseFloat(Ve(e,"marginLeft"))||e.getBoundingClientRect().left-ce(e,{marginLeft:0},(function(){return e.getBoundingClientRect().left})))+"px"})),E.each({margin:"",padding:"",border:"Width"},(function(e,t){E.cssHooks[e+t]={expand:function(n){for(var r=0,i={},o="string"==typeof n?n.split(" "):[n];r<4;r++)i[e+oe[r]+t]=o[r]||o[r-2]||o[0];return i}},"margin"!==e&&(E.cssHooks[e+t].set=rt)})),E.fn.extend({css:function(e,t){return $(this,(function(e,t,n){var r,i,o={},s=0;if(Array.isArray(t)){for(r=$e(e),i=t.length;s<i;s++)o[t[s]]=E.css(e,t[s],!1,r);return o}return void 0!==n?E.style(e,t,n):E.css(e,t)}),e,t,arguments.length>1)}}),E.Tween=st,st.prototype={constructor:st,init:function(e,t,n,r,i,o){this.elem=e,this.prop=n,this.easing=i||E.easing._default,this.options=t,this.start=this.now=this.cur(),this.end=r,this.unit=o||(E.cssNumber[n]?"":"px")},cur:function(){var e=st.propHooks[this.prop];return e&&e.get?e.get(this):st.propHooks._default.get(this)},run:function(e){var t,n=st.propHooks[this.prop];return this.options.duration?this.pos=t=E.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),n&&n.set?n.set(this):st.propHooks._default.set(this),this}},st.prototype.init.prototype=st.prototype,st.propHooks={_default:{get:function(e){var t;return 1!==e.elem.nodeType||null!=e.elem[e.prop]&&null==e.elem.style[e.prop]?e.elem[e.prop]:(t=E.css(e.elem,e.prop,""))&&"auto"!==t?t:0},set:function(e){E.fx.step[e.prop]?E.fx.step[e.prop](e):1!==e.elem.nodeType||!E.cssHooks[e.prop]&&null==e.elem.style[Je(e.prop)]?e.elem[e.prop]=e.now:E.style(e.elem,e.prop,e.now+e.unit)}}},st.propHooks.scrollTop=st.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},E.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},E.fx=st.prototype.init,E.fx.step={};var at,ut,lt=/^(?:toggle|show|hide)$/,ct=/queueHooks$/;function ft(){ut&&(!1===s.hidden&&n.requestAnimationFrame?n.requestAnimationFrame(ft):n.setTimeout(ft,E.fx.interval),E.fx.tick())}function ht(){return n.setTimeout((function(){at=void 0})),at=Date.now()}function pt(e,t){var n,r=0,i={height:e};for(t=t?1:0;r<4;r+=2-t)i["margin"+(n=oe[r])]=i["padding"+n]=e;return t&&(i.opacity=i.width=e),i}function dt(e,t,n){for(var r,i=(gt.tweeners[t]||[]).concat(gt.tweeners["*"]),o=0,s=i.length;o<s;o++)if(r=i[o].call(n,t,e))return r}function gt(e,t,n){var r,i,o=0,s=gt.prefilters.length,a=E.Deferred().always((function(){delete u.elem})),u=function(){if(i)return!1;for(var t=at||ht(),n=Math.max(0,l.startTime+l.duration-t),r=1-(n/l.duration||0),o=0,s=l.tweens.length;o<s;o++)l.tweens[o].run(r);return a.notifyWith(e,[l,r,n]),r<1&&s?n:(s||a.notifyWith(e,[l,1,0]),a.resolveWith(e,[l]),!1)},l=a.promise({elem:e,props:E.extend({},t),opts:E.extend(!0,{specialEasing:{},easing:E.easing._default},n),originalProperties:t,originalOptions:n,startTime:at||ht(),duration:n.duration,tweens:[],createTween:function(t,n){var r=E.Tween(e,l.opts,t,n,l.opts.specialEasing[t]||l.opts.easing);return l.tweens.push(r),r},stop:function(t){var n=0,r=t?l.tweens.length:0;if(i)return this;for(i=!0;n<r;n++)l.tweens[n].run(1);return t?(a.notifyWith(e,[l,1,0]),a.resolveWith(e,[l,t])):a.rejectWith(e,[l,t]),this}}),c=l.props;for(!function(e,t){var n,r,i,o,s;for(n in e)if(i=t[r=K(n)],o=e[n],Array.isArray(o)&&(i=o[1],o=e[n]=o[0]),n!==r&&(e[r]=o,delete e[n]),(s=E.cssHooks[r])&&"expand"in s)for(n in o=s.expand(o),delete e[r],o)n in e||(e[n]=o[n],t[n]=i);else t[r]=i}(c,l.opts.specialEasing);o<s;o++)if(r=gt.prefilters[o].call(l,e,c,l.opts))return y(r.stop)&&(E._queueHooks(l.elem,l.opts.queue).stop=r.stop.bind(r)),r;return E.map(c,dt,l),y(l.opts.start)&&l.opts.start.call(e,l),l.progress(l.opts.progress).done(l.opts.done,l.opts.complete).fail(l.opts.fail).always(l.opts.always),E.fx.timer(E.extend(u,{elem:e,anim:l,queue:l.opts.queue})),l}E.Animation=E.extend(gt,{tweeners:{"*":[function(e,t){var n=this.createTween(e,t);return fe(n.elem,e,ie.exec(t),n),n}]},tweener:function(e,t){y(e)?(t=e,e=["*"]):e=e.match(q);for(var n,r=0,i=e.length;r<i;r++)n=e[r],gt.tweeners[n]=gt.tweeners[n]||[],gt.tweeners[n].unshift(t)},prefilters:[function(e,t,n){var r,i,o,s,a,u,l,c,f="width"in t||"height"in t,h=this,p={},d=e.style,g=e.nodeType&&le(e),v=J.get(e,"fxshow");for(r in n.queue||(null==(s=E._queueHooks(e,"fx")).unqueued&&(s.unqueued=0,a=s.empty.fire,s.empty.fire=function(){s.unqueued||a()}),s.unqueued++,h.always((function(){h.always((function(){s.unqueued--,E.queue(e,"fx").length||s.empty.fire()}))}))),t)if(i=t[r],lt.test(i)){if(delete t[r],o=o||"toggle"===i,i===(g?"hide":"show")){if("show"!==i||!v||void 0===v[r])continue;g=!0}p[r]=v&&v[r]||E.style(e,r)}if((u=!E.isEmptyObject(t))||!E.isEmptyObject(p))for(r in f&&1===e.nodeType&&(n.overflow=[d.overflow,d.overflowX,d.overflowY],null==(l=v&&v.display)&&(l=J.get(e,"display")),"none"===(c=E.css(e,"display"))&&(l?c=l:(de([e],!0),l=e.style.display||l,c=E.css(e,"display"),de([e]))),("inline"===c||"inline-block"===c&&null!=l)&&"none"===E.css(e,"float")&&(u||(h.done((function(){d.display=l})),null==l&&(c=d.display,l="none"===c?"":c)),d.display="inline-block")),n.overflow&&(d.overflow="hidden",h.always((function(){d.overflow=n.overflow[0],d.overflowX=n.overflow[1],d.overflowY=n.overflow[2]}))),u=!1,p)u||(v?"hidden"in v&&(g=v.hidden):v=J.access(e,"fxshow",{display:l}),o&&(v.hidden=!g),g&&de([e],!0),h.done((function(){for(r in g||de([e]),J.remove(e,"fxshow"),p)E.style(e,r,p[r])}))),u=dt(g?v[r]:0,r,h),r in v||(v[r]=u.start,g&&(u.end=u.start,u.start=0))}],prefilter:function(e,t){t?gt.prefilters.unshift(e):gt.prefilters.push(e)}}),E.speed=function(e,t,n){var r=e&&"object"==typeof e?E.extend({},e):{complete:n||!n&&t||y(e)&&e,duration:e,easing:n&&t||t&&!y(t)&&t};return E.fx.off?r.duration=0:"number"!=typeof r.duration&&(r.duration in E.fx.speeds?r.duration=E.fx.speeds[r.duration]:r.duration=E.fx.speeds._default),null!=r.queue&&!0!==r.queue||(r.queue="fx"),r.old=r.complete,r.complete=function(){y(r.old)&&r.old.call(this),r.queue&&E.dequeue(this,r.queue)},r},E.fn.extend({fadeTo:function(e,t,n,r){return this.filter(le).css("opacity",0).show().end().animate({opacity:t},e,n,r)},animate:function(e,t,n,r){var i=E.isEmptyObject(e),o=E.speed(t,n,r),s=function(){var t=gt(this,E.extend({},e),o);(i||J.get(this,"finish"))&&t.stop(!0)};return s.finish=s,i||!1===o.queue?this.each(s):this.queue(o.queue,s)},stop:function(e,t,n){var r=function(e){var t=e.stop;delete e.stop,t(n)};return"string"!=typeof e&&(n=t,t=e,e=void 0),t&&!1!==e&&this.queue(e||"fx",[]),this.each((function(){var t=!0,i=null!=e&&e+"queueHooks",o=E.timers,s=J.get(this);if(i)s[i]&&s[i].stop&&r(s[i]);else for(i in s)s[i]&&s[i].stop&&ct.test(i)&&r(s[i]);for(i=o.length;i--;)o[i].elem!==this||null!=e&&o[i].queue!==e||(o[i].anim.stop(n),t=!1,o.splice(i,1));!t&&n||E.dequeue(this,e)}))},finish:function(e){return!1!==e&&(e=e||"fx"),this.each((function(){var t,n=J.get(this),r=n[e+"queue"],i=n[e+"queueHooks"],o=E.timers,s=r?r.length:0;for(n.finish=!0,E.queue(this,e,[]),i&&i.stop&&i.stop.call(this,!0),t=o.length;t--;)o[t].elem===this&&o[t].queue===e&&(o[t].anim.stop(!0),o.splice(t,1));for(t=0;t<s;t++)r[t]&&r[t].finish&&r[t].finish.call(this);delete n.finish}))}}),E.each(["toggle","show","hide"],(function(e,t){var n=E.fn[t];E.fn[t]=function(e,r,i){return null==e||"boolean"==typeof e?n.apply(this,arguments):this.animate(pt(t,!0),e,r,i)}})),E.each({slideDown:pt("show"),slideUp:pt("hide"),slideToggle:pt("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},(function(e,t){E.fn[e]=function(e,n,r){return this.animate(t,e,n,r)}})),E.timers=[],E.fx.tick=function(){var e,t=0,n=E.timers;for(at=Date.now();t<n.length;t++)(e=n[t])()||n[t]!==e||n.splice(t--,1);n.length||E.fx.stop(),at=void 0},E.fx.timer=function(e){E.timers.push(e),E.fx.start()},E.fx.interval=13,E.fx.start=function(){ut||(ut=!0,ft())},E.fx.stop=function(){ut=null},E.fx.speeds={slow:600,fast:200,_default:400},E.fn.delay=function(e,t){return e=E.fx&&E.fx.speeds[e]||e,t=t||"fx",this.queue(t,(function(t,r){var i=n.setTimeout(t,e);r.stop=function(){n.clearTimeout(i)}}))},function(){var e=s.createElement("input"),t=s.createElement("select").appendChild(s.createElement("option"));e.type="checkbox",m.checkOn=""!==e.value,m.optSelected=t.selected,(e=s.createElement("input")).value="t",e.type="radio",m.radioValue="t"===e.value}();var vt,mt=E.expr.attrHandle;E.fn.extend({attr:function(e,t){return $(this,E.attr,e,t,arguments.length>1)},removeAttr:function(e){return this.each((function(){E.removeAttr(this,e)}))}}),E.extend({attr:function(e,t,n){var r,i,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return void 0===e.getAttribute?E.prop(e,t,n):(1===o&&E.isXMLDoc(e)||(i=E.attrHooks[t.toLowerCase()]||(E.expr.match.bool.test(t)?vt:void 0)),void 0!==n?null===n?void E.removeAttr(e,t):i&&"set"in i&&void 0!==(r=i.set(e,n,t))?r:(e.setAttribute(t,n+""),n):i&&"get"in i&&null!==(r=i.get(e,t))?r:null==(r=E.find.attr(e,t))?void 0:r)},attrHooks:{type:{set:function(e,t){if(!m.radioValue&&"radio"===t&&k(e,"input")){var n=e.value;return e.setAttribute("type",t),n&&(e.value=n),t}}}},removeAttr:function(e,t){var n,r=0,i=t&&t.match(q);if(i&&1===e.nodeType)for(;n=i[r++];)e.removeAttribute(n)}}),vt={set:function(e,t,n){return!1===t?E.removeAttr(e,n):e.setAttribute(n,n),n}},E.each(E.expr.match.bool.source.match(/\w+/g),(function(e,t){var n=mt[t]||E.find.attr;mt[t]=function(e,t,r){var i,o,s=t.toLowerCase();return r||(o=mt[s],mt[s]=i,i=null!=n(e,t,r)?s:null,mt[s]=o),i}}));var yt=/^(?:input|select|textarea|button)$/i,_t=/^(?:a|area)$/i;function bt(e){return(e.match(q)||[]).join(" ")}function wt(e){return e.getAttribute&&e.getAttribute("class")||""}function xt(e){return Array.isArray(e)?e:"string"==typeof e&&e.match(q)||[]}E.fn.extend({prop:function(e,t){return $(this,E.prop,e,t,arguments.length>1)},removeProp:function(e){return this.each((function(){delete this[E.propFix[e]||e]}))}}),E.extend({prop:function(e,t,n){var r,i,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return 1===o&&E.isXMLDoc(e)||(t=E.propFix[t]||t,i=E.propHooks[t]),void 0!==n?i&&"set"in i&&void 0!==(r=i.set(e,n,t))?r:e[t]=n:i&&"get"in i&&null!==(r=i.get(e,t))?r:e[t]},propHooks:{tabIndex:{get:function(e){var t=E.find.attr(e,"tabindex");return t?parseInt(t,10):yt.test(e.nodeName)||_t.test(e.nodeName)&&e.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),m.optSelected||(E.propHooks.selected={get:function(e){var t=e.parentNode;return t&&t.parentNode&&t.parentNode.selectedIndex,null},set:function(e){var t=e.parentNode;t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex)}}),E.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],(function(){E.propFix[this.toLowerCase()]=this})),E.fn.extend({addClass:function(e){var t,n,r,i,o,s,a,u=0;if(y(e))return this.each((function(t){E(this).addClass(e.call(this,t,wt(this)))}));if((t=xt(e)).length)for(;n=this[u++];)if(i=wt(n),r=1===n.nodeType&&" "+bt(i)+" "){for(s=0;o=t[s++];)r.indexOf(" "+o+" ")<0&&(r+=o+" ");i!==(a=bt(r))&&n.setAttribute("class",a)}return this},removeClass:function(e){var t,n,r,i,o,s,a,u=0;if(y(e))return this.each((function(t){E(this).removeClass(e.call(this,t,wt(this)))}));if(!arguments.length)return this.attr("class","");if((t=xt(e)).length)for(;n=this[u++];)if(i=wt(n),r=1===n.nodeType&&" "+bt(i)+" "){for(s=0;o=t[s++];)for(;r.indexOf(" "+o+" ")>-1;)r=r.replace(" "+o+" "," ");i!==(a=bt(r))&&n.setAttribute("class",a)}return this},toggleClass:function(e,t){var n=typeof e,r="string"===n||Array.isArray(e);return"boolean"==typeof t&&r?t?this.addClass(e):this.removeClass(e):y(e)?this.each((function(n){E(this).toggleClass(e.call(this,n,wt(this),t),t)})):this.each((function(){var t,i,o,s;if(r)for(i=0,o=E(this),s=xt(e);t=s[i++];)o.hasClass(t)?o.removeClass(t):o.addClass(t);else void 0!==e&&"boolean"!==n||((t=wt(this))&&J.set(this,"__className__",t),this.setAttribute&&this.setAttribute("class",t||!1===e?"":J.get(this,"__className__")||""))}))},hasClass:function(e){var t,n,r=0;for(t=" "+e+" ";n=this[r++];)if(1===n.nodeType&&(" "+bt(wt(n))+" ").indexOf(t)>-1)return!0;return!1}});var Et=/\r/g;E.fn.extend({val:function(e){var t,n,r,i=this[0];return arguments.length?(r=y(e),this.each((function(n){var i;1===this.nodeType&&(null==(i=r?e.call(this,n,E(this).val()):e)?i="":"number"==typeof i?i+="":Array.isArray(i)&&(i=E.map(i,(function(e){return null==e?"":e+""}))),(t=E.valHooks[this.type]||E.valHooks[this.nodeName.toLowerCase()])&&"set"in t&&void 0!==t.set(this,i,"value")||(this.value=i))}))):i?(t=E.valHooks[i.type]||E.valHooks[i.nodeName.toLowerCase()])&&"get"in t&&void 0!==(n=t.get(i,"value"))?n:"string"==typeof(n=i.value)?n.replace(Et,""):null==n?"":n:void 0}}),E.extend({valHooks:{option:{get:function(e){var t=E.find.attr(e,"value");return null!=t?t:bt(E.text(e))}},select:{get:function(e){var t,n,r,i=e.options,o=e.selectedIndex,s="select-one"===e.type,a=s?null:[],u=s?o+1:i.length;for(r=o<0?u:s?o:0;r<u;r++)if(((n=i[r]).selected||r===o)&&!n.disabled&&(!n.parentNode.disabled||!k(n.parentNode,"optgroup"))){if(t=E(n).val(),s)return t;a.push(t)}return a},set:function(e,t){for(var n,r,i=e.options,o=E.makeArray(t),s=i.length;s--;)((r=i[s]).selected=E.inArray(E.valHooks.option.get(r),o)>-1)&&(n=!0);return n||(e.selectedIndex=-1),o}}}}),E.each(["radio","checkbox"],(function(){E.valHooks[this]={set:function(e,t){if(Array.isArray(t))return e.checked=E.inArray(E(e).val(),t)>-1}},m.checkOn||(E.valHooks[this].get=function(e){return null===e.getAttribute("value")?"on":e.value})})),m.focusin="onfocusin"in n;var Tt=/^(?:focusinfocus|focusoutblur)$/,Ct=function(e){e.stopPropagation()};E.extend(E.event,{trigger:function(e,t,r,i){var o,a,u,l,c,f,h,p,g=[r||s],v=d.call(e,"type")?e.type:e,m=d.call(e,"namespace")?e.namespace.split("."):[];if(a=p=u=r=r||s,3!==r.nodeType&&8!==r.nodeType&&!Tt.test(v+E.event.triggered)&&(v.indexOf(".")>-1&&(m=v.split("."),v=m.shift(),m.sort()),c=v.indexOf(":")<0&&"on"+v,(e=e[E.expando]?e:new E.Event(v,"object"==typeof e&&e)).isTrigger=i?2:3,e.namespace=m.join("."),e.rnamespace=e.namespace?new RegExp("(^|\\.)"+m.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=void 0,e.target||(e.target=r),t=null==t?[e]:E.makeArray(t,[e]),h=E.event.special[v]||{},i||!h.trigger||!1!==h.trigger.apply(r,t))){if(!i&&!h.noBubble&&!_(r)){for(l=h.delegateType||v,Tt.test(l+v)||(a=a.parentNode);a;a=a.parentNode)g.push(a),u=a;u===(r.ownerDocument||s)&&g.push(u.defaultView||u.parentWindow||n)}for(o=0;(a=g[o++])&&!e.isPropagationStopped();)p=a,e.type=o>1?l:h.bindType||v,(f=(J.get(a,"events")||{})[e.type]&&J.get(a,"handle"))&&f.apply(a,t),(f=c&&a[c])&&f.apply&&Y(a)&&(e.result=f.apply(a,t),!1===e.result&&e.preventDefault());return e.type=v,i||e.isDefaultPrevented()||h._default&&!1!==h._default.apply(g.pop(),t)||!Y(r)||c&&y(r[v])&&!_(r)&&((u=r[c])&&(r[c]=null),E.event.triggered=v,e.isPropagationStopped()&&p.addEventListener(v,Ct),r[v](),e.isPropagationStopped()&&p.removeEventListener(v,Ct),E.event.triggered=void 0,u&&(r[c]=u)),e.result}},simulate:function(e,t,n){var r=E.extend(new E.Event,n,{type:e,isSimulated:!0});E.event.trigger(r,null,t)}}),E.fn.extend({trigger:function(e,t){return this.each((function(){E.event.trigger(e,t,this)}))},triggerHandler:function(e,t){var n=this[0];if(n)return E.event.trigger(e,t,n,!0)}}),m.focusin||E.each({focus:"focusin",blur:"focusout"},(function(e,t){var n=function(e){E.event.simulate(t,e.target,E.event.fix(e))};E.event.special[t]={setup:function(){var r=this.ownerDocument||this,i=J.access(r,t);i||r.addEventListener(e,n,!0),J.access(r,t,(i||0)+1)},teardown:function(){var r=this.ownerDocument||this,i=J.access(r,t)-1;i?J.access(r,t,i):(r.removeEventListener(e,n,!0),J.remove(r,t))}}}));var St=n.location,At=Date.now(),Dt=/\?/;E.parseXML=function(e){var t;if(!e||"string"!=typeof e)return null;try{t=(new n.DOMParser).parseFromString(e,"text/xml")}catch(e){t=void 0}return t&&!t.getElementsByTagName("parsererror").length||E.error("Invalid XML: "+e),t};var zt=/\[\]$/,kt=/\r?\n/g,Rt=/^(?:submit|button|image|reset|file)$/i,Nt=/^(?:input|select|textarea|keygen)/i;function jt(e,t,n,r){var i;if(Array.isArray(t))E.each(t,(function(t,i){n||zt.test(e)?r(e,i):jt(e+"["+("object"==typeof i&&null!=i?t:"")+"]",i,n,r)}));else if(n||"object"!==x(t))r(e,t);else for(i in t)jt(e+"["+i+"]",t[i],n,r)}E.param=function(e,t){var n,r=[],i=function(e,t){var n=y(t)?t():t;r[r.length]=encodeURIComponent(e)+"="+encodeURIComponent(null==n?"":n)};if(null==e)return"";if(Array.isArray(e)||e.jquery&&!E.isPlainObject(e))E.each(e,(function(){i(this.name,this.value)}));else for(n in e)jt(n,e[n],t,i);return r.join("&")},E.fn.extend({serialize:function(){return E.param(this.serializeArray())},serializeArray:function(){return this.map((function(){var e=E.prop(this,"elements");return e?E.makeArray(e):this})).filter((function(){var e=this.type;return this.name&&!E(this).is(":disabled")&&Nt.test(this.nodeName)&&!Rt.test(e)&&(this.checked||!ge.test(e))})).map((function(e,t){var n=E(this).val();return null==n?null:Array.isArray(n)?E.map(n,(function(e){return{name:t.name,value:e.replace(kt,"\r\n")}})):{name:t.name,value:n.replace(kt,"\r\n")}})).get()}});var Ot=/%20/g,Ft=/#.*$/,Lt=/([?&])_=[^&]*/,Pt=/^(.*?):[ \t]*([^\r\n]*)$/gm,qt=/^(?:GET|HEAD)$/,Mt=/^\/\//,It={},Bt={},Wt="*/".concat("*"),Ht=s.createElement("a");function Ut(e){return function(t,n){"string"!=typeof t&&(n=t,t="*");var r,i=0,o=t.toLowerCase().match(q)||[];if(y(n))for(;r=o[i++];)"+"===r[0]?(r=r.slice(1)||"*",(e[r]=e[r]||[]).unshift(n)):(e[r]=e[r]||[]).push(n)}}function $t(e,t,n,r){var i={},o=e===Bt;function s(a){var u;return i[a]=!0,E.each(e[a]||[],(function(e,a){var l=a(t,n,r);return"string"!=typeof l||o||i[l]?o?!(u=l):void 0:(t.dataTypes.unshift(l),s(l),!1)})),u}return s(t.dataTypes[0])||!i["*"]&&s("*")}function Xt(e,t){var n,r,i=E.ajaxSettings.flatOptions||{};for(n in t)void 0!==t[n]&&((i[n]?e:r||(r={}))[n]=t[n]);return r&&E.extend(!0,e,r),e}Ht.href=St.href,E.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:St.href,type:"GET",isLocal:/^(?:about|app|app-storage|.+-extension|file|res|widget):$/.test(St.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Wt,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":E.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?Xt(Xt(e,E.ajaxSettings),t):Xt(E.ajaxSettings,e)},ajaxPrefilter:Ut(It),ajaxTransport:Ut(Bt),ajax:function(e,t){"object"==typeof e&&(t=e,e=void 0),t=t||{};var r,i,o,a,u,l,c,f,h,p,d=E.ajaxSetup({},t),g=d.context||d,v=d.context&&(g.nodeType||g.jquery)?E(g):E.event,m=E.Deferred(),y=E.Callbacks("once memory"),_=d.statusCode||{},b={},w={},x="canceled",T={readyState:0,getResponseHeader:function(e){var t;if(c){if(!a)for(a={};t=Pt.exec(o);)a[t[1].toLowerCase()+" "]=(a[t[1].toLowerCase()+" "]||[]).concat(t[2]);t=a[e.toLowerCase()+" "]}return null==t?null:t.join(", ")},getAllResponseHeaders:function(){return c?o:null},setRequestHeader:function(e,t){return null==c&&(e=w[e.toLowerCase()]=w[e.toLowerCase()]||e,b[e]=t),this},overrideMimeType:function(e){return null==c&&(d.mimeType=e),this},statusCode:function(e){var t;if(e)if(c)T.always(e[T.status]);else for(t in e)_[t]=[_[t],e[t]];return this},abort:function(e){var t=e||x;return r&&r.abort(t),C(0,t),this}};if(m.promise(T),d.url=((e||d.url||St.href)+"").replace(Mt,St.protocol+"//"),d.type=t.method||t.type||d.method||d.type,d.dataTypes=(d.dataType||"*").toLowerCase().match(q)||[""],null==d.crossDomain){l=s.createElement("a");try{l.href=d.url,l.href=l.href,d.crossDomain=Ht.protocol+"//"+Ht.host!=l.protocol+"//"+l.host}catch(e){d.crossDomain=!0}}if(d.data&&d.processData&&"string"!=typeof d.data&&(d.data=E.param(d.data,d.traditional)),$t(It,d,t,T),c)return T;for(h in(f=E.event&&d.global)&&0==E.active++&&E.event.trigger("ajaxStart"),d.type=d.type.toUpperCase(),d.hasContent=!qt.test(d.type),i=d.url.replace(Ft,""),d.hasContent?d.data&&d.processData&&0===(d.contentType||"").indexOf("application/x-www-form-urlencoded")&&(d.data=d.data.replace(Ot,"+")):(p=d.url.slice(i.length),d.data&&(d.processData||"string"==typeof d.data)&&(i+=(Dt.test(i)?"&":"?")+d.data,delete d.data),!1===d.cache&&(i=i.replace(Lt,"$1"),p=(Dt.test(i)?"&":"?")+"_="+At+++p),d.url=i+p),d.ifModified&&(E.lastModified[i]&&T.setRequestHeader("If-Modified-Since",E.lastModified[i]),E.etag[i]&&T.setRequestHeader("If-None-Match",E.etag[i])),(d.data&&d.hasContent&&!1!==d.contentType||t.contentType)&&T.setRequestHeader("Content-Type",d.contentType),T.setRequestHeader("Accept",d.dataTypes[0]&&d.accepts[d.dataTypes[0]]?d.accepts[d.dataTypes[0]]+("*"!==d.dataTypes[0]?", "+Wt+"; q=0.01":""):d.accepts["*"]),d.headers)T.setRequestHeader(h,d.headers[h]);if(d.beforeSend&&(!1===d.beforeSend.call(g,T,d)||c))return T.abort();if(x="abort",y.add(d.complete),T.done(d.success),T.fail(d.error),r=$t(Bt,d,t,T)){if(T.readyState=1,f&&v.trigger("ajaxSend",[T,d]),c)return T;d.async&&d.timeout>0&&(u=n.setTimeout((function(){T.abort("timeout")}),d.timeout));try{c=!1,r.send(b,C)}catch(e){if(c)throw e;C(-1,e)}}else C(-1,"No Transport");function C(e,t,s,a){var l,h,p,b,w,x=t;c||(c=!0,u&&n.clearTimeout(u),r=void 0,o=a||"",T.readyState=e>0?4:0,l=e>=200&&e<300||304===e,s&&(b=function(e,t,n){for(var r,i,o,s,a=e.contents,u=e.dataTypes;"*"===u[0];)u.shift(),void 0===r&&(r=e.mimeType||t.getResponseHeader("Content-Type"));if(r)for(i in a)if(a[i]&&a[i].test(r)){u.unshift(i);break}if(u[0]in n)o=u[0];else{for(i in n){if(!u[0]||e.converters[i+" "+u[0]]){o=i;break}s||(s=i)}o=o||s}if(o)return o!==u[0]&&u.unshift(o),n[o]}(d,T,s)),b=function(e,t,n,r){var i,o,s,a,u,l={},c=e.dataTypes.slice();if(c[1])for(s in e.converters)l[s.toLowerCase()]=e.converters[s];for(o=c.shift();o;)if(e.responseFields[o]&&(n[e.responseFields[o]]=t),!u&&r&&e.dataFilter&&(t=e.dataFilter(t,e.dataType)),u=o,o=c.shift())if("*"===o)o=u;else if("*"!==u&&u!==o){if(!(s=l[u+" "+o]||l["* "+o]))for(i in l)if((a=i.split(" "))[1]===o&&(s=l[u+" "+a[0]]||l["* "+a[0]])){!0===s?s=l[i]:!0!==l[i]&&(o=a[0],c.unshift(a[1]));break}if(!0!==s)if(s&&e.throws)t=s(t);else try{t=s(t)}catch(e){return{state:"parsererror",error:s?e:"No conversion from "+u+" to "+o}}}return{state:"success",data:t}}(d,b,T,l),l?(d.ifModified&&((w=T.getResponseHeader("Last-Modified"))&&(E.lastModified[i]=w),(w=T.getResponseHeader("etag"))&&(E.etag[i]=w)),204===e||"HEAD"===d.type?x="nocontent":304===e?x="notmodified":(x=b.state,h=b.data,l=!(p=b.error))):(p=x,!e&&x||(x="error",e<0&&(e=0))),T.status=e,T.statusText=(t||x)+"",l?m.resolveWith(g,[h,x,T]):m.rejectWith(g,[T,x,p]),T.statusCode(_),_=void 0,f&&v.trigger(l?"ajaxSuccess":"ajaxError",[T,d,l?h:p]),y.fireWith(g,[T,x]),f&&(v.trigger("ajaxComplete",[T,d]),--E.active||E.event.trigger("ajaxStop")))}return T},getJSON:function(e,t,n){return E.get(e,t,n,"json")},getScript:function(e,t){return E.get(e,void 0,t,"script")}}),E.each(["get","post"],(function(e,t){E[t]=function(e,n,r,i){return y(n)&&(i=i||r,r=n,n=void 0),E.ajax(E.extend({url:e,type:t,dataType:i,data:n,success:r},E.isPlainObject(e)&&e))}})),E._evalUrl=function(e,t){return E.ajax({url:e,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(e){E.globalEval(e,t)}})},E.fn.extend({wrapAll:function(e){var t;return this[0]&&(y(e)&&(e=e.call(this[0])),t=E(e,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&t.insertBefore(this[0]),t.map((function(){for(var e=this;e.firstElementChild;)e=e.firstElementChild;return e})).append(this)),this},wrapInner:function(e){return y(e)?this.each((function(t){E(this).wrapInner(e.call(this,t))})):this.each((function(){var t=E(this),n=t.contents();n.length?n.wrapAll(e):t.append(e)}))},wrap:function(e){var t=y(e);return this.each((function(n){E(this).wrapAll(t?e.call(this,n):e)}))},unwrap:function(e){return this.parent(e).not("body").each((function(){E(this).replaceWith(this.childNodes)})),this}}),E.expr.pseudos.hidden=function(e){return!E.expr.pseudos.visible(e)},E.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},E.ajaxSettings.xhr=function(){try{return new n.XMLHttpRequest}catch(e){}};var Vt={0:200,1223:204},Gt=E.ajaxSettings.xhr();m.cors=!!Gt&&"withCredentials"in Gt,m.ajax=Gt=!!Gt,E.ajaxTransport((function(e){var t,r;if(m.cors||Gt&&!e.crossDomain)return{send:function(i,o){var s,a=e.xhr();if(a.open(e.type,e.url,e.async,e.username,e.password),e.xhrFields)for(s in e.xhrFields)a[s]=e.xhrFields[s];for(s in e.mimeType&&a.overrideMimeType&&a.overrideMimeType(e.mimeType),e.crossDomain||i["X-Requested-With"]||(i["X-Requested-With"]="XMLHttpRequest"),i)a.setRequestHeader(s,i[s]);t=function(e){return function(){t&&(t=r=a.onload=a.onerror=a.onabort=a.ontimeout=a.onreadystatechange=null,"abort"===e?a.abort():"error"===e?"number"!=typeof a.status?o(0,"error"):o(a.status,a.statusText):o(Vt[a.status]||a.status,a.statusText,"text"!==(a.responseType||"text")||"string"!=typeof a.responseText?{binary:a.response}:{text:a.responseText},a.getAllResponseHeaders()))}},a.onload=t(),r=a.onerror=a.ontimeout=t("error"),void 0!==a.onabort?a.onabort=r:a.onreadystatechange=function(){4===a.readyState&&n.setTimeout((function(){t&&r()}))},t=t("abort");try{a.send(e.hasContent&&e.data||null)}catch(e){if(t)throw e}},abort:function(){t&&t()}}})),E.ajaxPrefilter((function(e){e.crossDomain&&(e.contents.script=!1)})),E.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(e){return E.globalEval(e),e}}}),E.ajaxPrefilter("script",(function(e){void 0===e.cache&&(e.cache=!1),e.crossDomain&&(e.type="GET")})),E.ajaxTransport("script",(function(e){var t,n;if(e.crossDomain||e.scriptAttrs)return{send:function(r,i){t=E("<script>").attr(e.scriptAttrs||{}).prop({charset:e.scriptCharset,src:e.url}).on("load error",n=function(e){t.remove(),n=null,e&&i("error"===e.type?404:200,e.type)}),s.head.appendChild(t[0])},abort:function(){n&&n()}}}));var Kt,Yt=[],Zt=/(=)\?(?=&|$)|\?\?/;E.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=Yt.pop()||E.expando+"_"+At++;return this[e]=!0,e}}),E.ajaxPrefilter("json jsonp",(function(e,t,r){var i,o,s,a=!1!==e.jsonp&&(Zt.test(e.url)?"url":"string"==typeof e.data&&0===(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&Zt.test(e.data)&&"data");if(a||"jsonp"===e.dataTypes[0])return i=e.jsonpCallback=y(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,a?e[a]=e[a].replace(Zt,"$1"+i):!1!==e.jsonp&&(e.url+=(Dt.test(e.url)?"&":"?")+e.jsonp+"="+i),e.converters["script json"]=function(){return s||E.error(i+" was not called"),s[0]},e.dataTypes[0]="json",o=n[i],n[i]=function(){s=arguments},r.always((function(){void 0===o?E(n).removeProp(i):n[i]=o,e[i]&&(e.jsonpCallback=t.jsonpCallback,Yt.push(i)),s&&y(o)&&o(s[0]),s=o=void 0})),"script"})),m.createHTMLDocument=((Kt=s.implementation.createHTMLDocument("").body).innerHTML="<form></form><form></form>",2===Kt.childNodes.length),E.parseHTML=function(e,t,n){return"string"!=typeof e?[]:("boolean"==typeof t&&(n=t,t=!1),t||(m.createHTMLDocument?((r=(t=s.implementation.createHTMLDocument("")).createElement("base")).href=s.location.href,t.head.appendChild(r)):t=s),o=!n&&[],(i=R.exec(e))?[t.createElement(i[1])]:(i=Te([e],t,o),o&&o.length&&E(o).remove(),E.merge([],i.childNodes)));var r,i,o},E.fn.load=function(e,t,n){var r,i,o,s=this,a=e.indexOf(" ");return a>-1&&(r=bt(e.slice(a)),e=e.slice(0,a)),y(t)?(n=t,t=void 0):t&&"object"==typeof t&&(i="POST"),s.length>0&&E.ajax({url:e,type:i||"GET",dataType:"html",data:t}).done((function(e){o=arguments,s.html(r?E("<div>").append(E.parseHTML(e)).find(r):e)})).always(n&&function(e,t){s.each((function(){n.apply(this,o||[e.responseText,t,e])}))}),this},E.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],(function(e,t){E.fn[t]=function(e){return this.on(t,e)}})),E.expr.pseudos.animated=function(e){return E.grep(E.timers,(function(t){return e===t.elem})).length},E.offset={setOffset:function(e,t,n){var r,i,o,s,a,u,l=E.css(e,"position"),c=E(e),f={};"static"===l&&(e.style.position="relative"),a=c.offset(),o=E.css(e,"top"),u=E.css(e,"left"),("absolute"===l||"fixed"===l)&&(o+u).indexOf("auto")>-1?(s=(r=c.position()).top,i=r.left):(s=parseFloat(o)||0,i=parseFloat(u)||0),y(t)&&(t=t.call(e,n,E.extend({},a))),null!=t.top&&(f.top=t.top-a.top+s),null!=t.left&&(f.left=t.left-a.left+i),"using"in t?t.using.call(e,f):c.css(f)}},E.fn.extend({offset:function(e){if(arguments.length)return void 0===e?this:this.each((function(t){E.offset.setOffset(this,e,t)}));var t,n,r=this[0];return r?r.getClientRects().length?(t=r.getBoundingClientRect(),n=r.ownerDocument.defaultView,{top:t.top+n.pageYOffset,left:t.left+n.pageXOffset}):{top:0,left:0}:void 0},position:function(){if(this[0]){var e,t,n,r=this[0],i={top:0,left:0};if("fixed"===E.css(r,"position"))t=r.getBoundingClientRect();else{for(t=this.offset(),n=r.ownerDocument,e=r.offsetParent||n.documentElement;e&&(e===n.body||e===n.documentElement)&&"static"===E.css(e,"position");)e=e.parentNode;e&&e!==r&&1===e.nodeType&&((i=E(e).offset()).top+=E.css(e,"borderTopWidth",!0),i.left+=E.css(e,"borderLeftWidth",!0))}return{top:t.top-i.top-E.css(r,"marginTop",!0),left:t.left-i.left-E.css(r,"marginLeft",!0)}}},offsetParent:function(){return this.map((function(){for(var e=this.offsetParent;e&&"static"===E.css(e,"position");)e=e.offsetParent;return e||se}))}}),E.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},(function(e,t){var n="pageYOffset"===t;E.fn[e]=function(r){return $(this,(function(e,r,i){var o;if(_(e)?o=e:9===e.nodeType&&(o=e.defaultView),void 0===i)return o?o[t]:e[r];o?o.scrollTo(n?o.pageXOffset:i,n?i:o.pageYOffset):e[r]=i}),e,r,arguments.length)}})),E.each(["top","left"],(function(e,t){E.cssHooks[t]=Ge(m.pixelPosition,(function(e,n){if(n)return n=Ve(e,t),Ue.test(n)?E(e).position()[t]+"px":n}))})),E.each({Height:"height",Width:"width"},(function(e,t){E.each({padding:"inner"+e,content:t,"":"outer"+e},(function(n,r){E.fn[r]=function(i,o){var s=arguments.length&&(n||"boolean"!=typeof i),a=n||(!0===i||!0===o?"margin":"border");return $(this,(function(t,n,i){var o;return _(t)?0===r.indexOf("outer")?t["inner"+e]:t.document.documentElement["client"+e]:9===t.nodeType?(o=t.documentElement,Math.max(t.body["scroll"+e],o["scroll"+e],t.body["offset"+e],o["offset"+e],o["client"+e])):void 0===i?E.css(t,n,a):E.style(t,n,i,a)}),t,s?i:void 0,s)}}))})),E.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),(function(e,t){E.fn[t]=function(e,n){return arguments.length>0?this.on(t,null,e,n):this.trigger(t)}})),E.fn.extend({hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),E.fn.extend({bind:function(e,t,n){return this.on(e,null,t,n)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,n,r){return this.on(t,e,n,r)},undelegate:function(e,t,n){return 1===arguments.length?this.off(e,"**"):this.off(t,e||"**",n)}}),E.proxy=function(e,t){var n,r,i;if("string"==typeof t&&(n=e[t],t=e,e=n),y(e))return r=u.call(arguments,2),(i=function(){return e.apply(t||this,r.concat(u.call(arguments)))}).guid=e.guid=e.guid||E.guid++,i},E.holdReady=function(e){e?E.readyWait++:E.ready(!0)},E.isArray=Array.isArray,E.parseJSON=JSON.parse,E.nodeName=k,E.isFunction=y,E.isWindow=_,E.camelCase=K,E.type=x,E.now=Date.now,E.isNumeric=function(e){var t=E.type(e);return("number"===t||"string"===t)&&!isNaN(e-parseFloat(e))},void 0===(r=function(){return E}.apply(t,[]))||(e.exports=r);var Jt=n.jQuery,Qt=n.$;return E.noConflict=function(e){return n.$===E&&(n.$=Qt),e&&n.jQuery===E&&(n.jQuery=Jt),E},i||(n.jQuery=n.$=E),E}))},function(e,t,n){var r,i,o;i=[n(0)],void 0===(o="function"==typeof(r=function(e){return e.ui=e.ui||{},e.ui.version="1.12.1"})?r.apply(t,i):r)||(e.exports=o)},function(e,t){e.exports=function(e){this.$el=e}},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Widget 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(0),n(1)],void 0===(o="function"==typeof(r=function(e){var t,n=0,r=Array.prototype.slice;return e.cleanData=(t=e.cleanData,function(n){var r,i,o;for(o=0;null!=(i=n[o]);o++)try{(r=e._data(i,"events"))&&r.remove&&e(i).triggerHandler("remove")}catch(e){}t(n)}),e.widget=function(t,n,r){var i,o,s,a={},u=t.split(".")[0],l=u+"-"+(t=t.split(".")[1]);return r||(r=n,n=e.Widget),e.isArray(r)&&(r=e.extend.apply(null,[{}].concat(r))),e.expr[":"][l.toLowerCase()]=function(t){return!!e.data(t,l)},e[u]=e[u]||{},i=e[u][t],o=e[u][t]=function(e,t){if(!this._createWidget)return new o(e,t);arguments.length&&this._createWidget(e,t)},e.extend(o,i,{version:r.version,_proto:e.extend({},r),_childConstructors:[]}),(s=new n).options=e.widget.extend({},s.options),e.each(r,(function(t,r){e.isFunction(r)?a[t]=function(){function e(){return n.prototype[t].apply(this,arguments)}function i(e){return n.prototype[t].apply(this,e)}return function(){var t,n=this._super,o=this._superApply;return this._super=e,this._superApply=i,t=r.apply(this,arguments),this._super=n,this._superApply=o,t}}():a[t]=r})),o.prototype=e.widget.extend(s,{widgetEventPrefix:i&&s.widgetEventPrefix||t},a,{constructor:o,namespace:u,widgetName:t,widgetFullName:l}),i?(e.each(i._childConstructors,(function(t,n){var r=n.prototype;e.widget(r.namespace+"."+r.widgetName,o,n._proto)})),delete i._childConstructors):n._childConstructors.push(o),e.widget.bridge(t,o),o},e.widget.extend=function(t){for(var n,i,o=r.call(arguments,1),s=0,a=o.length;s<a;s++)for(n in o[s])i=o[s][n],o[s].hasOwnProperty(n)&&void 0!==i&&(e.isPlainObject(i)?t[n]=e.isPlainObject(t[n])?e.widget.extend({},t[n],i):e.widget.extend({},i):t[n]=i);return t},e.widget.bridge=function(t,n){var i=n.prototype.widgetFullName||t;e.fn[t]=function(o){var s="string"==typeof o,a=r.call(arguments,1),u=this;return s?this.length||"instance"!==o?this.each((function(){var n,r=e.data(this,i);return"instance"===o?(u=r,!1):r?e.isFunction(r[o])&&"_"!==o.charAt(0)?(n=r[o].apply(r,a))!==r&&void 0!==n?(u=n&&n.jquery?u.pushStack(n.get()):n,!1):void 0:e.error("no such method '"+o+"' for "+t+" widget instance"):e.error("cannot call methods on "+t+" prior to initialization; attempted to call method '"+o+"'")})):u=void 0:(a.length&&(o=e.widget.extend.apply(null,[o].concat(a))),this.each((function(){var t=e.data(this,i);t?(t.option(o||{}),t._init&&t._init()):e.data(this,i,new n(o,this))}))),u}},e.Widget=function(){},e.Widget._childConstructors=[],e.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",defaultElement:"<div>",options:{classes:{},disabled:!1,create:null},_createWidget:function(t,r){r=e(r||this.defaultElement||this)[0],this.element=e(r),this.uuid=n++,this.eventNamespace="."+this.widgetName+this.uuid,this.bindings=e(),this.hoverable=e(),this.focusable=e(),this.classesElementLookup={},r!==this&&(e.data(r,this.widgetFullName,this),this._on(!0,this.element,{remove:function(e){e.target===r&&this.destroy()}}),this.document=e(r.style?r.ownerDocument:r.document||r),this.window=e(this.document[0].defaultView||this.document[0].parentWindow)),this.options=e.widget.extend({},this.options,this._getCreateOptions(),t),this._create(),this.options.disabled&&this._setOptionDisabled(this.options.disabled),this._trigger("create",null,this._getCreateEventData()),this._init()},_getCreateOptions:function(){return{}},_getCreateEventData:e.noop,_create:e.noop,_init:e.noop,destroy:function(){var t=this;this._destroy(),e.each(this.classesElementLookup,(function(e,n){t._removeClass(n,e)})),this.element.off(this.eventNamespace).removeData(this.widgetFullName),this.widget().off(this.eventNamespace).removeAttr("aria-disabled"),this.bindings.off(this.eventNamespace)},_destroy:e.noop,widget:function(){return this.element},option:function(t,n){var r,i,o,s=t;if(0===arguments.length)return e.widget.extend({},this.options);if("string"==typeof t)if(s={},r=t.split("."),t=r.shift(),r.length){for(i=s[t]=e.widget.extend({},this.options[t]),o=0;o<r.length-1;o++)i[r[o]]=i[r[o]]||{},i=i[r[o]];if(t=r.pop(),1===arguments.length)return void 0===i[t]?null:i[t];i[t]=n}else{if(1===arguments.length)return void 0===this.options[t]?null:this.options[t];s[t]=n}return this._setOptions(s),this},_setOptions:function(e){var t;for(t in e)this._setOption(t,e[t]);return this},_setOption:function(e,t){return"classes"===e&&this._setOptionClasses(t),this.options[e]=t,"disabled"===e&&this._setOptionDisabled(t),this},_setOptionClasses:function(t){var n,r,i;for(n in t)i=this.classesElementLookup[n],t[n]!==this.options.classes[n]&&i&&i.length&&(r=e(i.get()),this._removeClass(i,n),r.addClass(this._classes({element:r,keys:n,classes:t,add:!0})))},_setOptionDisabled:function(e){this._toggleClass(this.widget(),this.widgetFullName+"-disabled",null,!!e),e&&(this._removeClass(this.hoverable,null,"ui-state-hover"),this._removeClass(this.focusable,null,"ui-state-focus"))},enable:function(){return this._setOptions({disabled:!1})},disable:function(){return this._setOptions({disabled:!0})},_classes:function(t){var n=[],r=this;function i(i,o){var s,a;for(a=0;a<i.length;a++)s=r.classesElementLookup[i[a]]||e(),s=t.add?e(e.unique(s.get().concat(t.element.get()))):e(s.not(t.element).get()),r.classesElementLookup[i[a]]=s,n.push(i[a]),o&&t.classes[i[a]]&&n.push(t.classes[i[a]])}return t=e.extend({element:this.element,classes:this.options.classes||{}},t),this._on(t.element,{remove:"_untrackClassesElement"}),t.keys&&i(t.keys.match(/\S+/g)||[],!0),t.extra&&i(t.extra.match(/\S+/g)||[]),n.join(" ")},_untrackClassesElement:function(t){var n=this;e.each(n.classesElementLookup,(function(r,i){-1!==e.inArray(t.target,i)&&(n.classesElementLookup[r]=e(i.not(t.target).get()))}))},_removeClass:function(e,t,n){return this._toggleClass(e,t,n,!1)},_addClass:function(e,t,n){return this._toggleClass(e,t,n,!0)},_toggleClass:function(e,t,n,r){r="boolean"==typeof r?r:n;var i="string"==typeof e||null===e,o={extra:i?t:n,keys:i?e:t,element:i?this.element:e,add:r};return o.element.toggleClass(this._classes(o),r),this},_on:function(t,n,r){var i,o=this;"boolean"!=typeof t&&(r=n,n=t,t=!1),r?(n=i=e(n),this.bindings=this.bindings.add(n)):(r=n,n=this.element,i=this.widget()),e.each(r,(function(r,s){function a(){if(t||!0!==o.options.disabled&&!e(this).hasClass("ui-state-disabled"))return("string"==typeof s?o[s]:s).apply(o,arguments)}"string"!=typeof s&&(a.guid=s.guid=s.guid||a.guid||e.guid++);var u=r.match(/^([\w:-]*)\s*(.*)$/),l=u[1]+o.eventNamespace,c=u[2];c?i.on(l,c,a):n.on(l,a)}))},_off:function(t,n){n=(n||"").split(" ").join(this.eventNamespace+" ")+this.eventNamespace,t.off(n).off(n),this.bindings=e(this.bindings.not(t).get()),this.focusable=e(this.focusable.not(t).get()),this.hoverable=e(this.hoverable.not(t).get())},_delay:function(e,t){var n=this;return setTimeout((function(){return("string"==typeof e?n[e]:e).apply(n,arguments)}),t||0)},_hoverable:function(t){this.hoverable=this.hoverable.add(t),this._on(t,{mouseenter:function(t){this._addClass(e(t.currentTarget),null,"ui-state-hover")},mouseleave:function(t){this._removeClass(e(t.currentTarget),null,"ui-state-hover")}})},_focusable:function(t){this.focusable=this.focusable.add(t),this._on(t,{focusin:function(t){this._addClass(e(t.currentTarget),null,"ui-state-focus")},focusout:function(t){this._removeClass(e(t.currentTarget),null,"ui-state-focus")}})},_trigger:function(t,n,r){var i,o,s=this.options[t];if(r=r||{},(n=e.Event(n)).type=(t===this.widgetEventPrefix?t:this.widgetEventPrefix+t).toLowerCase(),n.target=this.element[0],o=n.originalEvent)for(i in o)i in n||(n[i]=o[i]);return this.element.trigger(n,r),!(e.isFunction(s)&&!1===s.apply(this.element[0],[n].concat(r))||n.isDefaultPrevented())}},e.each({show:"fadeIn",hide:"fadeOut"},(function(t,n){e.Widget.prototype["_"+t]=function(r,i,o){var s;"string"==typeof i&&(i={effect:i});var a=i?!0===i||"number"==typeof i?n:i.effect||n:t;"number"==typeof(i=i||{})&&(i={duration:i}),s=!e.isEmptyObject(i),i.complete=o,i.delay&&r.delay(i.delay),s&&e.effects&&e.effects.effect[a]?r[t](i):a!==t&&r[a]?r[a](i.duration,i.easing,o):r.queue((function(n){e(this)[t](),o&&o.call(r[0]),n()}))}})),e.widget})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){e.exports=n(5),e.exports.version=n(22).version},function(e,t,n){var r=n(6),i=n(7),o=n(0),s=n(10);var a=r.DOMWidgetView.extend({initialize:function(e){a.__super__.initialize.apply(this,[e]),this.model.on("msg:custom",this.on_msg,this),this.model.on("change:width",this.size_changed,this),this.model.on("change:height",this.size_changed,this)},render:function(){var e=this,t=o("<canvas></canvas>");t.css("background-color","#000"),t.attr("tabindex","1"),this.$el.append(t),this.$canvas=t,this.c=s.init(t),this.c.on_resize((function(t){e.model.set("width",t.size[0]),e.model.set("height",t.size[1]),e.touch()})),this.c.on_event_tick((function(){var t=e.c.event_queue.get();if(e.c.event_queue.clear(),t.length>0){var n={msg_type:"events",contents:t};e.send(n)}})),s.start_event_loop();this.send({msg_type:"init"}),this.size_changed(),this.c.resize(),this.c.resizable()},on_msg:function(e,t){if(null!=e&&"glir_commands"==e.msg_type){var n=e.commands;if("base64"==e.array_serialization)var r=e.buffers;else if("binary"==e.array_serialization){r=[];for(var i=0;i<t.length;i++)r[i]={storage_type:"binary",buffer:t[i]}}var o=function(e,t){for(var n=0;n<e.length;n++){var r=e[n];if("DATA"==r[0]){var i=r[3].buffer_index,o=r[3].buffer_shape,s=r[3].buffer_dtype;r[3]=t[i],r[3].shape=o,r[3].dtype=s}}return e}(n,r);for(i=0;i<o.length;i++){var s=n[i];this.c.command(s)}}},size_changed:function(){var e=[this.model.get("width"),this.model.get("height")];this.$canvas.css("width",e[0]+"px"),this.$canvas.css("height",e[1]+"px")},remove:function(){s.unregister(this.c),this.send({msg_type:"status",contents:"removed"})}}),u=r.DOMWidgetModel.extend({defaults:i.extend(r.DOMWidgetModel.prototype.defaults(),{_model_name:"VispyModel",_view_name:"VispyView",_model_module:"vispy",_view_module:"vispy",_model_module_version:"~0.2.0",_view_module_version:"~0.2.0"})});e.exports={VispyView:a,VispyModel:u}},function(t,n){t.exports=e},function(e,t,n){(function(e,r){var i;
/**
 * @license
 * Lodash <https://lodash.com/>
 * Copyright OpenJS Foundation and other contributors <https://openjsf.org/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */(function(){var o,s=200,a="Unsupported core-js use. Try https://npms.io/search?q=ponyfill.",u="Expected a function",l="__lodash_hash_undefined__",c=500,f="__lodash_placeholder__",h=1,p=2,d=4,g=1,v=2,m=1,y=2,_=4,b=8,w=16,x=32,E=64,T=128,C=256,S=512,A=30,D="...",z=800,k=16,R=1,N=2,j=1/0,O=9007199254740991,F=17976931348623157e292,L=NaN,P=4294967295,q=P-1,M=P>>>1,I=[["ary",T],["bind",m],["bindKey",y],["curry",b],["curryRight",w],["flip",S],["partial",x],["partialRight",E],["rearg",C]],B="[object Arguments]",W="[object Array]",H="[object AsyncFunction]",U="[object Boolean]",$="[object Date]",X="[object DOMException]",V="[object Error]",G="[object Function]",K="[object GeneratorFunction]",Y="[object Map]",Z="[object Number]",J="[object Null]",Q="[object Object]",ee="[object Proxy]",te="[object RegExp]",ne="[object Set]",re="[object String]",ie="[object Symbol]",oe="[object Undefined]",se="[object WeakMap]",ae="[object WeakSet]",ue="[object ArrayBuffer]",le="[object DataView]",ce="[object Float32Array]",fe="[object Float64Array]",he="[object Int8Array]",pe="[object Int16Array]",de="[object Int32Array]",ge="[object Uint8Array]",ve="[object Uint8ClampedArray]",me="[object Uint16Array]",ye="[object Uint32Array]",_e=/\b__p \+= '';/g,be=/\b(__p \+=) '' \+/g,we=/(__e\(.*?\)|\b__t\)) \+\n'';/g,xe=/&(?:amp|lt|gt|quot|#39);/g,Ee=/[&<>"']/g,Te=RegExp(xe.source),Ce=RegExp(Ee.source),Se=/<%-([\s\S]+?)%>/g,Ae=/<%([\s\S]+?)%>/g,De=/<%=([\s\S]+?)%>/g,ze=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,ke=/^\w*$/,Re=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,Ne=/[\\^$.*+?()[\]{}|]/g,je=RegExp(Ne.source),Oe=/^\s+|\s+$/g,Fe=/^\s+/,Le=/\s+$/,Pe=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,qe=/\{\n\/\* \[wrapped with (.+)\] \*/,Me=/,? & /,Ie=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,Be=/\\(\\)?/g,We=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,He=/\w*$/,Ue=/^[-+]0x[0-9a-f]+$/i,$e=/^0b[01]+$/i,Xe=/^\[object .+?Constructor\]$/,Ve=/^0o[0-7]+$/i,Ge=/^(?:0|[1-9]\d*)$/,Ke=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,Ye=/($^)/,Ze=/['\n\r\u2028\u2029\\]/g,Je="\\u0300-\\u036f\\ufe20-\\ufe2f\\u20d0-\\u20ff",Qe="\\xac\\xb1\\xd7\\xf7\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf\\u2000-\\u206f \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",et="[\\ud800-\\udfff]",tt="["+Qe+"]",nt="["+Je+"]",rt="\\d+",it="[\\u2700-\\u27bf]",ot="[a-z\\xdf-\\xf6\\xf8-\\xff]",st="[^\\ud800-\\udfff"+Qe+rt+"\\u2700-\\u27bfa-z\\xdf-\\xf6\\xf8-\\xffA-Z\\xc0-\\xd6\\xd8-\\xde]",at="\\ud83c[\\udffb-\\udfff]",ut="[^\\ud800-\\udfff]",lt="(?:\\ud83c[\\udde6-\\uddff]){2}",ct="[\\ud800-\\udbff][\\udc00-\\udfff]",ft="[A-Z\\xc0-\\xd6\\xd8-\\xde]",ht="(?:"+ot+"|"+st+")",pt="(?:"+ft+"|"+st+")",dt="(?:"+nt+"|"+at+")"+"?",gt="[\\ufe0e\\ufe0f]?"+dt+("(?:\\u200d(?:"+[ut,lt,ct].join("|")+")[\\ufe0e\\ufe0f]?"+dt+")*"),vt="(?:"+[it,lt,ct].join("|")+")"+gt,mt="(?:"+[ut+nt+"?",nt,lt,ct,et].join("|")+")",yt=RegExp("['’]","g"),_t=RegExp(nt,"g"),bt=RegExp(at+"(?="+at+")|"+mt+gt,"g"),wt=RegExp([ft+"?"+ot+"+(?:['’](?:d|ll|m|re|s|t|ve))?(?="+[tt,ft,"$"].join("|")+")",pt+"+(?:['’](?:D|LL|M|RE|S|T|VE))?(?="+[tt,ft+ht,"$"].join("|")+")",ft+"?"+ht+"+(?:['’](?:d|ll|m|re|s|t|ve))?",ft+"+(?:['’](?:D|LL|M|RE|S|T|VE))?","\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])","\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",rt,vt].join("|"),"g"),xt=RegExp("[\\u200d\\ud800-\\udfff"+Je+"\\ufe0e\\ufe0f]"),Et=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,Tt=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],Ct=-1,St={};St[ce]=St[fe]=St[he]=St[pe]=St[de]=St[ge]=St[ve]=St[me]=St[ye]=!0,St[B]=St[W]=St[ue]=St[U]=St[le]=St[$]=St[V]=St[G]=St[Y]=St[Z]=St[Q]=St[te]=St[ne]=St[re]=St[se]=!1;var At={};At[B]=At[W]=At[ue]=At[le]=At[U]=At[$]=At[ce]=At[fe]=At[he]=At[pe]=At[de]=At[Y]=At[Z]=At[Q]=At[te]=At[ne]=At[re]=At[ie]=At[ge]=At[ve]=At[me]=At[ye]=!0,At[V]=At[G]=At[se]=!1;var Dt={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},zt=parseFloat,kt=parseInt,Rt="object"==typeof e&&e&&e.Object===Object&&e,Nt="object"==typeof self&&self&&self.Object===Object&&self,jt=Rt||Nt||Function("return this")(),Ot=t&&!t.nodeType&&t,Ft=Ot&&"object"==typeof r&&r&&!r.nodeType&&r,Lt=Ft&&Ft.exports===Ot,Pt=Lt&&Rt.process,qt=function(){try{var e=Ft&&Ft.require&&Ft.require("util").types;return e||Pt&&Pt.binding&&Pt.binding("util")}catch(e){}}(),Mt=qt&&qt.isArrayBuffer,It=qt&&qt.isDate,Bt=qt&&qt.isMap,Wt=qt&&qt.isRegExp,Ht=qt&&qt.isSet,Ut=qt&&qt.isTypedArray;function $t(e,t,n){switch(n.length){case 0:return e.call(t);case 1:return e.call(t,n[0]);case 2:return e.call(t,n[0],n[1]);case 3:return e.call(t,n[0],n[1],n[2])}return e.apply(t,n)}function Xt(e,t,n,r){for(var i=-1,o=null==e?0:e.length;++i<o;){var s=e[i];t(r,s,n(s),e)}return r}function Vt(e,t){for(var n=-1,r=null==e?0:e.length;++n<r&&!1!==t(e[n],n,e););return e}function Gt(e,t){for(var n=null==e?0:e.length;n--&&!1!==t(e[n],n,e););return e}function Kt(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(!t(e[n],n,e))return!1;return!0}function Yt(e,t){for(var n=-1,r=null==e?0:e.length,i=0,o=[];++n<r;){var s=e[n];t(s,n,e)&&(o[i++]=s)}return o}function Zt(e,t){return!!(null==e?0:e.length)&&un(e,t,0)>-1}function Jt(e,t,n){for(var r=-1,i=null==e?0:e.length;++r<i;)if(n(t,e[r]))return!0;return!1}function Qt(e,t){for(var n=-1,r=null==e?0:e.length,i=Array(r);++n<r;)i[n]=t(e[n],n,e);return i}function en(e,t){for(var n=-1,r=t.length,i=e.length;++n<r;)e[i+n]=t[n];return e}function tn(e,t,n,r){var i=-1,o=null==e?0:e.length;for(r&&o&&(n=e[++i]);++i<o;)n=t(n,e[i],i,e);return n}function nn(e,t,n,r){var i=null==e?0:e.length;for(r&&i&&(n=e[--i]);i--;)n=t(n,e[i],i,e);return n}function rn(e,t){for(var n=-1,r=null==e?0:e.length;++n<r;)if(t(e[n],n,e))return!0;return!1}var on=hn("length");function sn(e,t,n){var r;return n(e,(function(e,n,i){if(t(e,n,i))return r=n,!1})),r}function an(e,t,n,r){for(var i=e.length,o=n+(r?1:-1);r?o--:++o<i;)if(t(e[o],o,e))return o;return-1}function un(e,t,n){return t==t?function(e,t,n){var r=n-1,i=e.length;for(;++r<i;)if(e[r]===t)return r;return-1}(e,t,n):an(e,cn,n)}function ln(e,t,n,r){for(var i=n-1,o=e.length;++i<o;)if(r(e[i],t))return i;return-1}function cn(e){return e!=e}function fn(e,t){var n=null==e?0:e.length;return n?gn(e,t)/n:L}function hn(e){return function(t){return null==t?o:t[e]}}function pn(e){return function(t){return null==e?o:e[t]}}function dn(e,t,n,r,i){return i(e,(function(e,i,o){n=r?(r=!1,e):t(n,e,i,o)})),n}function gn(e,t){for(var n,r=-1,i=e.length;++r<i;){var s=t(e[r]);s!==o&&(n=n===o?s:n+s)}return n}function vn(e,t){for(var n=-1,r=Array(e);++n<e;)r[n]=t(n);return r}function mn(e){return function(t){return e(t)}}function yn(e,t){return Qt(t,(function(t){return e[t]}))}function _n(e,t){return e.has(t)}function bn(e,t){for(var n=-1,r=e.length;++n<r&&un(t,e[n],0)>-1;);return n}function wn(e,t){for(var n=e.length;n--&&un(t,e[n],0)>-1;);return n}var xn=pn({"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"}),En=pn({"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"});function Tn(e){return"\\"+Dt[e]}function Cn(e){return xt.test(e)}function Sn(e){var t=-1,n=Array(e.size);return e.forEach((function(e,r){n[++t]=[r,e]})),n}function An(e,t){return function(n){return e(t(n))}}function Dn(e,t){for(var n=-1,r=e.length,i=0,o=[];++n<r;){var s=e[n];s!==t&&s!==f||(e[n]=f,o[i++]=n)}return o}function zn(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=e})),n}function kn(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=[e,e]})),n}function Rn(e){return Cn(e)?function(e){var t=bt.lastIndex=0;for(;bt.test(e);)++t;return t}(e):on(e)}function Nn(e){return Cn(e)?function(e){return e.match(bt)||[]}(e):function(e){return e.split("")}(e)}var jn=pn({"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"});var On=function e(t){var n,r=(t=null==t?jt:On.defaults(jt.Object(),t,On.pick(jt,Tt))).Array,i=t.Date,Je=t.Error,Qe=t.Function,et=t.Math,tt=t.Object,nt=t.RegExp,rt=t.String,it=t.TypeError,ot=r.prototype,st=Qe.prototype,at=tt.prototype,ut=t["__core-js_shared__"],lt=st.toString,ct=at.hasOwnProperty,ft=0,ht=(n=/[^.]+$/.exec(ut&&ut.keys&&ut.keys.IE_PROTO||""))?"Symbol(src)_1."+n:"",pt=at.toString,dt=lt.call(tt),gt=jt._,vt=nt("^"+lt.call(ct).replace(Ne,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),mt=Lt?t.Buffer:o,bt=t.Symbol,xt=t.Uint8Array,Dt=mt?mt.allocUnsafe:o,Rt=An(tt.getPrototypeOf,tt),Nt=tt.create,Ot=at.propertyIsEnumerable,Ft=ot.splice,Pt=bt?bt.isConcatSpreadable:o,qt=bt?bt.iterator:o,on=bt?bt.toStringTag:o,pn=function(){try{var e=Mo(tt,"defineProperty");return e({},"",{}),e}catch(e){}}(),Fn=t.clearTimeout!==jt.clearTimeout&&t.clearTimeout,Ln=i&&i.now!==jt.Date.now&&i.now,Pn=t.setTimeout!==jt.setTimeout&&t.setTimeout,qn=et.ceil,Mn=et.floor,In=tt.getOwnPropertySymbols,Bn=mt?mt.isBuffer:o,Wn=t.isFinite,Hn=ot.join,Un=An(tt.keys,tt),$n=et.max,Xn=et.min,Vn=i.now,Gn=t.parseInt,Kn=et.random,Yn=ot.reverse,Zn=Mo(t,"DataView"),Jn=Mo(t,"Map"),Qn=Mo(t,"Promise"),er=Mo(t,"Set"),tr=Mo(t,"WeakMap"),nr=Mo(tt,"create"),rr=tr&&new tr,ir={},or=fs(Zn),sr=fs(Jn),ar=fs(Qn),ur=fs(er),lr=fs(tr),cr=bt?bt.prototype:o,fr=cr?cr.valueOf:o,hr=cr?cr.toString:o;function pr(e){if(Da(e)&&!ma(e)&&!(e instanceof mr)){if(e instanceof vr)return e;if(ct.call(e,"__wrapped__"))return hs(e)}return new vr(e)}var dr=function(){function e(){}return function(t){if(!Aa(t))return{};if(Nt)return Nt(t);e.prototype=t;var n=new e;return e.prototype=o,n}}();function gr(){}function vr(e,t){this.__wrapped__=e,this.__actions__=[],this.__chain__=!!t,this.__index__=0,this.__values__=o}function mr(e){this.__wrapped__=e,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=P,this.__views__=[]}function yr(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function _r(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function br(e){var t=-1,n=null==e?0:e.length;for(this.clear();++t<n;){var r=e[t];this.set(r[0],r[1])}}function wr(e){var t=-1,n=null==e?0:e.length;for(this.__data__=new br;++t<n;)this.add(e[t])}function xr(e){var t=this.__data__=new _r(e);this.size=t.size}function Er(e,t){var n=ma(e),r=!n&&va(e),i=!n&&!r&&wa(e),o=!n&&!r&&!i&&La(e),s=n||r||i||o,a=s?vn(e.length,rt):[],u=a.length;for(var l in e)!t&&!ct.call(e,l)||s&&("length"==l||i&&("offset"==l||"parent"==l)||o&&("buffer"==l||"byteLength"==l||"byteOffset"==l)||Xo(l,u))||a.push(l);return a}function Tr(e){var t=e.length;return t?e[wi(0,t-1)]:o}function Cr(e,t){return us(no(e),Or(t,0,e.length))}function Sr(e){return us(no(e))}function Ar(e,t,n){(n===o||pa(e[t],n))&&(n!==o||t in e)||Nr(e,t,n)}function Dr(e,t,n){var r=e[t];ct.call(e,t)&&pa(r,n)&&(n!==o||t in e)||Nr(e,t,n)}function zr(e,t){for(var n=e.length;n--;)if(pa(e[n][0],t))return n;return-1}function kr(e,t,n,r){return Mr(e,(function(e,i,o){t(r,e,n(e),o)})),r}function Rr(e,t){return e&&ro(t,iu(t),e)}function Nr(e,t,n){"__proto__"==t&&pn?pn(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}function jr(e,t){for(var n=-1,i=t.length,s=r(i),a=null==e;++n<i;)s[n]=a?o:Qa(e,t[n]);return s}function Or(e,t,n){return e==e&&(n!==o&&(e=e<=n?e:n),t!==o&&(e=e>=t?e:t)),e}function Fr(e,t,n,r,i,s){var a,u=t&h,l=t&p,c=t&d;if(n&&(a=i?n(e,r,i,s):n(e)),a!==o)return a;if(!Aa(e))return e;var f=ma(e);if(f){if(a=function(e){var t=e.length,n=new e.constructor(t);t&&"string"==typeof e[0]&&ct.call(e,"index")&&(n.index=e.index,n.input=e.input);return n}(e),!u)return no(e,a)}else{var g=Wo(e),v=g==G||g==K;if(wa(e))return Yi(e,u);if(g==Q||g==B||v&&!i){if(a=l||v?{}:Uo(e),!u)return l?function(e,t){return ro(e,Bo(e),t)}(e,function(e,t){return e&&ro(t,ou(t),e)}(a,e)):function(e,t){return ro(e,Io(e),t)}(e,Rr(a,e))}else{if(!At[g])return i?e:{};a=function(e,t,n){var r=e.constructor;switch(t){case ue:return Zi(e);case U:case $:return new r(+e);case le:return function(e,t){var n=t?Zi(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.byteLength)}(e,n);case ce:case fe:case he:case pe:case de:case ge:case ve:case me:case ye:return Ji(e,n);case Y:return new r;case Z:case re:return new r(e);case te:return function(e){var t=new e.constructor(e.source,He.exec(e));return t.lastIndex=e.lastIndex,t}(e);case ne:return new r;case ie:return i=e,fr?tt(fr.call(i)):{}}var i}(e,g,u)}}s||(s=new xr);var m=s.get(e);if(m)return m;s.set(e,a),ja(e)?e.forEach((function(r){a.add(Fr(r,t,n,r,e,s))})):za(e)&&e.forEach((function(r,i){a.set(i,Fr(r,t,n,i,e,s))}));var y=f?o:(c?l?No:Ro:l?ou:iu)(e);return Vt(y||e,(function(r,i){y&&(r=e[i=r]),Dr(a,i,Fr(r,t,n,i,e,s))})),a}function Lr(e,t,n){var r=n.length;if(null==e)return!r;for(e=tt(e);r--;){var i=n[r],s=t[i],a=e[i];if(a===o&&!(i in e)||!s(a))return!1}return!0}function Pr(e,t,n){if("function"!=typeof e)throw new it(u);return is((function(){e.apply(o,n)}),t)}function qr(e,t,n,r){var i=-1,o=Zt,a=!0,u=e.length,l=[],c=t.length;if(!u)return l;n&&(t=Qt(t,mn(n))),r?(o=Jt,a=!1):t.length>=s&&(o=_n,a=!1,t=new wr(t));e:for(;++i<u;){var f=e[i],h=null==n?f:n(f);if(f=r||0!==f?f:0,a&&h==h){for(var p=c;p--;)if(t[p]===h)continue e;l.push(f)}else o(t,h,r)||l.push(f)}return l}pr.templateSettings={escape:Se,evaluate:Ae,interpolate:De,variable:"",imports:{_:pr}},pr.prototype=gr.prototype,pr.prototype.constructor=pr,vr.prototype=dr(gr.prototype),vr.prototype.constructor=vr,mr.prototype=dr(gr.prototype),mr.prototype.constructor=mr,yr.prototype.clear=function(){this.__data__=nr?nr(null):{},this.size=0},yr.prototype.delete=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t},yr.prototype.get=function(e){var t=this.__data__;if(nr){var n=t[e];return n===l?o:n}return ct.call(t,e)?t[e]:o},yr.prototype.has=function(e){var t=this.__data__;return nr?t[e]!==o:ct.call(t,e)},yr.prototype.set=function(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=nr&&t===o?l:t,this},_r.prototype.clear=function(){this.__data__=[],this.size=0},_r.prototype.delete=function(e){var t=this.__data__,n=zr(t,e);return!(n<0)&&(n==t.length-1?t.pop():Ft.call(t,n,1),--this.size,!0)},_r.prototype.get=function(e){var t=this.__data__,n=zr(t,e);return n<0?o:t[n][1]},_r.prototype.has=function(e){return zr(this.__data__,e)>-1},_r.prototype.set=function(e,t){var n=this.__data__,r=zr(n,e);return r<0?(++this.size,n.push([e,t])):n[r][1]=t,this},br.prototype.clear=function(){this.size=0,this.__data__={hash:new yr,map:new(Jn||_r),string:new yr}},br.prototype.delete=function(e){var t=Po(this,e).delete(e);return this.size-=t?1:0,t},br.prototype.get=function(e){return Po(this,e).get(e)},br.prototype.has=function(e){return Po(this,e).has(e)},br.prototype.set=function(e,t){var n=Po(this,e),r=n.size;return n.set(e,t),this.size+=n.size==r?0:1,this},wr.prototype.add=wr.prototype.push=function(e){return this.__data__.set(e,l),this},wr.prototype.has=function(e){return this.__data__.has(e)},xr.prototype.clear=function(){this.__data__=new _r,this.size=0},xr.prototype.delete=function(e){var t=this.__data__,n=t.delete(e);return this.size=t.size,n},xr.prototype.get=function(e){return this.__data__.get(e)},xr.prototype.has=function(e){return this.__data__.has(e)},xr.prototype.set=function(e,t){var n=this.__data__;if(n instanceof _r){var r=n.__data__;if(!Jn||r.length<s-1)return r.push([e,t]),this.size=++n.size,this;n=this.__data__=new br(r)}return n.set(e,t),this.size=n.size,this};var Mr=so(Vr),Ir=so(Gr,!0);function Br(e,t){var n=!0;return Mr(e,(function(e,r,i){return n=!!t(e,r,i)})),n}function Wr(e,t,n){for(var r=-1,i=e.length;++r<i;){var s=e[r],a=t(s);if(null!=a&&(u===o?a==a&&!Fa(a):n(a,u)))var u=a,l=s}return l}function Hr(e,t){var n=[];return Mr(e,(function(e,r,i){t(e,r,i)&&n.push(e)})),n}function Ur(e,t,n,r,i){var o=-1,s=e.length;for(n||(n=$o),i||(i=[]);++o<s;){var a=e[o];t>0&&n(a)?t>1?Ur(a,t-1,n,r,i):en(i,a):r||(i[i.length]=a)}return i}var $r=ao(),Xr=ao(!0);function Vr(e,t){return e&&$r(e,t,iu)}function Gr(e,t){return e&&Xr(e,t,iu)}function Kr(e,t){return Yt(t,(function(t){return Ta(e[t])}))}function Yr(e,t){for(var n=0,r=(t=Xi(t,e)).length;null!=e&&n<r;)e=e[cs(t[n++])];return n&&n==r?e:o}function Zr(e,t,n){var r=t(e);return ma(e)?r:en(r,n(e))}function Jr(e){return null==e?e===o?oe:J:on&&on in tt(e)?function(e){var t=ct.call(e,on),n=e[on];try{e[on]=o;var r=!0}catch(e){}var i=pt.call(e);r&&(t?e[on]=n:delete e[on]);return i}(e):function(e){return pt.call(e)}(e)}function Qr(e,t){return e>t}function ei(e,t){return null!=e&&ct.call(e,t)}function ti(e,t){return null!=e&&t in tt(e)}function ni(e,t,n){for(var i=n?Jt:Zt,s=e[0].length,a=e.length,u=a,l=r(a),c=1/0,f=[];u--;){var h=e[u];u&&t&&(h=Qt(h,mn(t))),c=Xn(h.length,c),l[u]=!n&&(t||s>=120&&h.length>=120)?new wr(u&&h):o}h=e[0];var p=-1,d=l[0];e:for(;++p<s&&f.length<c;){var g=h[p],v=t?t(g):g;if(g=n||0!==g?g:0,!(d?_n(d,v):i(f,v,n))){for(u=a;--u;){var m=l[u];if(!(m?_n(m,v):i(e[u],v,n)))continue e}d&&d.push(v),f.push(g)}}return f}function ri(e,t,n){var r=null==(e=ts(e,t=Xi(t,e)))?e:e[cs(Es(t))];return null==r?o:$t(r,e,n)}function ii(e){return Da(e)&&Jr(e)==B}function oi(e,t,n,r,i){return e===t||(null==e||null==t||!Da(e)&&!Da(t)?e!=e&&t!=t:function(e,t,n,r,i,s){var a=ma(e),u=ma(t),l=a?W:Wo(e),c=u?W:Wo(t),f=(l=l==B?Q:l)==Q,h=(c=c==B?Q:c)==Q,p=l==c;if(p&&wa(e)){if(!wa(t))return!1;a=!0,f=!1}if(p&&!f)return s||(s=new xr),a||La(e)?zo(e,t,n,r,i,s):function(e,t,n,r,i,o,s){switch(n){case le:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case ue:return!(e.byteLength!=t.byteLength||!o(new xt(e),new xt(t)));case U:case $:case Z:return pa(+e,+t);case V:return e.name==t.name&&e.message==t.message;case te:case re:return e==t+"";case Y:var a=Sn;case ne:var u=r&g;if(a||(a=zn),e.size!=t.size&&!u)return!1;var l=s.get(e);if(l)return l==t;r|=v,s.set(e,t);var c=zo(a(e),a(t),r,i,o,s);return s.delete(e),c;case ie:if(fr)return fr.call(e)==fr.call(t)}return!1}(e,t,l,n,r,i,s);if(!(n&g)){var d=f&&ct.call(e,"__wrapped__"),m=h&&ct.call(t,"__wrapped__");if(d||m){var y=d?e.value():e,_=m?t.value():t;return s||(s=new xr),i(y,_,n,r,s)}}if(!p)return!1;return s||(s=new xr),function(e,t,n,r,i,s){var a=n&g,u=Ro(e),l=u.length,c=Ro(t).length;if(l!=c&&!a)return!1;var f=l;for(;f--;){var h=u[f];if(!(a?h in t:ct.call(t,h)))return!1}var p=s.get(e);if(p&&s.get(t))return p==t;var d=!0;s.set(e,t),s.set(t,e);var v=a;for(;++f<l;){h=u[f];var m=e[h],y=t[h];if(r)var _=a?r(y,m,h,t,e,s):r(m,y,h,e,t,s);if(!(_===o?m===y||i(m,y,n,r,s):_)){d=!1;break}v||(v="constructor"==h)}if(d&&!v){var b=e.constructor,w=t.constructor;b!=w&&"constructor"in e&&"constructor"in t&&!("function"==typeof b&&b instanceof b&&"function"==typeof w&&w instanceof w)&&(d=!1)}return s.delete(e),s.delete(t),d}(e,t,n,r,i,s)}(e,t,n,r,oi,i))}function si(e,t,n,r){var i=n.length,s=i,a=!r;if(null==e)return!s;for(e=tt(e);i--;){var u=n[i];if(a&&u[2]?u[1]!==e[u[0]]:!(u[0]in e))return!1}for(;++i<s;){var l=(u=n[i])[0],c=e[l],f=u[1];if(a&&u[2]){if(c===o&&!(l in e))return!1}else{var h=new xr;if(r)var p=r(c,f,l,e,t,h);if(!(p===o?oi(f,c,g|v,r,h):p))return!1}}return!0}function ai(e){return!(!Aa(e)||(t=e,ht&&ht in t))&&(Ta(e)?vt:Xe).test(fs(e));var t}function ui(e){return"function"==typeof e?e:null==e?ku:"object"==typeof e?ma(e)?di(e[0],e[1]):pi(e):Mu(e)}function li(e){if(!Zo(e))return Un(e);var t=[];for(var n in tt(e))ct.call(e,n)&&"constructor"!=n&&t.push(n);return t}function ci(e){if(!Aa(e))return function(e){var t=[];if(null!=e)for(var n in tt(e))t.push(n);return t}(e);var t=Zo(e),n=[];for(var r in e)("constructor"!=r||!t&&ct.call(e,r))&&n.push(r);return n}function fi(e,t){return e<t}function hi(e,t){var n=-1,i=_a(e)?r(e.length):[];return Mr(e,(function(e,r,o){i[++n]=t(e,r,o)})),i}function pi(e){var t=qo(e);return 1==t.length&&t[0][2]?Qo(t[0][0],t[0][1]):function(n){return n===e||si(n,e,t)}}function di(e,t){return Go(e)&&Jo(t)?Qo(cs(e),t):function(n){var r=Qa(n,e);return r===o&&r===t?eu(n,e):oi(t,r,g|v)}}function gi(e,t,n,r,i){e!==t&&$r(t,(function(s,a){if(i||(i=new xr),Aa(s))!function(e,t,n,r,i,s,a){var u=ns(e,n),l=ns(t,n),c=a.get(l);if(c)return void Ar(e,n,c);var f=s?s(u,l,n+"",e,t,a):o,h=f===o;if(h){var p=ma(l),d=!p&&wa(l),g=!p&&!d&&La(l);f=l,p||d||g?ma(u)?f=u:ba(u)?f=no(u):d?(h=!1,f=Yi(l,!0)):g?(h=!1,f=Ji(l,!0)):f=[]:Ra(l)||va(l)?(f=u,va(u)?f=Ua(u):Aa(u)&&!Ta(u)||(f=Uo(l))):h=!1}h&&(a.set(l,f),i(f,l,r,s,a),a.delete(l));Ar(e,n,f)}(e,t,a,n,gi,r,i);else{var u=r?r(ns(e,a),s,a+"",e,t,i):o;u===o&&(u=s),Ar(e,a,u)}}),ou)}function vi(e,t){var n=e.length;if(n)return Xo(t+=t<0?n:0,n)?e[t]:o}function mi(e,t,n){var r=-1;return t=Qt(t.length?t:[ku],mn(Lo())),function(e,t){var n=e.length;for(e.sort(t);n--;)e[n]=e[n].value;return e}(hi(e,(function(e,n,i){return{criteria:Qt(t,(function(t){return t(e)})),index:++r,value:e}})),(function(e,t){return function(e,t,n){var r=-1,i=e.criteria,o=t.criteria,s=i.length,a=n.length;for(;++r<s;){var u=Qi(i[r],o[r]);if(u){if(r>=a)return u;var l=n[r];return u*("desc"==l?-1:1)}}return e.index-t.index}(e,t,n)}))}function yi(e,t,n){for(var r=-1,i=t.length,o={};++r<i;){var s=t[r],a=Yr(e,s);n(a,s)&&Si(o,Xi(s,e),a)}return o}function _i(e,t,n,r){var i=r?ln:un,o=-1,s=t.length,a=e;for(e===t&&(t=no(t)),n&&(a=Qt(e,mn(n)));++o<s;)for(var u=0,l=t[o],c=n?n(l):l;(u=i(a,c,u,r))>-1;)a!==e&&Ft.call(a,u,1),Ft.call(e,u,1);return e}function bi(e,t){for(var n=e?t.length:0,r=n-1;n--;){var i=t[n];if(n==r||i!==o){var o=i;Xo(i)?Ft.call(e,i,1):qi(e,i)}}return e}function wi(e,t){return e+Mn(Kn()*(t-e+1))}function xi(e,t){var n="";if(!e||t<1||t>O)return n;do{t%2&&(n+=e),(t=Mn(t/2))&&(e+=e)}while(t);return n}function Ei(e,t){return os(es(e,t,ku),e+"")}function Ti(e){return Tr(pu(e))}function Ci(e,t){var n=pu(e);return us(n,Or(t,0,n.length))}function Si(e,t,n,r){if(!Aa(e))return e;for(var i=-1,s=(t=Xi(t,e)).length,a=s-1,u=e;null!=u&&++i<s;){var l=cs(t[i]),c=n;if(i!=a){var f=u[l];(c=r?r(f,l,u):o)===o&&(c=Aa(f)?f:Xo(t[i+1])?[]:{})}Dr(u,l,c),u=u[l]}return e}var Ai=rr?function(e,t){return rr.set(e,t),e}:ku,Di=pn?function(e,t){return pn(e,"toString",{configurable:!0,enumerable:!1,value:Au(t),writable:!0})}:ku;function zi(e){return us(pu(e))}function ki(e,t,n){var i=-1,o=e.length;t<0&&(t=-t>o?0:o+t),(n=n>o?o:n)<0&&(n+=o),o=t>n?0:n-t>>>0,t>>>=0;for(var s=r(o);++i<o;)s[i]=e[i+t];return s}function Ri(e,t){var n;return Mr(e,(function(e,r,i){return!(n=t(e,r,i))})),!!n}function Ni(e,t,n){var r=0,i=null==e?r:e.length;if("number"==typeof t&&t==t&&i<=M){for(;r<i;){var o=r+i>>>1,s=e[o];null!==s&&!Fa(s)&&(n?s<=t:s<t)?r=o+1:i=o}return i}return ji(e,t,ku,n)}function ji(e,t,n,r){t=n(t);for(var i=0,s=null==e?0:e.length,a=t!=t,u=null===t,l=Fa(t),c=t===o;i<s;){var f=Mn((i+s)/2),h=n(e[f]),p=h!==o,d=null===h,g=h==h,v=Fa(h);if(a)var m=r||g;else m=c?g&&(r||p):u?g&&p&&(r||!d):l?g&&p&&!d&&(r||!v):!d&&!v&&(r?h<=t:h<t);m?i=f+1:s=f}return Xn(s,q)}function Oi(e,t){for(var n=-1,r=e.length,i=0,o=[];++n<r;){var s=e[n],a=t?t(s):s;if(!n||!pa(a,u)){var u=a;o[i++]=0===s?0:s}}return o}function Fi(e){return"number"==typeof e?e:Fa(e)?L:+e}function Li(e){if("string"==typeof e)return e;if(ma(e))return Qt(e,Li)+"";if(Fa(e))return hr?hr.call(e):"";var t=e+"";return"0"==t&&1/e==-j?"-0":t}function Pi(e,t,n){var r=-1,i=Zt,o=e.length,a=!0,u=[],l=u;if(n)a=!1,i=Jt;else if(o>=s){var c=t?null:Eo(e);if(c)return zn(c);a=!1,i=_n,l=new wr}else l=t?[]:u;e:for(;++r<o;){var f=e[r],h=t?t(f):f;if(f=n||0!==f?f:0,a&&h==h){for(var p=l.length;p--;)if(l[p]===h)continue e;t&&l.push(h),u.push(f)}else i(l,h,n)||(l!==u&&l.push(h),u.push(f))}return u}function qi(e,t){return null==(e=ts(e,t=Xi(t,e)))||delete e[cs(Es(t))]}function Mi(e,t,n,r){return Si(e,t,n(Yr(e,t)),r)}function Ii(e,t,n,r){for(var i=e.length,o=r?i:-1;(r?o--:++o<i)&&t(e[o],o,e););return n?ki(e,r?0:o,r?o+1:i):ki(e,r?o+1:0,r?i:o)}function Bi(e,t){var n=e;return n instanceof mr&&(n=n.value()),tn(t,(function(e,t){return t.func.apply(t.thisArg,en([e],t.args))}),n)}function Wi(e,t,n){var i=e.length;if(i<2)return i?Pi(e[0]):[];for(var o=-1,s=r(i);++o<i;)for(var a=e[o],u=-1;++u<i;)u!=o&&(s[o]=qr(s[o]||a,e[u],t,n));return Pi(Ur(s,1),t,n)}function Hi(e,t,n){for(var r=-1,i=e.length,s=t.length,a={};++r<i;){var u=r<s?t[r]:o;n(a,e[r],u)}return a}function Ui(e){return ba(e)?e:[]}function $i(e){return"function"==typeof e?e:ku}function Xi(e,t){return ma(e)?e:Go(e,t)?[e]:ls($a(e))}var Vi=Ei;function Gi(e,t,n){var r=e.length;return n=n===o?r:n,!t&&n>=r?e:ki(e,t,n)}var Ki=Fn||function(e){return jt.clearTimeout(e)};function Yi(e,t){if(t)return e.slice();var n=e.length,r=Dt?Dt(n):new e.constructor(n);return e.copy(r),r}function Zi(e){var t=new e.constructor(e.byteLength);return new xt(t).set(new xt(e)),t}function Ji(e,t){var n=t?Zi(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.length)}function Qi(e,t){if(e!==t){var n=e!==o,r=null===e,i=e==e,s=Fa(e),a=t!==o,u=null===t,l=t==t,c=Fa(t);if(!u&&!c&&!s&&e>t||s&&a&&l&&!u&&!c||r&&a&&l||!n&&l||!i)return 1;if(!r&&!s&&!c&&e<t||c&&n&&i&&!r&&!s||u&&n&&i||!a&&i||!l)return-1}return 0}function eo(e,t,n,i){for(var o=-1,s=e.length,a=n.length,u=-1,l=t.length,c=$n(s-a,0),f=r(l+c),h=!i;++u<l;)f[u]=t[u];for(;++o<a;)(h||o<s)&&(f[n[o]]=e[o]);for(;c--;)f[u++]=e[o++];return f}function to(e,t,n,i){for(var o=-1,s=e.length,a=-1,u=n.length,l=-1,c=t.length,f=$n(s-u,0),h=r(f+c),p=!i;++o<f;)h[o]=e[o];for(var d=o;++l<c;)h[d+l]=t[l];for(;++a<u;)(p||o<s)&&(h[d+n[a]]=e[o++]);return h}function no(e,t){var n=-1,i=e.length;for(t||(t=r(i));++n<i;)t[n]=e[n];return t}function ro(e,t,n,r){var i=!n;n||(n={});for(var s=-1,a=t.length;++s<a;){var u=t[s],l=r?r(n[u],e[u],u,n,e):o;l===o&&(l=e[u]),i?Nr(n,u,l):Dr(n,u,l)}return n}function io(e,t){return function(n,r){var i=ma(n)?Xt:kr,o=t?t():{};return i(n,e,Lo(r,2),o)}}function oo(e){return Ei((function(t,n){var r=-1,i=n.length,s=i>1?n[i-1]:o,a=i>2?n[2]:o;for(s=e.length>3&&"function"==typeof s?(i--,s):o,a&&Vo(n[0],n[1],a)&&(s=i<3?o:s,i=1),t=tt(t);++r<i;){var u=n[r];u&&e(t,u,r,s)}return t}))}function so(e,t){return function(n,r){if(null==n)return n;if(!_a(n))return e(n,r);for(var i=n.length,o=t?i:-1,s=tt(n);(t?o--:++o<i)&&!1!==r(s[o],o,s););return n}}function ao(e){return function(t,n,r){for(var i=-1,o=tt(t),s=r(t),a=s.length;a--;){var u=s[e?a:++i];if(!1===n(o[u],u,o))break}return t}}function uo(e){return function(t){var n=Cn(t=$a(t))?Nn(t):o,r=n?n[0]:t.charAt(0),i=n?Gi(n,1).join(""):t.slice(1);return r[e]()+i}}function lo(e){return function(t){return tn(Tu(vu(t).replace(yt,"")),e,"")}}function co(e){return function(){var t=arguments;switch(t.length){case 0:return new e;case 1:return new e(t[0]);case 2:return new e(t[0],t[1]);case 3:return new e(t[0],t[1],t[2]);case 4:return new e(t[0],t[1],t[2],t[3]);case 5:return new e(t[0],t[1],t[2],t[3],t[4]);case 6:return new e(t[0],t[1],t[2],t[3],t[4],t[5]);case 7:return new e(t[0],t[1],t[2],t[3],t[4],t[5],t[6])}var n=dr(e.prototype),r=e.apply(n,t);return Aa(r)?r:n}}function fo(e){return function(t,n,r){var i=tt(t);if(!_a(t)){var s=Lo(n,3);t=iu(t),n=function(e){return s(i[e],e,i)}}var a=e(t,n,r);return a>-1?i[s?t[a]:a]:o}}function ho(e){return ko((function(t){var n=t.length,r=n,i=vr.prototype.thru;for(e&&t.reverse();r--;){var s=t[r];if("function"!=typeof s)throw new it(u);if(i&&!a&&"wrapper"==Oo(s))var a=new vr([],!0)}for(r=a?r:n;++r<n;){var l=Oo(s=t[r]),c="wrapper"==l?jo(s):o;a=c&&Ko(c[0])&&c[1]==(T|b|x|C)&&!c[4].length&&1==c[9]?a[Oo(c[0])].apply(a,c[3]):1==s.length&&Ko(s)?a[l]():a.thru(s)}return function(){var e=arguments,r=e[0];if(a&&1==e.length&&ma(r))return a.plant(r).value();for(var i=0,o=n?t[i].apply(this,e):r;++i<n;)o=t[i].call(this,o);return o}}))}function po(e,t,n,i,s,a,u,l,c,f){var h=t&T,p=t&m,d=t&y,g=t&(b|w),v=t&S,_=d?o:co(e);return function m(){for(var y=arguments.length,b=r(y),w=y;w--;)b[w]=arguments[w];if(g)var x=Fo(m),E=function(e,t){for(var n=e.length,r=0;n--;)e[n]===t&&++r;return r}(b,x);if(i&&(b=eo(b,i,s,g)),a&&(b=to(b,a,u,g)),y-=E,g&&y<f){var T=Dn(b,x);return wo(e,t,po,m.placeholder,n,b,T,l,c,f-y)}var C=p?n:this,S=d?C[e]:e;return y=b.length,l?b=function(e,t){var n=e.length,r=Xn(t.length,n),i=no(e);for(;r--;){var s=t[r];e[r]=Xo(s,n)?i[s]:o}return e}(b,l):v&&y>1&&b.reverse(),h&&c<y&&(b.length=c),this&&this!==jt&&this instanceof m&&(S=_||co(S)),S.apply(C,b)}}function go(e,t){return function(n,r){return function(e,t,n,r){return Vr(e,(function(e,i,o){t(r,n(e),i,o)})),r}(n,e,t(r),{})}}function vo(e,t){return function(n,r){var i;if(n===o&&r===o)return t;if(n!==o&&(i=n),r!==o){if(i===o)return r;"string"==typeof n||"string"==typeof r?(n=Li(n),r=Li(r)):(n=Fi(n),r=Fi(r)),i=e(n,r)}return i}}function mo(e){return ko((function(t){return t=Qt(t,mn(Lo())),Ei((function(n){var r=this;return e(t,(function(e){return $t(e,r,n)}))}))}))}function yo(e,t){var n=(t=t===o?" ":Li(t)).length;if(n<2)return n?xi(t,e):t;var r=xi(t,qn(e/Rn(t)));return Cn(t)?Gi(Nn(r),0,e).join(""):r.slice(0,e)}function _o(e){return function(t,n,i){return i&&"number"!=typeof i&&Vo(t,n,i)&&(n=i=o),t=Ia(t),n===o?(n=t,t=0):n=Ia(n),function(e,t,n,i){for(var o=-1,s=$n(qn((t-e)/(n||1)),0),a=r(s);s--;)a[i?s:++o]=e,e+=n;return a}(t,n,i=i===o?t<n?1:-1:Ia(i),e)}}function bo(e){return function(t,n){return"string"==typeof t&&"string"==typeof n||(t=Ha(t),n=Ha(n)),e(t,n)}}function wo(e,t,n,r,i,s,a,u,l,c){var f=t&b;t|=f?x:E,(t&=~(f?E:x))&_||(t&=~(m|y));var h=[e,t,i,f?s:o,f?a:o,f?o:s,f?o:a,u,l,c],p=n.apply(o,h);return Ko(e)&&rs(p,h),p.placeholder=r,ss(p,e,t)}function xo(e){var t=et[e];return function(e,n){if(e=Ha(e),(n=null==n?0:Xn(Ba(n),292))&&Wn(e)){var r=($a(e)+"e").split("e");return+((r=($a(t(r[0]+"e"+(+r[1]+n)))+"e").split("e"))[0]+"e"+(+r[1]-n))}return t(e)}}var Eo=er&&1/zn(new er([,-0]))[1]==j?function(e){return new er(e)}:Fu;function To(e){return function(t){var n=Wo(t);return n==Y?Sn(t):n==ne?kn(t):function(e,t){return Qt(t,(function(t){return[t,e[t]]}))}(t,e(t))}}function Co(e,t,n,i,s,a,l,c){var h=t&y;if(!h&&"function"!=typeof e)throw new it(u);var p=i?i.length:0;if(p||(t&=~(x|E),i=s=o),l=l===o?l:$n(Ba(l),0),c=c===o?c:Ba(c),p-=s?s.length:0,t&E){var d=i,g=s;i=s=o}var v=h?o:jo(e),S=[e,t,n,i,s,d,g,a,l,c];if(v&&function(e,t){var n=e[1],r=t[1],i=n|r,o=i<(m|y|T),s=r==T&&n==b||r==T&&n==C&&e[7].length<=t[8]||r==(T|C)&&t[7].length<=t[8]&&n==b;if(!o&&!s)return e;r&m&&(e[2]=t[2],i|=n&m?0:_);var a=t[3];if(a){var u=e[3];e[3]=u?eo(u,a,t[4]):a,e[4]=u?Dn(e[3],f):t[4]}(a=t[5])&&(u=e[5],e[5]=u?to(u,a,t[6]):a,e[6]=u?Dn(e[5],f):t[6]);(a=t[7])&&(e[7]=a);r&T&&(e[8]=null==e[8]?t[8]:Xn(e[8],t[8]));null==e[9]&&(e[9]=t[9]);e[0]=t[0],e[1]=i}(S,v),e=S[0],t=S[1],n=S[2],i=S[3],s=S[4],!(c=S[9]=S[9]===o?h?0:e.length:$n(S[9]-p,0))&&t&(b|w)&&(t&=~(b|w)),t&&t!=m)A=t==b||t==w?function(e,t,n){var i=co(e);return function s(){for(var a=arguments.length,u=r(a),l=a,c=Fo(s);l--;)u[l]=arguments[l];var f=a<3&&u[0]!==c&&u[a-1]!==c?[]:Dn(u,c);return(a-=f.length)<n?wo(e,t,po,s.placeholder,o,u,f,o,o,n-a):$t(this&&this!==jt&&this instanceof s?i:e,this,u)}}(e,t,c):t!=x&&t!=(m|x)||s.length?po.apply(o,S):function(e,t,n,i){var o=t&m,s=co(e);return function t(){for(var a=-1,u=arguments.length,l=-1,c=i.length,f=r(c+u),h=this&&this!==jt&&this instanceof t?s:e;++l<c;)f[l]=i[l];for(;u--;)f[l++]=arguments[++a];return $t(h,o?n:this,f)}}(e,t,n,i);else var A=function(e,t,n){var r=t&m,i=co(e);return function t(){return(this&&this!==jt&&this instanceof t?i:e).apply(r?n:this,arguments)}}(e,t,n);return ss((v?Ai:rs)(A,S),e,t)}function So(e,t,n,r){return e===o||pa(e,at[n])&&!ct.call(r,n)?t:e}function Ao(e,t,n,r,i,s){return Aa(e)&&Aa(t)&&(s.set(t,e),gi(e,t,o,Ao,s),s.delete(t)),e}function Do(e){return Ra(e)?o:e}function zo(e,t,n,r,i,s){var a=n&g,u=e.length,l=t.length;if(u!=l&&!(a&&l>u))return!1;var c=s.get(e);if(c&&s.get(t))return c==t;var f=-1,h=!0,p=n&v?new wr:o;for(s.set(e,t),s.set(t,e);++f<u;){var d=e[f],m=t[f];if(r)var y=a?r(m,d,f,t,e,s):r(d,m,f,e,t,s);if(y!==o){if(y)continue;h=!1;break}if(p){if(!rn(t,(function(e,t){if(!_n(p,t)&&(d===e||i(d,e,n,r,s)))return p.push(t)}))){h=!1;break}}else if(d!==m&&!i(d,m,n,r,s)){h=!1;break}}return s.delete(e),s.delete(t),h}function ko(e){return os(es(e,o,ys),e+"")}function Ro(e){return Zr(e,iu,Io)}function No(e){return Zr(e,ou,Bo)}var jo=rr?function(e){return rr.get(e)}:Fu;function Oo(e){for(var t=e.name+"",n=ir[t],r=ct.call(ir,t)?n.length:0;r--;){var i=n[r],o=i.func;if(null==o||o==e)return i.name}return t}function Fo(e){return(ct.call(pr,"placeholder")?pr:e).placeholder}function Lo(){var e=pr.iteratee||Ru;return e=e===Ru?ui:e,arguments.length?e(arguments[0],arguments[1]):e}function Po(e,t){var n,r,i=e.__data__;return("string"==(r=typeof(n=t))||"number"==r||"symbol"==r||"boolean"==r?"__proto__"!==n:null===n)?i["string"==typeof t?"string":"hash"]:i.map}function qo(e){for(var t=iu(e),n=t.length;n--;){var r=t[n],i=e[r];t[n]=[r,i,Jo(i)]}return t}function Mo(e,t){var n=function(e,t){return null==e?o:e[t]}(e,t);return ai(n)?n:o}var Io=In?function(e){return null==e?[]:(e=tt(e),Yt(In(e),(function(t){return Ot.call(e,t)})))}:Wu,Bo=In?function(e){for(var t=[];e;)en(t,Io(e)),e=Rt(e);return t}:Wu,Wo=Jr;function Ho(e,t,n){for(var r=-1,i=(t=Xi(t,e)).length,o=!1;++r<i;){var s=cs(t[r]);if(!(o=null!=e&&n(e,s)))break;e=e[s]}return o||++r!=i?o:!!(i=null==e?0:e.length)&&Sa(i)&&Xo(s,i)&&(ma(e)||va(e))}function Uo(e){return"function"!=typeof e.constructor||Zo(e)?{}:dr(Rt(e))}function $o(e){return ma(e)||va(e)||!!(Pt&&e&&e[Pt])}function Xo(e,t){var n=typeof e;return!!(t=null==t?O:t)&&("number"==n||"symbol"!=n&&Ge.test(e))&&e>-1&&e%1==0&&e<t}function Vo(e,t,n){if(!Aa(n))return!1;var r=typeof t;return!!("number"==r?_a(n)&&Xo(t,n.length):"string"==r&&t in n)&&pa(n[t],e)}function Go(e,t){if(ma(e))return!1;var n=typeof e;return!("number"!=n&&"symbol"!=n&&"boolean"!=n&&null!=e&&!Fa(e))||(ke.test(e)||!ze.test(e)||null!=t&&e in tt(t))}function Ko(e){var t=Oo(e),n=pr[t];if("function"!=typeof n||!(t in mr.prototype))return!1;if(e===n)return!0;var r=jo(n);return!!r&&e===r[0]}(Zn&&Wo(new Zn(new ArrayBuffer(1)))!=le||Jn&&Wo(new Jn)!=Y||Qn&&"[object Promise]"!=Wo(Qn.resolve())||er&&Wo(new er)!=ne||tr&&Wo(new tr)!=se)&&(Wo=function(e){var t=Jr(e),n=t==Q?e.constructor:o,r=n?fs(n):"";if(r)switch(r){case or:return le;case sr:return Y;case ar:return"[object Promise]";case ur:return ne;case lr:return se}return t});var Yo=ut?Ta:Hu;function Zo(e){var t=e&&e.constructor;return e===("function"==typeof t&&t.prototype||at)}function Jo(e){return e==e&&!Aa(e)}function Qo(e,t){return function(n){return null!=n&&(n[e]===t&&(t!==o||e in tt(n)))}}function es(e,t,n){return t=$n(t===o?e.length-1:t,0),function(){for(var i=arguments,o=-1,s=$n(i.length-t,0),a=r(s);++o<s;)a[o]=i[t+o];o=-1;for(var u=r(t+1);++o<t;)u[o]=i[o];return u[t]=n(a),$t(e,this,u)}}function ts(e,t){return t.length<2?e:Yr(e,ki(t,0,-1))}function ns(e,t){if(("constructor"!==t||"function"!=typeof e[t])&&"__proto__"!=t)return e[t]}var rs=as(Ai),is=Pn||function(e,t){return jt.setTimeout(e,t)},os=as(Di);function ss(e,t,n){var r=t+"";return os(e,function(e,t){var n=t.length;if(!n)return e;var r=n-1;return t[r]=(n>1?"& ":"")+t[r],t=t.join(n>2?", ":" "),e.replace(Pe,"{\n/* [wrapped with "+t+"] */\n")}(r,function(e,t){return Vt(I,(function(n){var r="_."+n[0];t&n[1]&&!Zt(e,r)&&e.push(r)})),e.sort()}(function(e){var t=e.match(qe);return t?t[1].split(Me):[]}(r),n)))}function as(e){var t=0,n=0;return function(){var r=Vn(),i=k-(r-n);if(n=r,i>0){if(++t>=z)return arguments[0]}else t=0;return e.apply(o,arguments)}}function us(e,t){var n=-1,r=e.length,i=r-1;for(t=t===o?r:t;++n<t;){var s=wi(n,i),a=e[s];e[s]=e[n],e[n]=a}return e.length=t,e}var ls=function(e){var t=aa(e,(function(e){return n.size===c&&n.clear(),e})),n=t.cache;return t}((function(e){var t=[];return 46===e.charCodeAt(0)&&t.push(""),e.replace(Re,(function(e,n,r,i){t.push(r?i.replace(Be,"$1"):n||e)})),t}));function cs(e){if("string"==typeof e||Fa(e))return e;var t=e+"";return"0"==t&&1/e==-j?"-0":t}function fs(e){if(null!=e){try{return lt.call(e)}catch(e){}try{return e+""}catch(e){}}return""}function hs(e){if(e instanceof mr)return e.clone();var t=new vr(e.__wrapped__,e.__chain__);return t.__actions__=no(e.__actions__),t.__index__=e.__index__,t.__values__=e.__values__,t}var ps=Ei((function(e,t){return ba(e)?qr(e,Ur(t,1,ba,!0)):[]})),ds=Ei((function(e,t){var n=Es(t);return ba(n)&&(n=o),ba(e)?qr(e,Ur(t,1,ba,!0),Lo(n,2)):[]})),gs=Ei((function(e,t){var n=Es(t);return ba(n)&&(n=o),ba(e)?qr(e,Ur(t,1,ba,!0),o,n):[]}));function vs(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=null==n?0:Ba(n);return i<0&&(i=$n(r+i,0)),an(e,Lo(t,3),i)}function ms(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=r-1;return n!==o&&(i=Ba(n),i=n<0?$n(r+i,0):Xn(i,r-1)),an(e,Lo(t,3),i,!0)}function ys(e){return(null==e?0:e.length)?Ur(e,1):[]}function _s(e){return e&&e.length?e[0]:o}var bs=Ei((function(e){var t=Qt(e,Ui);return t.length&&t[0]===e[0]?ni(t):[]})),ws=Ei((function(e){var t=Es(e),n=Qt(e,Ui);return t===Es(n)?t=o:n.pop(),n.length&&n[0]===e[0]?ni(n,Lo(t,2)):[]})),xs=Ei((function(e){var t=Es(e),n=Qt(e,Ui);return(t="function"==typeof t?t:o)&&n.pop(),n.length&&n[0]===e[0]?ni(n,o,t):[]}));function Es(e){var t=null==e?0:e.length;return t?e[t-1]:o}var Ts=Ei(Cs);function Cs(e,t){return e&&e.length&&t&&t.length?_i(e,t):e}var Ss=ko((function(e,t){var n=null==e?0:e.length,r=jr(e,t);return bi(e,Qt(t,(function(e){return Xo(e,n)?+e:e})).sort(Qi)),r}));function As(e){return null==e?e:Yn.call(e)}var Ds=Ei((function(e){return Pi(Ur(e,1,ba,!0))})),zs=Ei((function(e){var t=Es(e);return ba(t)&&(t=o),Pi(Ur(e,1,ba,!0),Lo(t,2))})),ks=Ei((function(e){var t=Es(e);return t="function"==typeof t?t:o,Pi(Ur(e,1,ba,!0),o,t)}));function Rs(e){if(!e||!e.length)return[];var t=0;return e=Yt(e,(function(e){if(ba(e))return t=$n(e.length,t),!0})),vn(t,(function(t){return Qt(e,hn(t))}))}function Ns(e,t){if(!e||!e.length)return[];var n=Rs(e);return null==t?n:Qt(n,(function(e){return $t(t,o,e)}))}var js=Ei((function(e,t){return ba(e)?qr(e,t):[]})),Os=Ei((function(e){return Wi(Yt(e,ba))})),Fs=Ei((function(e){var t=Es(e);return ba(t)&&(t=o),Wi(Yt(e,ba),Lo(t,2))})),Ls=Ei((function(e){var t=Es(e);return t="function"==typeof t?t:o,Wi(Yt(e,ba),o,t)})),Ps=Ei(Rs);var qs=Ei((function(e){var t=e.length,n=t>1?e[t-1]:o;return n="function"==typeof n?(e.pop(),n):o,Ns(e,n)}));function Ms(e){var t=pr(e);return t.__chain__=!0,t}function Is(e,t){return t(e)}var Bs=ko((function(e){var t=e.length,n=t?e[0]:0,r=this.__wrapped__,i=function(t){return jr(t,e)};return!(t>1||this.__actions__.length)&&r instanceof mr&&Xo(n)?((r=r.slice(n,+n+(t?1:0))).__actions__.push({func:Is,args:[i],thisArg:o}),new vr(r,this.__chain__).thru((function(e){return t&&!e.length&&e.push(o),e}))):this.thru(i)}));var Ws=io((function(e,t,n){ct.call(e,n)?++e[n]:Nr(e,n,1)}));var Hs=fo(vs),Us=fo(ms);function $s(e,t){return(ma(e)?Vt:Mr)(e,Lo(t,3))}function Xs(e,t){return(ma(e)?Gt:Ir)(e,Lo(t,3))}var Vs=io((function(e,t,n){ct.call(e,n)?e[n].push(t):Nr(e,n,[t])}));var Gs=Ei((function(e,t,n){var i=-1,o="function"==typeof t,s=_a(e)?r(e.length):[];return Mr(e,(function(e){s[++i]=o?$t(t,e,n):ri(e,t,n)})),s})),Ks=io((function(e,t,n){Nr(e,n,t)}));function Ys(e,t){return(ma(e)?Qt:hi)(e,Lo(t,3))}var Zs=io((function(e,t,n){e[n?0:1].push(t)}),(function(){return[[],[]]}));var Js=Ei((function(e,t){if(null==e)return[];var n=t.length;return n>1&&Vo(e,t[0],t[1])?t=[]:n>2&&Vo(t[0],t[1],t[2])&&(t=[t[0]]),mi(e,Ur(t,1),[])})),Qs=Ln||function(){return jt.Date.now()};function ea(e,t,n){return t=n?o:t,t=e&&null==t?e.length:t,Co(e,T,o,o,o,o,t)}function ta(e,t){var n;if("function"!=typeof t)throw new it(u);return e=Ba(e),function(){return--e>0&&(n=t.apply(this,arguments)),e<=1&&(t=o),n}}var na=Ei((function(e,t,n){var r=m;if(n.length){var i=Dn(n,Fo(na));r|=x}return Co(e,r,t,n,i)})),ra=Ei((function(e,t,n){var r=m|y;if(n.length){var i=Dn(n,Fo(ra));r|=x}return Co(t,r,e,n,i)}));function ia(e,t,n){var r,i,s,a,l,c,f=0,h=!1,p=!1,d=!0;if("function"!=typeof e)throw new it(u);function g(t){var n=r,s=i;return r=i=o,f=t,a=e.apply(s,n)}function v(e){var n=e-c;return c===o||n>=t||n<0||p&&e-f>=s}function m(){var e=Qs();if(v(e))return y(e);l=is(m,function(e){var n=t-(e-c);return p?Xn(n,s-(e-f)):n}(e))}function y(e){return l=o,d&&r?g(e):(r=i=o,a)}function _(){var e=Qs(),n=v(e);if(r=arguments,i=this,c=e,n){if(l===o)return function(e){return f=e,l=is(m,t),h?g(e):a}(c);if(p)return Ki(l),l=is(m,t),g(c)}return l===o&&(l=is(m,t)),a}return t=Ha(t)||0,Aa(n)&&(h=!!n.leading,s=(p="maxWait"in n)?$n(Ha(n.maxWait)||0,t):s,d="trailing"in n?!!n.trailing:d),_.cancel=function(){l!==o&&Ki(l),f=0,r=c=i=l=o},_.flush=function(){return l===o?a:y(Qs())},_}var oa=Ei((function(e,t){return Pr(e,1,t)})),sa=Ei((function(e,t,n){return Pr(e,Ha(t)||0,n)}));function aa(e,t){if("function"!=typeof e||null!=t&&"function"!=typeof t)throw new it(u);var n=function(){var r=arguments,i=t?t.apply(this,r):r[0],o=n.cache;if(o.has(i))return o.get(i);var s=e.apply(this,r);return n.cache=o.set(i,s)||o,s};return n.cache=new(aa.Cache||br),n}function ua(e){if("function"!=typeof e)throw new it(u);return function(){var t=arguments;switch(t.length){case 0:return!e.call(this);case 1:return!e.call(this,t[0]);case 2:return!e.call(this,t[0],t[1]);case 3:return!e.call(this,t[0],t[1],t[2])}return!e.apply(this,t)}}aa.Cache=br;var la=Vi((function(e,t){var n=(t=1==t.length&&ma(t[0])?Qt(t[0],mn(Lo())):Qt(Ur(t,1),mn(Lo()))).length;return Ei((function(r){for(var i=-1,o=Xn(r.length,n);++i<o;)r[i]=t[i].call(this,r[i]);return $t(e,this,r)}))})),ca=Ei((function(e,t){var n=Dn(t,Fo(ca));return Co(e,x,o,t,n)})),fa=Ei((function(e,t){var n=Dn(t,Fo(fa));return Co(e,E,o,t,n)})),ha=ko((function(e,t){return Co(e,C,o,o,o,t)}));function pa(e,t){return e===t||e!=e&&t!=t}var da=bo(Qr),ga=bo((function(e,t){return e>=t})),va=ii(function(){return arguments}())?ii:function(e){return Da(e)&&ct.call(e,"callee")&&!Ot.call(e,"callee")},ma=r.isArray,ya=Mt?mn(Mt):function(e){return Da(e)&&Jr(e)==ue};function _a(e){return null!=e&&Sa(e.length)&&!Ta(e)}function ba(e){return Da(e)&&_a(e)}var wa=Bn||Hu,xa=It?mn(It):function(e){return Da(e)&&Jr(e)==$};function Ea(e){if(!Da(e))return!1;var t=Jr(e);return t==V||t==X||"string"==typeof e.message&&"string"==typeof e.name&&!Ra(e)}function Ta(e){if(!Aa(e))return!1;var t=Jr(e);return t==G||t==K||t==H||t==ee}function Ca(e){return"number"==typeof e&&e==Ba(e)}function Sa(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=O}function Aa(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}function Da(e){return null!=e&&"object"==typeof e}var za=Bt?mn(Bt):function(e){return Da(e)&&Wo(e)==Y};function ka(e){return"number"==typeof e||Da(e)&&Jr(e)==Z}function Ra(e){if(!Da(e)||Jr(e)!=Q)return!1;var t=Rt(e);if(null===t)return!0;var n=ct.call(t,"constructor")&&t.constructor;return"function"==typeof n&&n instanceof n&&lt.call(n)==dt}var Na=Wt?mn(Wt):function(e){return Da(e)&&Jr(e)==te};var ja=Ht?mn(Ht):function(e){return Da(e)&&Wo(e)==ne};function Oa(e){return"string"==typeof e||!ma(e)&&Da(e)&&Jr(e)==re}function Fa(e){return"symbol"==typeof e||Da(e)&&Jr(e)==ie}var La=Ut?mn(Ut):function(e){return Da(e)&&Sa(e.length)&&!!St[Jr(e)]};var Pa=bo(fi),qa=bo((function(e,t){return e<=t}));function Ma(e){if(!e)return[];if(_a(e))return Oa(e)?Nn(e):no(e);if(qt&&e[qt])return function(e){for(var t,n=[];!(t=e.next()).done;)n.push(t.value);return n}(e[qt]());var t=Wo(e);return(t==Y?Sn:t==ne?zn:pu)(e)}function Ia(e){return e?(e=Ha(e))===j||e===-j?(e<0?-1:1)*F:e==e?e:0:0===e?e:0}function Ba(e){var t=Ia(e),n=t%1;return t==t?n?t-n:t:0}function Wa(e){return e?Or(Ba(e),0,P):0}function Ha(e){if("number"==typeof e)return e;if(Fa(e))return L;if(Aa(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=Aa(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=e.replace(Oe,"");var n=$e.test(e);return n||Ve.test(e)?kt(e.slice(2),n?2:8):Ue.test(e)?L:+e}function Ua(e){return ro(e,ou(e))}function $a(e){return null==e?"":Li(e)}var Xa=oo((function(e,t){if(Zo(t)||_a(t))ro(t,iu(t),e);else for(var n in t)ct.call(t,n)&&Dr(e,n,t[n])})),Va=oo((function(e,t){ro(t,ou(t),e)})),Ga=oo((function(e,t,n,r){ro(t,ou(t),e,r)})),Ka=oo((function(e,t,n,r){ro(t,iu(t),e,r)})),Ya=ko(jr);var Za=Ei((function(e,t){e=tt(e);var n=-1,r=t.length,i=r>2?t[2]:o;for(i&&Vo(t[0],t[1],i)&&(r=1);++n<r;)for(var s=t[n],a=ou(s),u=-1,l=a.length;++u<l;){var c=a[u],f=e[c];(f===o||pa(f,at[c])&&!ct.call(e,c))&&(e[c]=s[c])}return e})),Ja=Ei((function(e){return e.push(o,Ao),$t(au,o,e)}));function Qa(e,t,n){var r=null==e?o:Yr(e,t);return r===o?n:r}function eu(e,t){return null!=e&&Ho(e,t,ti)}var tu=go((function(e,t,n){null!=t&&"function"!=typeof t.toString&&(t=pt.call(t)),e[t]=n}),Au(ku)),nu=go((function(e,t,n){null!=t&&"function"!=typeof t.toString&&(t=pt.call(t)),ct.call(e,t)?e[t].push(n):e[t]=[n]}),Lo),ru=Ei(ri);function iu(e){return _a(e)?Er(e):li(e)}function ou(e){return _a(e)?Er(e,!0):ci(e)}var su=oo((function(e,t,n){gi(e,t,n)})),au=oo((function(e,t,n,r){gi(e,t,n,r)})),uu=ko((function(e,t){var n={};if(null==e)return n;var r=!1;t=Qt(t,(function(t){return t=Xi(t,e),r||(r=t.length>1),t})),ro(e,No(e),n),r&&(n=Fr(n,h|p|d,Do));for(var i=t.length;i--;)qi(n,t[i]);return n}));var lu=ko((function(e,t){return null==e?{}:function(e,t){return yi(e,t,(function(t,n){return eu(e,n)}))}(e,t)}));function cu(e,t){if(null==e)return{};var n=Qt(No(e),(function(e){return[e]}));return t=Lo(t),yi(e,n,(function(e,n){return t(e,n[0])}))}var fu=To(iu),hu=To(ou);function pu(e){return null==e?[]:yn(e,iu(e))}var du=lo((function(e,t,n){return t=t.toLowerCase(),e+(n?gu(t):t)}));function gu(e){return Eu($a(e).toLowerCase())}function vu(e){return(e=$a(e))&&e.replace(Ke,xn).replace(_t,"")}var mu=lo((function(e,t,n){return e+(n?"-":"")+t.toLowerCase()})),yu=lo((function(e,t,n){return e+(n?" ":"")+t.toLowerCase()})),_u=uo("toLowerCase");var bu=lo((function(e,t,n){return e+(n?"_":"")+t.toLowerCase()}));var wu=lo((function(e,t,n){return e+(n?" ":"")+Eu(t)}));var xu=lo((function(e,t,n){return e+(n?" ":"")+t.toUpperCase()})),Eu=uo("toUpperCase");function Tu(e,t,n){return e=$a(e),(t=n?o:t)===o?function(e){return Et.test(e)}(e)?function(e){return e.match(wt)||[]}(e):function(e){return e.match(Ie)||[]}(e):e.match(t)||[]}var Cu=Ei((function(e,t){try{return $t(e,o,t)}catch(e){return Ea(e)?e:new Je(e)}})),Su=ko((function(e,t){return Vt(t,(function(t){t=cs(t),Nr(e,t,na(e[t],e))})),e}));function Au(e){return function(){return e}}var Du=ho(),zu=ho(!0);function ku(e){return e}function Ru(e){return ui("function"==typeof e?e:Fr(e,h))}var Nu=Ei((function(e,t){return function(n){return ri(n,e,t)}})),ju=Ei((function(e,t){return function(n){return ri(e,n,t)}}));function Ou(e,t,n){var r=iu(t),i=Kr(t,r);null!=n||Aa(t)&&(i.length||!r.length)||(n=t,t=e,e=this,i=Kr(t,iu(t)));var o=!(Aa(n)&&"chain"in n&&!n.chain),s=Ta(e);return Vt(i,(function(n){var r=t[n];e[n]=r,s&&(e.prototype[n]=function(){var t=this.__chain__;if(o||t){var n=e(this.__wrapped__),i=n.__actions__=no(this.__actions__);return i.push({func:r,args:arguments,thisArg:e}),n.__chain__=t,n}return r.apply(e,en([this.value()],arguments))})})),e}function Fu(){}var Lu=mo(Qt),Pu=mo(Kt),qu=mo(rn);function Mu(e){return Go(e)?hn(cs(e)):function(e){return function(t){return Yr(t,e)}}(e)}var Iu=_o(),Bu=_o(!0);function Wu(){return[]}function Hu(){return!1}var Uu=vo((function(e,t){return e+t}),0),$u=xo("ceil"),Xu=vo((function(e,t){return e/t}),1),Vu=xo("floor");var Gu,Ku=vo((function(e,t){return e*t}),1),Yu=xo("round"),Zu=vo((function(e,t){return e-t}),0);return pr.after=function(e,t){if("function"!=typeof t)throw new it(u);return e=Ba(e),function(){if(--e<1)return t.apply(this,arguments)}},pr.ary=ea,pr.assign=Xa,pr.assignIn=Va,pr.assignInWith=Ga,pr.assignWith=Ka,pr.at=Ya,pr.before=ta,pr.bind=na,pr.bindAll=Su,pr.bindKey=ra,pr.castArray=function(){if(!arguments.length)return[];var e=arguments[0];return ma(e)?e:[e]},pr.chain=Ms,pr.chunk=function(e,t,n){t=(n?Vo(e,t,n):t===o)?1:$n(Ba(t),0);var i=null==e?0:e.length;if(!i||t<1)return[];for(var s=0,a=0,u=r(qn(i/t));s<i;)u[a++]=ki(e,s,s+=t);return u},pr.compact=function(e){for(var t=-1,n=null==e?0:e.length,r=0,i=[];++t<n;){var o=e[t];o&&(i[r++]=o)}return i},pr.concat=function(){var e=arguments.length;if(!e)return[];for(var t=r(e-1),n=arguments[0],i=e;i--;)t[i-1]=arguments[i];return en(ma(n)?no(n):[n],Ur(t,1))},pr.cond=function(e){var t=null==e?0:e.length,n=Lo();return e=t?Qt(e,(function(e){if("function"!=typeof e[1])throw new it(u);return[n(e[0]),e[1]]})):[],Ei((function(n){for(var r=-1;++r<t;){var i=e[r];if($t(i[0],this,n))return $t(i[1],this,n)}}))},pr.conforms=function(e){return function(e){var t=iu(e);return function(n){return Lr(n,e,t)}}(Fr(e,h))},pr.constant=Au,pr.countBy=Ws,pr.create=function(e,t){var n=dr(e);return null==t?n:Rr(n,t)},pr.curry=function e(t,n,r){var i=Co(t,b,o,o,o,o,o,n=r?o:n);return i.placeholder=e.placeholder,i},pr.curryRight=function e(t,n,r){var i=Co(t,w,o,o,o,o,o,n=r?o:n);return i.placeholder=e.placeholder,i},pr.debounce=ia,pr.defaults=Za,pr.defaultsDeep=Ja,pr.defer=oa,pr.delay=sa,pr.difference=ps,pr.differenceBy=ds,pr.differenceWith=gs,pr.drop=function(e,t,n){var r=null==e?0:e.length;return r?ki(e,(t=n||t===o?1:Ba(t))<0?0:t,r):[]},pr.dropRight=function(e,t,n){var r=null==e?0:e.length;return r?ki(e,0,(t=r-(t=n||t===o?1:Ba(t)))<0?0:t):[]},pr.dropRightWhile=function(e,t){return e&&e.length?Ii(e,Lo(t,3),!0,!0):[]},pr.dropWhile=function(e,t){return e&&e.length?Ii(e,Lo(t,3),!0):[]},pr.fill=function(e,t,n,r){var i=null==e?0:e.length;return i?(n&&"number"!=typeof n&&Vo(e,t,n)&&(n=0,r=i),function(e,t,n,r){var i=e.length;for((n=Ba(n))<0&&(n=-n>i?0:i+n),(r=r===o||r>i?i:Ba(r))<0&&(r+=i),r=n>r?0:Wa(r);n<r;)e[n++]=t;return e}(e,t,n,r)):[]},pr.filter=function(e,t){return(ma(e)?Yt:Hr)(e,Lo(t,3))},pr.flatMap=function(e,t){return Ur(Ys(e,t),1)},pr.flatMapDeep=function(e,t){return Ur(Ys(e,t),j)},pr.flatMapDepth=function(e,t,n){return n=n===o?1:Ba(n),Ur(Ys(e,t),n)},pr.flatten=ys,pr.flattenDeep=function(e){return(null==e?0:e.length)?Ur(e,j):[]},pr.flattenDepth=function(e,t){return(null==e?0:e.length)?Ur(e,t=t===o?1:Ba(t)):[]},pr.flip=function(e){return Co(e,S)},pr.flow=Du,pr.flowRight=zu,pr.fromPairs=function(e){for(var t=-1,n=null==e?0:e.length,r={};++t<n;){var i=e[t];r[i[0]]=i[1]}return r},pr.functions=function(e){return null==e?[]:Kr(e,iu(e))},pr.functionsIn=function(e){return null==e?[]:Kr(e,ou(e))},pr.groupBy=Vs,pr.initial=function(e){return(null==e?0:e.length)?ki(e,0,-1):[]},pr.intersection=bs,pr.intersectionBy=ws,pr.intersectionWith=xs,pr.invert=tu,pr.invertBy=nu,pr.invokeMap=Gs,pr.iteratee=Ru,pr.keyBy=Ks,pr.keys=iu,pr.keysIn=ou,pr.map=Ys,pr.mapKeys=function(e,t){var n={};return t=Lo(t,3),Vr(e,(function(e,r,i){Nr(n,t(e,r,i),e)})),n},pr.mapValues=function(e,t){var n={};return t=Lo(t,3),Vr(e,(function(e,r,i){Nr(n,r,t(e,r,i))})),n},pr.matches=function(e){return pi(Fr(e,h))},pr.matchesProperty=function(e,t){return di(e,Fr(t,h))},pr.memoize=aa,pr.merge=su,pr.mergeWith=au,pr.method=Nu,pr.methodOf=ju,pr.mixin=Ou,pr.negate=ua,pr.nthArg=function(e){return e=Ba(e),Ei((function(t){return vi(t,e)}))},pr.omit=uu,pr.omitBy=function(e,t){return cu(e,ua(Lo(t)))},pr.once=function(e){return ta(2,e)},pr.orderBy=function(e,t,n,r){return null==e?[]:(ma(t)||(t=null==t?[]:[t]),ma(n=r?o:n)||(n=null==n?[]:[n]),mi(e,t,n))},pr.over=Lu,pr.overArgs=la,pr.overEvery=Pu,pr.overSome=qu,pr.partial=ca,pr.partialRight=fa,pr.partition=Zs,pr.pick=lu,pr.pickBy=cu,pr.property=Mu,pr.propertyOf=function(e){return function(t){return null==e?o:Yr(e,t)}},pr.pull=Ts,pr.pullAll=Cs,pr.pullAllBy=function(e,t,n){return e&&e.length&&t&&t.length?_i(e,t,Lo(n,2)):e},pr.pullAllWith=function(e,t,n){return e&&e.length&&t&&t.length?_i(e,t,o,n):e},pr.pullAt=Ss,pr.range=Iu,pr.rangeRight=Bu,pr.rearg=ha,pr.reject=function(e,t){return(ma(e)?Yt:Hr)(e,ua(Lo(t,3)))},pr.remove=function(e,t){var n=[];if(!e||!e.length)return n;var r=-1,i=[],o=e.length;for(t=Lo(t,3);++r<o;){var s=e[r];t(s,r,e)&&(n.push(s),i.push(r))}return bi(e,i),n},pr.rest=function(e,t){if("function"!=typeof e)throw new it(u);return Ei(e,t=t===o?t:Ba(t))},pr.reverse=As,pr.sampleSize=function(e,t,n){return t=(n?Vo(e,t,n):t===o)?1:Ba(t),(ma(e)?Cr:Ci)(e,t)},pr.set=function(e,t,n){return null==e?e:Si(e,t,n)},pr.setWith=function(e,t,n,r){return r="function"==typeof r?r:o,null==e?e:Si(e,t,n,r)},pr.shuffle=function(e){return(ma(e)?Sr:zi)(e)},pr.slice=function(e,t,n){var r=null==e?0:e.length;return r?(n&&"number"!=typeof n&&Vo(e,t,n)?(t=0,n=r):(t=null==t?0:Ba(t),n=n===o?r:Ba(n)),ki(e,t,n)):[]},pr.sortBy=Js,pr.sortedUniq=function(e){return e&&e.length?Oi(e):[]},pr.sortedUniqBy=function(e,t){return e&&e.length?Oi(e,Lo(t,2)):[]},pr.split=function(e,t,n){return n&&"number"!=typeof n&&Vo(e,t,n)&&(t=n=o),(n=n===o?P:n>>>0)?(e=$a(e))&&("string"==typeof t||null!=t&&!Na(t))&&!(t=Li(t))&&Cn(e)?Gi(Nn(e),0,n):e.split(t,n):[]},pr.spread=function(e,t){if("function"!=typeof e)throw new it(u);return t=null==t?0:$n(Ba(t),0),Ei((function(n){var r=n[t],i=Gi(n,0,t);return r&&en(i,r),$t(e,this,i)}))},pr.tail=function(e){var t=null==e?0:e.length;return t?ki(e,1,t):[]},pr.take=function(e,t,n){return e&&e.length?ki(e,0,(t=n||t===o?1:Ba(t))<0?0:t):[]},pr.takeRight=function(e,t,n){var r=null==e?0:e.length;return r?ki(e,(t=r-(t=n||t===o?1:Ba(t)))<0?0:t,r):[]},pr.takeRightWhile=function(e,t){return e&&e.length?Ii(e,Lo(t,3),!1,!0):[]},pr.takeWhile=function(e,t){return e&&e.length?Ii(e,Lo(t,3)):[]},pr.tap=function(e,t){return t(e),e},pr.throttle=function(e,t,n){var r=!0,i=!0;if("function"!=typeof e)throw new it(u);return Aa(n)&&(r="leading"in n?!!n.leading:r,i="trailing"in n?!!n.trailing:i),ia(e,t,{leading:r,maxWait:t,trailing:i})},pr.thru=Is,pr.toArray=Ma,pr.toPairs=fu,pr.toPairsIn=hu,pr.toPath=function(e){return ma(e)?Qt(e,cs):Fa(e)?[e]:no(ls($a(e)))},pr.toPlainObject=Ua,pr.transform=function(e,t,n){var r=ma(e),i=r||wa(e)||La(e);if(t=Lo(t,4),null==n){var o=e&&e.constructor;n=i?r?new o:[]:Aa(e)&&Ta(o)?dr(Rt(e)):{}}return(i?Vt:Vr)(e,(function(e,r,i){return t(n,e,r,i)})),n},pr.unary=function(e){return ea(e,1)},pr.union=Ds,pr.unionBy=zs,pr.unionWith=ks,pr.uniq=function(e){return e&&e.length?Pi(e):[]},pr.uniqBy=function(e,t){return e&&e.length?Pi(e,Lo(t,2)):[]},pr.uniqWith=function(e,t){return t="function"==typeof t?t:o,e&&e.length?Pi(e,o,t):[]},pr.unset=function(e,t){return null==e||qi(e,t)},pr.unzip=Rs,pr.unzipWith=Ns,pr.update=function(e,t,n){return null==e?e:Mi(e,t,$i(n))},pr.updateWith=function(e,t,n,r){return r="function"==typeof r?r:o,null==e?e:Mi(e,t,$i(n),r)},pr.values=pu,pr.valuesIn=function(e){return null==e?[]:yn(e,ou(e))},pr.without=js,pr.words=Tu,pr.wrap=function(e,t){return ca($i(t),e)},pr.xor=Os,pr.xorBy=Fs,pr.xorWith=Ls,pr.zip=Ps,pr.zipObject=function(e,t){return Hi(e||[],t||[],Dr)},pr.zipObjectDeep=function(e,t){return Hi(e||[],t||[],Si)},pr.zipWith=qs,pr.entries=fu,pr.entriesIn=hu,pr.extend=Va,pr.extendWith=Ga,Ou(pr,pr),pr.add=Uu,pr.attempt=Cu,pr.camelCase=du,pr.capitalize=gu,pr.ceil=$u,pr.clamp=function(e,t,n){return n===o&&(n=t,t=o),n!==o&&(n=(n=Ha(n))==n?n:0),t!==o&&(t=(t=Ha(t))==t?t:0),Or(Ha(e),t,n)},pr.clone=function(e){return Fr(e,d)},pr.cloneDeep=function(e){return Fr(e,h|d)},pr.cloneDeepWith=function(e,t){return Fr(e,h|d,t="function"==typeof t?t:o)},pr.cloneWith=function(e,t){return Fr(e,d,t="function"==typeof t?t:o)},pr.conformsTo=function(e,t){return null==t||Lr(e,t,iu(t))},pr.deburr=vu,pr.defaultTo=function(e,t){return null==e||e!=e?t:e},pr.divide=Xu,pr.endsWith=function(e,t,n){e=$a(e),t=Li(t);var r=e.length,i=n=n===o?r:Or(Ba(n),0,r);return(n-=t.length)>=0&&e.slice(n,i)==t},pr.eq=pa,pr.escape=function(e){return(e=$a(e))&&Ce.test(e)?e.replace(Ee,En):e},pr.escapeRegExp=function(e){return(e=$a(e))&&je.test(e)?e.replace(Ne,"\\$&"):e},pr.every=function(e,t,n){var r=ma(e)?Kt:Br;return n&&Vo(e,t,n)&&(t=o),r(e,Lo(t,3))},pr.find=Hs,pr.findIndex=vs,pr.findKey=function(e,t){return sn(e,Lo(t,3),Vr)},pr.findLast=Us,pr.findLastIndex=ms,pr.findLastKey=function(e,t){return sn(e,Lo(t,3),Gr)},pr.floor=Vu,pr.forEach=$s,pr.forEachRight=Xs,pr.forIn=function(e,t){return null==e?e:$r(e,Lo(t,3),ou)},pr.forInRight=function(e,t){return null==e?e:Xr(e,Lo(t,3),ou)},pr.forOwn=function(e,t){return e&&Vr(e,Lo(t,3))},pr.forOwnRight=function(e,t){return e&&Gr(e,Lo(t,3))},pr.get=Qa,pr.gt=da,pr.gte=ga,pr.has=function(e,t){return null!=e&&Ho(e,t,ei)},pr.hasIn=eu,pr.head=_s,pr.identity=ku,pr.includes=function(e,t,n,r){e=_a(e)?e:pu(e),n=n&&!r?Ba(n):0;var i=e.length;return n<0&&(n=$n(i+n,0)),Oa(e)?n<=i&&e.indexOf(t,n)>-1:!!i&&un(e,t,n)>-1},pr.indexOf=function(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=null==n?0:Ba(n);return i<0&&(i=$n(r+i,0)),un(e,t,i)},pr.inRange=function(e,t,n){return t=Ia(t),n===o?(n=t,t=0):n=Ia(n),function(e,t,n){return e>=Xn(t,n)&&e<$n(t,n)}(e=Ha(e),t,n)},pr.invoke=ru,pr.isArguments=va,pr.isArray=ma,pr.isArrayBuffer=ya,pr.isArrayLike=_a,pr.isArrayLikeObject=ba,pr.isBoolean=function(e){return!0===e||!1===e||Da(e)&&Jr(e)==U},pr.isBuffer=wa,pr.isDate=xa,pr.isElement=function(e){return Da(e)&&1===e.nodeType&&!Ra(e)},pr.isEmpty=function(e){if(null==e)return!0;if(_a(e)&&(ma(e)||"string"==typeof e||"function"==typeof e.splice||wa(e)||La(e)||va(e)))return!e.length;var t=Wo(e);if(t==Y||t==ne)return!e.size;if(Zo(e))return!li(e).length;for(var n in e)if(ct.call(e,n))return!1;return!0},pr.isEqual=function(e,t){return oi(e,t)},pr.isEqualWith=function(e,t,n){var r=(n="function"==typeof n?n:o)?n(e,t):o;return r===o?oi(e,t,o,n):!!r},pr.isError=Ea,pr.isFinite=function(e){return"number"==typeof e&&Wn(e)},pr.isFunction=Ta,pr.isInteger=Ca,pr.isLength=Sa,pr.isMap=za,pr.isMatch=function(e,t){return e===t||si(e,t,qo(t))},pr.isMatchWith=function(e,t,n){return n="function"==typeof n?n:o,si(e,t,qo(t),n)},pr.isNaN=function(e){return ka(e)&&e!=+e},pr.isNative=function(e){if(Yo(e))throw new Je(a);return ai(e)},pr.isNil=function(e){return null==e},pr.isNull=function(e){return null===e},pr.isNumber=ka,pr.isObject=Aa,pr.isObjectLike=Da,pr.isPlainObject=Ra,pr.isRegExp=Na,pr.isSafeInteger=function(e){return Ca(e)&&e>=-O&&e<=O},pr.isSet=ja,pr.isString=Oa,pr.isSymbol=Fa,pr.isTypedArray=La,pr.isUndefined=function(e){return e===o},pr.isWeakMap=function(e){return Da(e)&&Wo(e)==se},pr.isWeakSet=function(e){return Da(e)&&Jr(e)==ae},pr.join=function(e,t){return null==e?"":Hn.call(e,t)},pr.kebabCase=mu,pr.last=Es,pr.lastIndexOf=function(e,t,n){var r=null==e?0:e.length;if(!r)return-1;var i=r;return n!==o&&(i=(i=Ba(n))<0?$n(r+i,0):Xn(i,r-1)),t==t?function(e,t,n){for(var r=n+1;r--;)if(e[r]===t)return r;return r}(e,t,i):an(e,cn,i,!0)},pr.lowerCase=yu,pr.lowerFirst=_u,pr.lt=Pa,pr.lte=qa,pr.max=function(e){return e&&e.length?Wr(e,ku,Qr):o},pr.maxBy=function(e,t){return e&&e.length?Wr(e,Lo(t,2),Qr):o},pr.mean=function(e){return fn(e,ku)},pr.meanBy=function(e,t){return fn(e,Lo(t,2))},pr.min=function(e){return e&&e.length?Wr(e,ku,fi):o},pr.minBy=function(e,t){return e&&e.length?Wr(e,Lo(t,2),fi):o},pr.stubArray=Wu,pr.stubFalse=Hu,pr.stubObject=function(){return{}},pr.stubString=function(){return""},pr.stubTrue=function(){return!0},pr.multiply=Ku,pr.nth=function(e,t){return e&&e.length?vi(e,Ba(t)):o},pr.noConflict=function(){return jt._===this&&(jt._=gt),this},pr.noop=Fu,pr.now=Qs,pr.pad=function(e,t,n){e=$a(e);var r=(t=Ba(t))?Rn(e):0;if(!t||r>=t)return e;var i=(t-r)/2;return yo(Mn(i),n)+e+yo(qn(i),n)},pr.padEnd=function(e,t,n){e=$a(e);var r=(t=Ba(t))?Rn(e):0;return t&&r<t?e+yo(t-r,n):e},pr.padStart=function(e,t,n){e=$a(e);var r=(t=Ba(t))?Rn(e):0;return t&&r<t?yo(t-r,n)+e:e},pr.parseInt=function(e,t,n){return n||null==t?t=0:t&&(t=+t),Gn($a(e).replace(Fe,""),t||0)},pr.random=function(e,t,n){if(n&&"boolean"!=typeof n&&Vo(e,t,n)&&(t=n=o),n===o&&("boolean"==typeof t?(n=t,t=o):"boolean"==typeof e&&(n=e,e=o)),e===o&&t===o?(e=0,t=1):(e=Ia(e),t===o?(t=e,e=0):t=Ia(t)),e>t){var r=e;e=t,t=r}if(n||e%1||t%1){var i=Kn();return Xn(e+i*(t-e+zt("1e-"+((i+"").length-1))),t)}return wi(e,t)},pr.reduce=function(e,t,n){var r=ma(e)?tn:dn,i=arguments.length<3;return r(e,Lo(t,4),n,i,Mr)},pr.reduceRight=function(e,t,n){var r=ma(e)?nn:dn,i=arguments.length<3;return r(e,Lo(t,4),n,i,Ir)},pr.repeat=function(e,t,n){return t=(n?Vo(e,t,n):t===o)?1:Ba(t),xi($a(e),t)},pr.replace=function(){var e=arguments,t=$a(e[0]);return e.length<3?t:t.replace(e[1],e[2])},pr.result=function(e,t,n){var r=-1,i=(t=Xi(t,e)).length;for(i||(i=1,e=o);++r<i;){var s=null==e?o:e[cs(t[r])];s===o&&(r=i,s=n),e=Ta(s)?s.call(e):s}return e},pr.round=Yu,pr.runInContext=e,pr.sample=function(e){return(ma(e)?Tr:Ti)(e)},pr.size=function(e){if(null==e)return 0;if(_a(e))return Oa(e)?Rn(e):e.length;var t=Wo(e);return t==Y||t==ne?e.size:li(e).length},pr.snakeCase=bu,pr.some=function(e,t,n){var r=ma(e)?rn:Ri;return n&&Vo(e,t,n)&&(t=o),r(e,Lo(t,3))},pr.sortedIndex=function(e,t){return Ni(e,t)},pr.sortedIndexBy=function(e,t,n){return ji(e,t,Lo(n,2))},pr.sortedIndexOf=function(e,t){var n=null==e?0:e.length;if(n){var r=Ni(e,t);if(r<n&&pa(e[r],t))return r}return-1},pr.sortedLastIndex=function(e,t){return Ni(e,t,!0)},pr.sortedLastIndexBy=function(e,t,n){return ji(e,t,Lo(n,2),!0)},pr.sortedLastIndexOf=function(e,t){if(null==e?0:e.length){var n=Ni(e,t,!0)-1;if(pa(e[n],t))return n}return-1},pr.startCase=wu,pr.startsWith=function(e,t,n){return e=$a(e),n=null==n?0:Or(Ba(n),0,e.length),t=Li(t),e.slice(n,n+t.length)==t},pr.subtract=Zu,pr.sum=function(e){return e&&e.length?gn(e,ku):0},pr.sumBy=function(e,t){return e&&e.length?gn(e,Lo(t,2)):0},pr.template=function(e,t,n){var r=pr.templateSettings;n&&Vo(e,t,n)&&(t=o),e=$a(e),t=Ga({},t,r,So);var i,s,a=Ga({},t.imports,r.imports,So),u=iu(a),l=yn(a,u),c=0,f=t.interpolate||Ye,h="__p += '",p=nt((t.escape||Ye).source+"|"+f.source+"|"+(f===De?We:Ye).source+"|"+(t.evaluate||Ye).source+"|$","g"),d="//# sourceURL="+(ct.call(t,"sourceURL")?(t.sourceURL+"").replace(/[\r\n]/g," "):"lodash.templateSources["+ ++Ct+"]")+"\n";e.replace(p,(function(t,n,r,o,a,u){return r||(r=o),h+=e.slice(c,u).replace(Ze,Tn),n&&(i=!0,h+="' +\n__e("+n+") +\n'"),a&&(s=!0,h+="';\n"+a+";\n__p += '"),r&&(h+="' +\n((__t = ("+r+")) == null ? '' : __t) +\n'"),c=u+t.length,t})),h+="';\n";var g=ct.call(t,"variable")&&t.variable;g||(h="with (obj) {\n"+h+"\n}\n"),h=(s?h.replace(_e,""):h).replace(be,"$1").replace(we,"$1;"),h="function("+(g||"obj")+") {\n"+(g?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(i?", __e = _.escape":"")+(s?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+h+"return __p\n}";var v=Cu((function(){return Qe(u,d+"return "+h).apply(o,l)}));if(v.source=h,Ea(v))throw v;return v},pr.times=function(e,t){if((e=Ba(e))<1||e>O)return[];var n=P,r=Xn(e,P);t=Lo(t),e-=P;for(var i=vn(r,t);++n<e;)t(n);return i},pr.toFinite=Ia,pr.toInteger=Ba,pr.toLength=Wa,pr.toLower=function(e){return $a(e).toLowerCase()},pr.toNumber=Ha,pr.toSafeInteger=function(e){return e?Or(Ba(e),-O,O):0===e?e:0},pr.toString=$a,pr.toUpper=function(e){return $a(e).toUpperCase()},pr.trim=function(e,t,n){if((e=$a(e))&&(n||t===o))return e.replace(Oe,"");if(!e||!(t=Li(t)))return e;var r=Nn(e),i=Nn(t);return Gi(r,bn(r,i),wn(r,i)+1).join("")},pr.trimEnd=function(e,t,n){if((e=$a(e))&&(n||t===o))return e.replace(Le,"");if(!e||!(t=Li(t)))return e;var r=Nn(e);return Gi(r,0,wn(r,Nn(t))+1).join("")},pr.trimStart=function(e,t,n){if((e=$a(e))&&(n||t===o))return e.replace(Fe,"");if(!e||!(t=Li(t)))return e;var r=Nn(e);return Gi(r,bn(r,Nn(t))).join("")},pr.truncate=function(e,t){var n=A,r=D;if(Aa(t)){var i="separator"in t?t.separator:i;n="length"in t?Ba(t.length):n,r="omission"in t?Li(t.omission):r}var s=(e=$a(e)).length;if(Cn(e)){var a=Nn(e);s=a.length}if(n>=s)return e;var u=n-Rn(r);if(u<1)return r;var l=a?Gi(a,0,u).join(""):e.slice(0,u);if(i===o)return l+r;if(a&&(u+=l.length-u),Na(i)){if(e.slice(u).search(i)){var c,f=l;for(i.global||(i=nt(i.source,$a(He.exec(i))+"g")),i.lastIndex=0;c=i.exec(f);)var h=c.index;l=l.slice(0,h===o?u:h)}}else if(e.indexOf(Li(i),u)!=u){var p=l.lastIndexOf(i);p>-1&&(l=l.slice(0,p))}return l+r},pr.unescape=function(e){return(e=$a(e))&&Te.test(e)?e.replace(xe,jn):e},pr.uniqueId=function(e){var t=++ft;return $a(e)+t},pr.upperCase=xu,pr.upperFirst=Eu,pr.each=$s,pr.eachRight=Xs,pr.first=_s,Ou(pr,(Gu={},Vr(pr,(function(e,t){ct.call(pr.prototype,t)||(Gu[t]=e)})),Gu),{chain:!1}),pr.VERSION="4.17.15",Vt(["bind","bindKey","curry","curryRight","partial","partialRight"],(function(e){pr[e].placeholder=pr})),Vt(["drop","take"],(function(e,t){mr.prototype[e]=function(n){n=n===o?1:$n(Ba(n),0);var r=this.__filtered__&&!t?new mr(this):this.clone();return r.__filtered__?r.__takeCount__=Xn(n,r.__takeCount__):r.__views__.push({size:Xn(n,P),type:e+(r.__dir__<0?"Right":"")}),r},mr.prototype[e+"Right"]=function(t){return this.reverse()[e](t).reverse()}})),Vt(["filter","map","takeWhile"],(function(e,t){var n=t+1,r=n==R||3==n;mr.prototype[e]=function(e){var t=this.clone();return t.__iteratees__.push({iteratee:Lo(e,3),type:n}),t.__filtered__=t.__filtered__||r,t}})),Vt(["head","last"],(function(e,t){var n="take"+(t?"Right":"");mr.prototype[e]=function(){return this[n](1).value()[0]}})),Vt(["initial","tail"],(function(e,t){var n="drop"+(t?"":"Right");mr.prototype[e]=function(){return this.__filtered__?new mr(this):this[n](1)}})),mr.prototype.compact=function(){return this.filter(ku)},mr.prototype.find=function(e){return this.filter(e).head()},mr.prototype.findLast=function(e){return this.reverse().find(e)},mr.prototype.invokeMap=Ei((function(e,t){return"function"==typeof e?new mr(this):this.map((function(n){return ri(n,e,t)}))})),mr.prototype.reject=function(e){return this.filter(ua(Lo(e)))},mr.prototype.slice=function(e,t){e=Ba(e);var n=this;return n.__filtered__&&(e>0||t<0)?new mr(n):(e<0?n=n.takeRight(-e):e&&(n=n.drop(e)),t!==o&&(n=(t=Ba(t))<0?n.dropRight(-t):n.take(t-e)),n)},mr.prototype.takeRightWhile=function(e){return this.reverse().takeWhile(e).reverse()},mr.prototype.toArray=function(){return this.take(P)},Vr(mr.prototype,(function(e,t){var n=/^(?:filter|find|map|reject)|While$/.test(t),r=/^(?:head|last)$/.test(t),i=pr[r?"take"+("last"==t?"Right":""):t],s=r||/^find/.test(t);i&&(pr.prototype[t]=function(){var t=this.__wrapped__,a=r?[1]:arguments,u=t instanceof mr,l=a[0],c=u||ma(t),f=function(e){var t=i.apply(pr,en([e],a));return r&&h?t[0]:t};c&&n&&"function"==typeof l&&1!=l.length&&(u=c=!1);var h=this.__chain__,p=!!this.__actions__.length,d=s&&!h,g=u&&!p;if(!s&&c){t=g?t:new mr(this);var v=e.apply(t,a);return v.__actions__.push({func:Is,args:[f],thisArg:o}),new vr(v,h)}return d&&g?e.apply(this,a):(v=this.thru(f),d?r?v.value()[0]:v.value():v)})})),Vt(["pop","push","shift","sort","splice","unshift"],(function(e){var t=ot[e],n=/^(?:push|sort|unshift)$/.test(e)?"tap":"thru",r=/^(?:pop|shift)$/.test(e);pr.prototype[e]=function(){var e=arguments;if(r&&!this.__chain__){var i=this.value();return t.apply(ma(i)?i:[],e)}return this[n]((function(n){return t.apply(ma(n)?n:[],e)}))}})),Vr(mr.prototype,(function(e,t){var n=pr[t];if(n){var r=n.name+"";ct.call(ir,r)||(ir[r]=[]),ir[r].push({name:t,func:n})}})),ir[po(o,y).name]=[{name:"wrapper",func:o}],mr.prototype.clone=function(){var e=new mr(this.__wrapped__);return e.__actions__=no(this.__actions__),e.__dir__=this.__dir__,e.__filtered__=this.__filtered__,e.__iteratees__=no(this.__iteratees__),e.__takeCount__=this.__takeCount__,e.__views__=no(this.__views__),e},mr.prototype.reverse=function(){if(this.__filtered__){var e=new mr(this);e.__dir__=-1,e.__filtered__=!0}else(e=this.clone()).__dir__*=-1;return e},mr.prototype.value=function(){var e=this.__wrapped__.value(),t=this.__dir__,n=ma(e),r=t<0,i=n?e.length:0,o=function(e,t,n){var r=-1,i=n.length;for(;++r<i;){var o=n[r],s=o.size;switch(o.type){case"drop":e+=s;break;case"dropRight":t-=s;break;case"take":t=Xn(t,e+s);break;case"takeRight":e=$n(e,t-s)}}return{start:e,end:t}}(0,i,this.__views__),s=o.start,a=o.end,u=a-s,l=r?a:s-1,c=this.__iteratees__,f=c.length,h=0,p=Xn(u,this.__takeCount__);if(!n||!r&&i==u&&p==u)return Bi(e,this.__actions__);var d=[];e:for(;u--&&h<p;){for(var g=-1,v=e[l+=t];++g<f;){var m=c[g],y=m.iteratee,_=m.type,b=y(v);if(_==N)v=b;else if(!b){if(_==R)continue e;break e}}d[h++]=v}return d},pr.prototype.at=Bs,pr.prototype.chain=function(){return Ms(this)},pr.prototype.commit=function(){return new vr(this.value(),this.__chain__)},pr.prototype.next=function(){this.__values__===o&&(this.__values__=Ma(this.value()));var e=this.__index__>=this.__values__.length;return{done:e,value:e?o:this.__values__[this.__index__++]}},pr.prototype.plant=function(e){for(var t,n=this;n instanceof gr;){var r=hs(n);r.__index__=0,r.__values__=o,t?i.__wrapped__=r:t=r;var i=r;n=n.__wrapped__}return i.__wrapped__=e,t},pr.prototype.reverse=function(){var e=this.__wrapped__;if(e instanceof mr){var t=e;return this.__actions__.length&&(t=new mr(this)),(t=t.reverse()).__actions__.push({func:Is,args:[As],thisArg:o}),new vr(t,this.__chain__)}return this.thru(As)},pr.prototype.toJSON=pr.prototype.valueOf=pr.prototype.value=function(){return Bi(this.__wrapped__,this.__actions__)},pr.prototype.first=pr.prototype.head,qt&&(pr.prototype[qt]=function(){return this}),pr}();jt._=On,(i=function(){return On}.call(t,n,t,r))===o||(r.exports=i)}).call(this)}).call(this,n(8),n(9)(e))},function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(e){"object"==typeof window&&(n=window)}e.exports=n},function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},function(e,t,n){var r=n(0),i=n(2),o=n(11),s=n(15),a=function(){this.events=s,this.gloo=o,this._is_loop_running=!1,this._canvases=[]};a.prototype.init=function(e){var t;t=r(e);var n=new i(t);return n.deactivate_context_menu(),this.events.init(n),this.gloo.init(n),this.register(n),n},a.prototype.register=function(e){this._canvases.push(e)},a.prototype.unregister=function(e){var t=this._canvases.indexOf(e);t>-1&&this._canvases.splice(t,1)},a.prototype.start_event_loop=function(){if(!this._is_loop_running){window.requestAnimFrame=window.requestAnimationFrame||window.webkitRequestAnimationFrame||window.mozRequestAnimationFrame||function(e){window.setTimeout(e,1e3/60)};var e=this;!function t(){e._request_id=requestAnimFrame(t);try{for(var n=0;n<e._canvases.length;n++)e._canvases[n].event_tick()}catch(t){throw e.stop_event_loop(),t}}(),this._is_loop_running=!0,console.debug("Event loop started.")}},a.prototype.stop_event_loop=function(){window.cancelAnimationFrame(this._request_id),this._is_loop_running=!1,console.debug("Event loop stopped.")},e.exports=new a},function(e,t,n){var r=n(12);var i=function(){this.glir=r};i.prototype.init=function(e){!function(e){var t=e.$el.get(0);e.gl=t.getContext("webgl")||t.getContext("experimental-webgl");var n=e.gl.getExtension("OES_standard_derivatives")||e.gl.getExtension("MOZ_OES_standard_derivatives")||e.gl.getExtension("WEBKIT_OES_standard_derivatives");null===n&&console.warn("Extension 'OES_standard_derivatives' is not supported in this browser. Some features may not work as expected"),null===(n=e.gl.getExtension("OES_element_index_uint")||e.gl.getExtension("MOZ_OES_element_index_uint")||e.gl.getExtension("WEBKIT_OES_element_index_uint"))&&console.warn("Extension 'OES_element_index_uint' is not supported in this browser. Some features may not work as expected"),null===(n=e.gl.getExtension("OES_texture_float"))&&console.warn("Extension 'OES_texture_float' is not supported in this browser. Some features may not work as expected"),null===(n=e.gl.getExtension("OES_texture_float_linear"))&&console.warn("Extension 'OES_texture_float_linear' is not supported in this browser. Some features may not work as expected")}(e),this.glir.init(e)},e.exports=new i},function(e,t,n){var r=n(2),i=n(13),o=n(14),s=i.debug,a=o.to_array_buffer,u="JUST_DELETED";function l(e,t,n,r,i,o){var s=function(e){return v[e]}(r),a=s[0],u=s[1];_vbo_info=e._ns[n];var l=_vbo_info.handle;e.gl.enableVertexAttribArray(t),e.gl.bindBuffer(e.gl.ARRAY_BUFFER,l),e.gl.vertexAttribPointer(t,u,e.gl[a],!1,i,o)}function c(e,t){e.gl.disableVertexAttribArray(t)}function f(e,t,n,r){t!==u&&(e.gl.activeTexture(e.gl.TEXTURE0+r),e.gl.bindTexture(e.gl.TEXTURE_2D,t))}function h(e,t,n,r){e.gl.activeTexture(e.gl.TEXTURE0+r),e.gl.bindTexture(e.gl.TEXTURE_2D,null)}function p(e,t,n,r,i,o,s,a,u,l){if(e.gl.bindTexture(n,t),e.gl.pixelStorei(e.gl.UNPACK_ALIGNMENT,1),null===s)e.gl.texImage2D(n,0,r,i,o,0,r,e.gl.UNSIGNED_BYTE,s);else if(s.getContext)e.gl.texImage2D(n,0,e.gl.RGBA,e.gl.RGBA,e.gl.UNSIGNED_BYTE,s);else if(s.canvas)e.gl.texImage2D(n,0,e.gl.RGBA,e.gl.RGBA,e.gl.UNSIGNED_BYTE,s.canvas);else{var c;if(c=l==e.gl.FLOAT?new Float32Array(s):new Uint8Array(s),a&&u&&(u[0]!==o||u[1]!==i)){var f=function(e){for(var t=[8,4,2,1],n=0;n<t.length;n++)if(e%t[n]==0)return t[n]}(u[u.length-2]*u[u.length-1]);e.gl.pixelStorei(e.gl.UNPACK_ALIGNMENT,f),e.gl.texSubImage2D(n,0,a[1],a[0],u[1],u[0],r,l,c)}else e.gl.pixelStorei(e.gl.UNPACK_ALIGNMENT,1),e.gl.texImage2D(n,0,r,i,o,0,r,l,c)}}function d(e,t,n,r,i,o){e.gl.bindBuffer(n,t),o?e.gl.bufferSubData(n,r,i):e.gl.bufferData(n,i,e.gl.STATIC_DRAW)}var g={float32:"FLOAT",uint8:"UNSIGNED_BYTE"};var v={float:["FLOAT",1],vec2:["FLOAT",2],vec3:["FLOAT",3],vec4:["FLOAT",4],int:["INT",1],ivec2:["INT",2],ivec3:["INT",3],ivec4:["INT",4]};var m={float:"uniform1fv",vec2:"uniform2fv",vec3:"uniform3fv",vec4:"uniform4fv",int:"uniform1iv",ivec2:"uniform2iv",ivec3:"uniform3iv",ivec4:"uniform4iv",mat2:"uniformMatrix2fv",mat3:"uniformMatrix3fv",mat4:"uniformMatrix4fv"};var y={VertexBuffer:"ARRAY_BUFFER",IndexBuffer:"ELEMENT_ARRAY_BUFFER",Texture2D:"TEXTURE_2D"};function _(e){return y[e]}var b={color:["COLOR_ATTACHMENT0","RGBA4"],depth:["DEPTH_ATTACHMENT","DEPTH_COMPONENT16"],stencil:["STENCIL_ATTACHMENT","STENCIL_INDEX8"]};function w(e,t){for(var n=t.split("|"),r=0,i=0;i<n.length;i++){var o=n[i].toUpperCase().trim();r|=e.gl[o]}return r}function x(e,t){var n=e._ns[t],r=e.env.fb_stack;0==r.length&&r.push(null),r[r.length-1]!==n.handle?(s("Binding frame buffer {0}.".format(t)),e.gl.bindFramebuffer(e.gl.FRAMEBUFFER,n.handle),r.push(n.handle)):s("Frame buffer already active {0}".format(t))}function E(e,t){var n=e._ns[t],r=e.env.fb_stack;for(0==r.length&&r.push(null);r[r.length-1]===n.handle;)r.pop();s("Binding previous frame buffer"),e.gl.bindFramebuffer(e.gl.FRAMEBUFFER,r[r.length-1])}function T(e){e.env={fb_stack:[]}}function C(){this._queue=[]}C.prototype.clear=function(){this._queue=[]},C.prototype.append=function(e){this._queue.push(e)},C.prototype.append_multi=function(e){for(var t=0;t<e.length;t++)this._queue.push(e[t])},C.prototype.get=function(){return this._queue},Object.defineProperty(C.prototype,"length",{get:function(){return this._queue.length}}),r.prototype.set_deferred=function(e){this._deferred=e},r.prototype.execute_pending_commands=function(){var e=this.glir_queue.get(),t=-1;if(0==e.length)return 0;for(var n=0;n<e.length;n++)if("SWAP"===e[n][0]){t=n;break}for(n=0;n<=t;n++)this.command(e[n],!1);return t>=0&&(s("Processed {0} events.".format(t+1)),this.glir_queue._queue=this.glir_queue._queue.slice(t+1)),t+1},r.prototype.command=function(e,t){void 0===t&&(t=this._deferred);var n=e[0].toLowerCase();t?this.glir_queue.append(e):this.glir[n](this,e.slice(1))};var S=function(){};S.prototype.init=function(e){e._ns={},e._deferred=!0,T(e),e.glir_queue=new C,e.glir=this},S.prototype.current=function(e,t){T(e),e.gl.bindFramebuffer(e.gl.FRAMEBUFFER,null)},S.prototype.swap=function(e,t){},S.prototype.create=function(e,t){var n=t[0],r=t[1];"VertexBuffer"==r?(s("Creating vertex buffer '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createBuffer(),size:0}):"IndexBuffer"==r?(s("Creating index buffer '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createBuffer(),size:0}):"FrameBuffer"==r?(s("Creating frame buffer '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createFramebuffer(),size:0,validated:!1}):"RenderBuffer"==r?(s("Creating render buffer '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createRenderbuffer(),size:0}):"Texture2D"==r?(s("Creating texture '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createTexture(),size:0,shape:[]}):"Program"==r?(s("Creating program '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createProgram(),attributes:{},_handles_cache:{},textures:{},texture_uniforms:{},shaders:{}}):"VertexShader"==r?(s("Creating VertexShader '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createShader(e.gl.VERTEX_SHADER)}):"FragmentShader"==r&&(s("Creating FragmentShader '{0}'.".format(n)),e._ns[n]={object_type:r,handle:e.gl.createShader(e.gl.FRAGMENT_SHADER)})},S.prototype.delete=function(e,t){var n=t[0],r=e._ns[n].object_type,i=e._ns[n].handle;e._ns[n].handle=u,"VertexBuffer"==r?(s("Deleting vertex buffer '{0}'.".format(n)),e.gl.deleteBuffer(i)):"IndexBuffer"==r?(s("Deleting index buffer '{0}'.".format(n)),e.gl.deleteBuffer(i)):"FrameBuffer"==r?(s("Deleting frame buffer '{0}'.".format(n)),e.gl.deleteFramebuffer(i)):"RenderBuffer"==r?(s("Deleting render buffer '{0}'.".format(n)),e.gl.deleteRenderbuffer(i)):"Texture2D"==r?(s("Deleting texture '{0}'.".format(n)),e.gl.deleteTexture(i)):"Program"==r?(s("Deleting program '{0}'.".format(n)),e.gl.deleteProgram(i)):r.indexOf("Shader")>=0&&(s("Deleting shader '{0}'.".format(n)),e.gl.deleteShader(i))},S.prototype.size=function(e,t){var n,r=t[0],i=t[1],o=t[2],a=e._ns[r],u=a.handle,l=a.object_type,c=e.gl[_(l)];l.indexOf("Texture")>=0?(a.format=o.toUpperCase(),s("Setting texture size to {1} for '{0}'.".format(r,i))):"RenderBuffer"==l?(e.gl.bindRenderbuffer(e.gl.RENDERBUFFER,u),a.format=e.gl[(n=o,b[n][1])],e.gl.renderbufferStorage(e.gl.RENDERBUFFER,a.format,i[1],i[0]),e.gl.bindRenderbuffer(e.gl.RENDERBUFFER,null)):(s("Setting buffer size to {1} for '{0}'.".format(r,i)),d(e,u,c,0,i,!1)),a.size=i},S.prototype.data=function(e,t){var n,r=t[0],i=t[1],o=t[2],u=e._ns[r],l=u.object_type,c=u.handle,f=e.gl[_(l)],h=a(o);if(l.indexOf("Shader")>=0)!function(e,t,n){"string"!=typeof n&&(n=String.fromCharCode.apply(null,new Uint8Array(n))),n=n.replace(/\\n/g,"\n"),e.gl.shaderSource(t,n),e.gl.compileShader(t),!!e.gl.getShaderParameter(t,e.gl.COMPILE_STATUS)||console.error(e.gl.getShaderInfoLog(t))}(e,c,h);else if(l.indexOf("Texture")>=0){var v=u.size,m=v[0],y=v[1],b=e.gl[u.format];s("Setting texture data for '{0}'.".format(r));var w=e.gl[(n=o.dtype,g[n])];p(e,c,f,b,y,m,h,i,o.shape,w),u.shape=v}else s("Setting buffer data for '{0}'.".format(r)),d(e,c,f,i,h,u.size>0),u.size=h.byteLength},S.prototype.attribute=function(e,t){var n=t[0],r=t[1],i=t[2],o=t[3][0],a=t[3][1],u=t[3][2],l=e._ns[n],c=l.handle;s("Creating attribute '{0}' for program '{1}'.".format(r,n));var f=function(e,t,n){return e.gl.getAttribLocation(t,n)}(e,c,r);l.attributes[r]={handle:f,type:i,vbo_id:o,stride:a,offset:u}},S.prototype.uniform=function(e,t){var n,r,i,o=t[0],u=t[1],l=t[2],c=t[3],f=e._ns[o],h=f.handle;void 0===f._handles_cache[u]&&(s("Creating uniform '{0}' for program '{1}'.".format(u,o)),n=e.gl.getUniformLocation(h,u),r=function(e){return m[e]}(l),f._handles_cache[u]=[n,r]),e.gl.useProgram(h),s("Setting uniform '{0}' to '{1}' with {2} elements.".format(u,c,c.length)),function(e,t,n,r){array=a(r),n.indexOf("Matrix")>0?e.gl[n](t,!1,array):e.gl[n](t,array)}(e,n=(i=e._ns[o]._handles_cache[u])[0],r=i[1],c)},S.prototype.texture=function(e,t){var n=t[0],r=t[1],i=t[2],o=e._ns[n],a=o.handle,l=e._ns[i].handle;if(l===u)return s("Removing texture '{0}' from program '{1}'".format(i,n)),void delete o.textures[i];s("Initializing texture '{0}' for program '{1}'.".format(i,n)),o.texture_uniforms.hasOwnProperty(r)&&(s("Removing previously assigned texture for '{0}'".format(r)),delete o.textures[o.texture_uniforms[r]]);var c=e.gl.getUniformLocation(a,r);o.texture_uniforms[r]=i,e._ns[n].textures[i]={sampler_name:r,sampler_handle:c,number:-1,handle:l}},S.prototype.interpolation=function(e,t){var n=t[0],r=t[1].toUpperCase(),i=t[2].toUpperCase(),o=e._ns[n].handle,s=e.gl.TEXTURE_2D;e.gl.bindTexture(s,o),e.gl.texParameteri(s,e.gl.TEXTURE_MIN_FILTER,e.gl[r]),e.gl.texParameteri(s,e.gl.TEXTURE_MAG_FILTER,e.gl[i]),e.gl.bindTexture(s,null)},S.prototype.wrapping=function(e,t){var n=t[0],r=t[1],i=e._ns[n].handle,o=e.gl.TEXTURE_2D;e.gl.bindTexture(o,i),e.gl.texParameteri(o,e.gl.TEXTURE_WRAP_S,e.gl[r[0].toUpperCase()]),e.gl.texParameteri(o,e.gl.TEXTURE_WRAP_T,e.gl[r[1].toUpperCase()]),e.gl.bindTexture(o,null)},S.prototype.draw=function(e,t){var n,r,i,o,a=t[0],p=t[1].toUpperCase(),d=t[2],g=e._ns[a].handle,v=e._ns[a].attributes,m=e._ns[a].textures,y=0;for(r in e.gl.useProgram(g),v){var _=v[r];s("Activating attribute '{0}' for program '{1}'.".format(r,a)),l(e,_.handle,_.vbo_id,_.type,_.stride,_.offset)}for(i in m)n=m[i],e._ns[i].handle!==u?(n.number=y,y+=1,s("Activating texture '{0}' for program '{1}' as number '{2}'.".format(i,a,n.number)),f(e,n.handle,n.sampler_handle,n.number),e.gl.uniform1i(n.sampler_handle,n.number)):(s("Ignoring texture '{0}' from program '{1}'".format(i,a)),n.handle=u);if(2==d.length){var b=d[0];o=d[1],s("Rendering program '{0}' with {1}.".format(a,p)),e.gl.drawArrays(e.gl[p],b,o)}else if(3==d.length){var w=d[0],x=d[1];o=d[2];var E=e._ns[w].handle;s("Rendering program '{0}' with {1} and index buffer '{2}' of type '{3}'.".format(a,p,w,x)),e.gl.bindBuffer(e.gl.ELEMENT_ARRAY_BUFFER,E),e.gl.drawElements(e.gl[p],o,e.gl[x],0)}for(r in v)s("Deactivating attribute '{0}' for program '{1}'.".format(r,a)),c(e,v[r].handle);var T={};for(i in m)n=m[i],s("Deactivating texture '{0}' for program '{1}'.".format(i,a)),h(e,n.handle,n.sampler_handle,n.number),e._ns[i].handle!=u&&(T[i]=n);e._ns[a].textures=T},S.prototype.attach=function(e,t){var n=t[0],r=e._ns[n],i=r.object_type,o=r.handle;if("Program"==i){var a=t[1],u=e._ns[a].handle;return r.shaders[a]=u,void e.gl.attachShader(o,u)}var l,c,f=t[2],h=e.gl[(l=t[1],b[l][0])];x(e,n),0==f?(s("Attaching RenderBuffer object {0} to framebuffer {1}".format(f,n)),e.gl.framebufferRenderbuffer(e.gl.FRAMEBUFFER,h,e.gl.RENDERBUFFER,null)):(c=e._ns[f],s("Attaching {0} object {1} to framebuffer {2} for {3}".format(c.object_type,f,n,t[1])),"RenderBuffer"==c.object_type?(e.gl.bindRenderbuffer(e.gl.RENDERBUFFER,c.handle),e.gl.framebufferRenderbuffer(e.gl.FRAMEBUFFER,h,e.gl.RENDERBUFFER,c.handle),e.gl.bindRenderbuffer(e.gl.RENDERBUFFER,null)):"Texture2D"==c.object_type&&(0==c.shape.length&&(s("Setting empty texture data to unset texture before attaching to framebuffer"),p(e,c.handle,e.gl.TEXTURE_2D,e.gl[c.format],c.size[1],c.size[0],null)),e.gl.bindTexture(e.gl.TEXTURE_2D,c.handle),e.gl.framebufferTexture2D(e.gl.FRAMEBUFFER,h,e.gl.TEXTURE_2D,c.handle,0),e.gl.bindTexture(e.gl.TEXTURE_2D,null))),e._ns[n].validated=!1,E(e,n)},S.prototype.link=function(e,t){var n,r=t[0],i=e._ns[r],o=i.handle,s=i.shaders;for(n in e.gl.linkProgram(o),e.gl.getProgramParameter(o,e.gl.LINK_STATUS)||console.warn("Could not initialise shaders on program '{0}'.".format(o)),s)e.gl.detachShader(o,s[n]);i.shaders={},i._handles_cache={}},S.prototype.framebuffer=function(e,t){var n=t[0],r=t[1],i=e._ns[n];r?(s("Binding framebuffer {0}".format(n)),x(e,n),i.validated||(i.validated=!0,function(e){var t=e.gl.checkFramebufferStatus(e.gl.FRAMEBUFFER);if(t!=e.gl.FRAMEBUFFER_COMPLETE)throw t==e.gl.FRAMEBUFFER_INCOMPLETE_ATTACHMENT?"FrameBuffer attachments are incomplete.":t==e.gl.FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT?"No valid attachments in the FrameBuffer.":t==e.gl.FRAMEBUFFER_INCOMPLETE_DIMENSIONS?"attachments do not have the same width and height.":t==e.gl.FRAMEBUFFER_UNSUPPORTED?"Combination of internal formats used by attachments is not supported.":"Unknown framebuffer error"+t}(e))):(s("Unbinding framebuffer {0}".format(n)),E(e,n))},S.prototype.func=function(e,t){var n=t[0];s("Calling {0}({1}).".format(n,t.slice(1)));for(var r=1;r<t.length;r++)"string"==typeof t[r]&&(t[r]=w(e,t[r]));var i=e.gl[n],o=t.slice(1);i.apply(e.gl,o)},e.exports=new S},function(e,t){String.prototype.format||(String.prototype.format=function(){var e=arguments;return this.replace(/{(\d+)}/g,(function(t,n){return void 0!==e[n]?e[n]:t}))}),void 0===String.prototype.trim&&(String.prototype.trim=function(){return String(this).replace(/^\s+|\s+$/g,"")}),Array.prototype.equals=function(e){if(!e)return!1;if(this.length!=e.length)return!1;for(var t=0,n=this.length;t<n;t++)if(this[t]instanceof Array&&e[t]instanceof Array){if(!this[t].equals(e[t]))return!1}else if(this[t]!=e[t])return!1;return!0},"function"!=typeof String.prototype.startsWith&&(String.prototype.startsWith=function(e){return this.slice(0,e.length)==e}),window.VISPY_DEBUG=!1,e.exports={debug:function(e){window.VISPY_DEBUG&&console.debug(e)}}},function(e,t){var n={float32:Float32Array,int8:Int8Array,int16:Int16Array,int32:Int32Array,uint8:Uint8Array,uint16:Uint16Array,uint32:Uint32Array};e.exports={to_array_buffer:function(e){var t=e.storage_type;if(null==t)return e;var r=e.data_type,i=e.buffer;return"javascript_array"==t?n[r](i):"javascript_typed_array"==t||"array_buffer"==t?i:"binary"==t?i.buffer:"base64"==t?function(e){for(var t=window.atob(e),n=t.length,r=new Uint8Array(n),i=0;i<n;i++){var o=t.charCodeAt(i);r[i]=o}return r.buffer}(i):void 0}}},function(e,t,n){var r=n(2),i=n(16);function o(e){var t=[];return e.altKey&&t.push("alt"),e.ctrlKey&&t.push("ctrl"),e.metaKey&&t.push("meta"),e.shiftKey&&t.push("shift"),t}n(0),n(17);var s={8:"BACKSPACE",9:"TAB",13:"ENTER",16:"SHIFT",17:"CONTROL",18:"ALT",27:"ESCAPE",32:"SPACE",33:"PAGEUP",34:"PAGEDOWN",35:"END",36:"HOME",37:"LEFT",38:"UP",39:"RIGHT",40:"DOWN",45:"INSERT",46:"DELETE",91:"META",92:"META",96:"0",97:"1",98:"2",99:"3",100:"4",101:"5",102:"6",103:"7",104:"8",105:"9",106:"*",107:"+",109:"-",110:".",111:"/",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'"};function a(e){var t=function(e){return window.event?e.keyCode:e.which?e.which:void 0}(e),n=s[t];return null==n&&(n=function(e){return String.fromCharCode(e).toUpperCase().trim()}(t)),n}var u={0:1,2:2,1:3};function l(e,t,n){var r;r=e._eventinfo.is_button_pressed?u[t.button]:null;var i=function(e,t){var n=e.getBoundingClientRect();return[t.clientX-n.left,t.clientY-n.top]}(e.$el.get(0),t),s=o(t),a=e._eventinfo.press_event;e._eventinfo.last_event;return{type:n,pos:i,button:r,is_dragging:null!=a,modifiers:s,delta:null,press_event:a,last_event:null}}function c(e,t,n){var r=o(t);e._eventinfo.last_event;return{type:n,modifiers:r,key_code:a(t),last_event:null}}function f(e){null==e&&(e=100),this._queue=[],this.maxlen=e}r.prototype._mouse_press=function(e){},r.prototype._mouse_release=function(e){},r.prototype._mouse_move=function(e){},r.prototype._mouse_wheel=function(e){},r.prototype._mouse_click=function(e){},r.prototype._mouse_dblclick=function(e){},r.prototype._key_press=function(e){},r.prototype._key_release=function(e){},r.prototype._initialize=function(e){},r.prototype._resize=function(e){},r.prototype._paint=function(e){},r.prototype._event_tick=function(e){},r.prototype.on_mouse_press=function(e){this._mouse_press=e},r.prototype.on_mouse_release=function(e){this._mouse_release=e},r.prototype.on_mouse_move=function(e){this._mouse_move=e},r.prototype.on_mouse_wheel=function(e){this._mouse_wheel=e},r.prototype.on_mouse_dblclick=function(e){this._mouse_dblclick=e},r.prototype.on_key_press=function(e){this._key_press=e},r.prototype.on_key_release=function(e){this._key_release=e},r.prototype.on_initialize=function(e){this._initialize=e},r.prototype.on_resize=function(e){this._resize=e},r.prototype.on_paint=function(e){this._paint=e},r.prototype.on_event_tick=function(e){this._event_tick=e},r.prototype.initialize=function(){var e={type:"initialize"};this._set_size(),this._initialize(e)},r.prototype._set_size=function(e){return null==e&&(e=[this.$el.width(),this.$el.height()]),this.size=e,this.width=e[0],this.height=e[1],e},r.prototype.paint=function(){var e={type:"paint"};this.event_queue.append(e)},r.prototype.update=r.prototype.paint,r.prototype.resize=function(e){var t=function(e,t){return{type:"resize",size:t}}(0,e=this._set_size(e));this.gl.canvas.width=e[0],this.gl.canvas.height=e[1],this.event_queue.append(t),this._resize(t)},r.prototype.event_tick=function(){if(this._event_tick(),this.execute_pending_commands()>0){var e={type:"paint"};this._paint(e)}},r.prototype.is_fullscreen=function(){return i.enabled&i.isFullscreen},r.prototype.toggle_fullscreen=function(){i.enabled&&(i.isFullscreen?(i.exit(),this.resize(this._size)):(this.$el.width("100%").height("100%"),this._size=[this.$el.width(),this.$el.height()],i.request(this.$el[0]),this.resize([screen.width,screen.height])))},r.prototype.deactivate_context_menu=function(){document.oncontextmenu=function(){return!1}},r.prototype.resizable=function(){var e=this;this.$el.resizable({resize:function(t,n){e.resize([n.size.width,n.size.height])}})},f.prototype.clear=function(){this._queue=[]},f.prototype.append=function(e,t){var n,r,i,o=!0;if(null==t&&(t=!0),t){var s=this._queue[this._queue.length-1];if(null!=s){var a=(r=s,(i=(n=e).type)==r.type&&"mouse_move"==i&&n.button==r.button&n.is_dragging==r.is_dragging&n.modifiers.equals(r.modifiers)?["pos"]:[]);if(a.length>0){for(var u=0;u<a.length;u++){var l=a[u];this._queue[this._queue.length-1][l]=e[l]}o=!1}}}o&&this._queue.push(e),this._queue.length>this.maxlen&&(this._queue.shift(),this._queue[0].last_event=null)},f.prototype.get=function(){return this._queue},Object.defineProperty(f.prototype,"length",{get:function(){return this._queue.length}});var h=function(){};h.prototype.init=function(e){!function(e){e.$el.on("resize",(function(t){e.resize([t.width(),t.height()])})),e.event_queue=new f,e._eventinfo={type:null,pos:null,button:null,is_dragging:null,key:null,modifiers:[],press_event:null,last_event:null,delta:null},e._eventinfo.is_button_pressed=0,e.$el.on("mousemove",(function(t){var n=l(e,t,"mouse_move");e._mouse_move(n),e.event_queue.append(n)})),e.$el.on("mousedown",(function(t){++e._eventinfo.is_button_pressed;var n=l(e,t,"mouse_press");e._mouse_press(n),e._eventinfo.press_event=n,e.event_queue.append(n)})),e.$el.on("mouseup",(function(t){--e._eventinfo.is_button_pressed;var n=l(e,t,"mouse_release");e._mouse_release(n),e._eventinfo.press_event=null,e.event_queue.append(n)})),e.$el.on("click",(function(t){e._eventinfo.press_event=null})),e.$el.on("dblclick",(function(t){e._eventinfo.press_event=null})),e.$el.on("wheel",(function(t){var n=l(e,t,"mouse_wheel");n.delta=[.01*t.originalEvent.deltaX,.01*-t.originalEvent.deltaY],e._mouse_wheel(n),e.event_queue.append(n),t.preventDefault(),t.stopPropagation()})),e.$el.on("keydown",(function(t){var n=c(e,t,"key_press");e._key_press(n),e.event_queue.append(n)})),e.$el.on("keyup",(function(t){var n=c(e,t,"key_release");e._key_release(n),e.event_queue.append(n)})),e.$el.on("mouseout",(function(e){}))}(e)},e.exports=new h},function(e,t,n){
/*!
* screenfull
* v4.2.1 - 2019-07-27
* (c) Sindre Sorhus; MIT License
*/
!function(){"use strict";var t="undefined"!=typeof window&&void 0!==window.document?window.document:{},n=e.exports,r="undefined"!=typeof Element&&"ALLOW_KEYBOARD_INPUT"in Element,i=function(){for(var e,n=[["requestFullscreen","exitFullscreen","fullscreenElement","fullscreenEnabled","fullscreenchange","fullscreenerror"],["webkitRequestFullscreen","webkitExitFullscreen","webkitFullscreenElement","webkitFullscreenEnabled","webkitfullscreenchange","webkitfullscreenerror"],["webkitRequestFullScreen","webkitCancelFullScreen","webkitCurrentFullScreenElement","webkitCancelFullScreen","webkitfullscreenchange","webkitfullscreenerror"],["mozRequestFullScreen","mozCancelFullScreen","mozFullScreenElement","mozFullScreenEnabled","mozfullscreenchange","mozfullscreenerror"],["msRequestFullscreen","msExitFullscreen","msFullscreenElement","msFullscreenEnabled","MSFullscreenChange","MSFullscreenError"]],r=0,i=n.length,o={};r<i;r++)if((e=n[r])&&e[1]in t){for(r=0;r<e.length;r++)o[n[0][r]]=e[r];return o}return!1}(),o={change:i.fullscreenchange,error:i.fullscreenerror},s={request:function(e){return new Promise(function(n,o){var s,a=i.requestFullscreen,u=function(){this.off("change",u),n()}.bind(this);this.on("change",u),e=e||t.documentElement,s=/ Version\/5\.1(?:\.\d+)? Safari\//.test(navigator.userAgent)?e[a]():e[a](r?Element.ALLOW_KEYBOARD_INPUT:{}),Promise.resolve(s).catch(o)}.bind(this))},exit:function(){return new Promise(function(e){if(this.isFullscreen){var n=function(){this.off("change",n),e()}.bind(this);t[i.exitFullscreen](),this.on("change",n)}else e()}.bind(this))},toggle:function(e){return this.isFullscreen?this.exit():this.request(e)},onchange:function(e){this.on("change",e)},onerror:function(e){this.on("error",e)},on:function(e,n){var r=o[e];r&&t.addEventListener(r,n,!1)},off:function(e,n){var r=o[e];r&&t.removeEventListener(r,n,!1)},raw:i};i?(Object.defineProperties(s,{isFullscreen:{get:function(){return Boolean(t[i.fullscreenElement])}},element:{enumerable:!0,get:function(){return t[i.fullscreenElement]}},enabled:{enumerable:!0,get:function(){return Boolean(t[i.fullscreenEnabled])}}}),n?(e.exports=s,e.exports.default=s):window.screenfull=s):n?e.exports=!1:window.screenfull=!1}()},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Resizable 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(0),n(18),n(20),n(21),n(1),n(3)],void 0===(o="function"==typeof(r=function(e){return e.widget("ui.resizable",e.ui.mouse,{version:"1.12.1",widgetEventPrefix:"resize",options:{alsoResize:!1,animate:!1,animateDuration:"slow",animateEasing:"swing",aspectRatio:!1,autoHide:!1,classes:{"ui-resizable-se":"ui-icon ui-icon-gripsmall-diagonal-se"},containment:!1,ghost:!1,grid:!1,handles:"e,s,se",helper:!1,maxHeight:null,maxWidth:null,minHeight:10,minWidth:10,zIndex:90,resize:null,start:null,stop:null},_num:function(e){return parseFloat(e)||0},_isNumber:function(e){return!isNaN(parseFloat(e))},_hasScroll:function(t,n){if("hidden"===e(t).css("overflow"))return!1;var r,i=n&&"left"===n?"scrollLeft":"scrollTop";return t[i]>0||(t[i]=1,r=t[i]>0,t[i]=0,r)},_create:function(){var t,n=this.options,r=this;this._addClass("ui-resizable"),e.extend(this,{_aspectRatio:!!n.aspectRatio,aspectRatio:n.aspectRatio,originalElement:this.element,_proportionallyResizeElements:[],_helper:n.helper||n.ghost||n.animate?n.helper||"ui-resizable-helper":null}),this.element[0].nodeName.match(/^(canvas|textarea|input|select|button|img)$/i)&&(this.element.wrap(e("<div class='ui-wrapper' style='overflow: hidden;'></div>").css({position:this.element.css("position"),width:this.element.outerWidth(),height:this.element.outerHeight(),top:this.element.css("top"),left:this.element.css("left")})),this.element=this.element.parent().data("ui-resizable",this.element.resizable("instance")),this.elementIsWrapper=!0,t={marginTop:this.originalElement.css("marginTop"),marginRight:this.originalElement.css("marginRight"),marginBottom:this.originalElement.css("marginBottom"),marginLeft:this.originalElement.css("marginLeft")},this.element.css(t),this.originalElement.css("margin",0),this.originalResizeStyle=this.originalElement.css("resize"),this.originalElement.css("resize","none"),this._proportionallyResizeElements.push(this.originalElement.css({position:"static",zoom:1,display:"block"})),this.originalElement.css(t),this._proportionallyResize()),this._setupHandles(),n.autoHide&&e(this.element).on("mouseenter",(function(){n.disabled||(r._removeClass("ui-resizable-autohide"),r._handles.show())})).on("mouseleave",(function(){n.disabled||r.resizing||(r._addClass("ui-resizable-autohide"),r._handles.hide())})),this._mouseInit()},_destroy:function(){this._mouseDestroy();var t,n=function(t){e(t).removeData("resizable").removeData("ui-resizable").off(".resizable").find(".ui-resizable-handle").remove()};return this.elementIsWrapper&&(n(this.element),t=this.element,this.originalElement.css({position:t.css("position"),width:t.outerWidth(),height:t.outerHeight(),top:t.css("top"),left:t.css("left")}).insertAfter(t),t.remove()),this.originalElement.css("resize",this.originalResizeStyle),n(this.originalElement),this},_setOption:function(e,t){switch(this._super(e,t),e){case"handles":this._removeHandles(),this._setupHandles()}},_setupHandles:function(){var t,n,r,i,o,s=this.options,a=this;if(this.handles=s.handles||(e(".ui-resizable-handle",this.element).length?{n:".ui-resizable-n",e:".ui-resizable-e",s:".ui-resizable-s",w:".ui-resizable-w",se:".ui-resizable-se",sw:".ui-resizable-sw",ne:".ui-resizable-ne",nw:".ui-resizable-nw"}:"e,s,se"),this._handles=e(),this.handles.constructor===String)for("all"===this.handles&&(this.handles="n,e,s,w,se,sw,ne,nw"),r=this.handles.split(","),this.handles={},n=0;n<r.length;n++)i="ui-resizable-"+(t=e.trim(r[n])),o=e("<div>"),this._addClass(o,"ui-resizable-handle "+i),o.css({zIndex:s.zIndex}),this.handles[t]=".ui-resizable-"+t,this.element.append(o);this._renderAxis=function(t){var n,r,i,o;for(n in t=t||this.element,this.handles)this.handles[n].constructor===String?this.handles[n]=this.element.children(this.handles[n]).first().show():(this.handles[n].jquery||this.handles[n].nodeType)&&(this.handles[n]=e(this.handles[n]),this._on(this.handles[n],{mousedown:a._mouseDown})),this.elementIsWrapper&&this.originalElement[0].nodeName.match(/^(textarea|input|select|button)$/i)&&(r=e(this.handles[n],this.element),o=/sw|ne|nw|se|n|s/.test(n)?r.outerHeight():r.outerWidth(),i=["padding",/ne|nw|n/.test(n)?"Top":/se|sw|s/.test(n)?"Bottom":/^e$/.test(n)?"Right":"Left"].join(""),t.css(i,o),this._proportionallyResize()),this._handles=this._handles.add(this.handles[n])},this._renderAxis(this.element),this._handles=this._handles.add(this.element.find(".ui-resizable-handle")),this._handles.disableSelection(),this._handles.on("mouseover",(function(){a.resizing||(this.className&&(o=this.className.match(/ui-resizable-(se|sw|ne|nw|n|e|s|w)/i)),a.axis=o&&o[1]?o[1]:"se")})),s.autoHide&&(this._handles.hide(),this._addClass("ui-resizable-autohide"))},_removeHandles:function(){this._handles.remove()},_mouseCapture:function(t){var n,r,i=!1;for(n in this.handles)((r=e(this.handles[n])[0])===t.target||e.contains(r,t.target))&&(i=!0);return!this.options.disabled&&i},_mouseStart:function(t){var n,r,i,o=this.options,s=this.element;return this.resizing=!0,this._renderProxy(),n=this._num(this.helper.css("left")),r=this._num(this.helper.css("top")),o.containment&&(n+=e(o.containment).scrollLeft()||0,r+=e(o.containment).scrollTop()||0),this.offset=this.helper.offset(),this.position={left:n,top:r},this.size=this._helper?{width:this.helper.width(),height:this.helper.height()}:{width:s.width(),height:s.height()},this.originalSize=this._helper?{width:s.outerWidth(),height:s.outerHeight()}:{width:s.width(),height:s.height()},this.sizeDiff={width:s.outerWidth()-s.width(),height:s.outerHeight()-s.height()},this.originalPosition={left:n,top:r},this.originalMousePosition={left:t.pageX,top:t.pageY},this.aspectRatio="number"==typeof o.aspectRatio?o.aspectRatio:this.originalSize.width/this.originalSize.height||1,i=e(".ui-resizable-"+this.axis).css("cursor"),e("body").css("cursor","auto"===i?this.axis+"-resize":i),this._addClass("ui-resizable-resizing"),this._propagate("start",t),!0},_mouseDrag:function(t){var n,r,i=this.originalMousePosition,o=this.axis,s=t.pageX-i.left||0,a=t.pageY-i.top||0,u=this._change[o];return this._updatePrevProperties(),!!u&&(n=u.apply(this,[t,s,a]),this._updateVirtualBoundaries(t.shiftKey),(this._aspectRatio||t.shiftKey)&&(n=this._updateRatio(n,t)),n=this._respectSize(n,t),this._updateCache(n),this._propagate("resize",t),r=this._applyChanges(),!this._helper&&this._proportionallyResizeElements.length&&this._proportionallyResize(),e.isEmptyObject(r)||(this._updatePrevProperties(),this._trigger("resize",t,this.ui()),this._applyChanges()),!1)},_mouseStop:function(t){this.resizing=!1;var n,r,i,o,s,a,u,l=this.options;return this._helper&&(i=(r=(n=this._proportionallyResizeElements).length&&/textarea/i.test(n[0].nodeName))&&this._hasScroll(n[0],"left")?0:this.sizeDiff.height,o=r?0:this.sizeDiff.width,s={width:this.helper.width()-o,height:this.helper.height()-i},a=parseFloat(this.element.css("left"))+(this.position.left-this.originalPosition.left)||null,u=parseFloat(this.element.css("top"))+(this.position.top-this.originalPosition.top)||null,l.animate||this.element.css(e.extend(s,{top:u,left:a})),this.helper.height(this.size.height),this.helper.width(this.size.width),this._helper&&!l.animate&&this._proportionallyResize()),e("body").css("cursor","auto"),this._removeClass("ui-resizable-resizing"),this._propagate("stop",t),this._helper&&this.helper.remove(),!1},_updatePrevProperties:function(){this.prevPosition={top:this.position.top,left:this.position.left},this.prevSize={width:this.size.width,height:this.size.height}},_applyChanges:function(){var e={};return this.position.top!==this.prevPosition.top&&(e.top=this.position.top+"px"),this.position.left!==this.prevPosition.left&&(e.left=this.position.left+"px"),this.size.width!==this.prevSize.width&&(e.width=this.size.width+"px"),this.size.height!==this.prevSize.height&&(e.height=this.size.height+"px"),this.helper.css(e),e},_updateVirtualBoundaries:function(e){var t,n,r,i,o,s=this.options;o={minWidth:this._isNumber(s.minWidth)?s.minWidth:0,maxWidth:this._isNumber(s.maxWidth)?s.maxWidth:1/0,minHeight:this._isNumber(s.minHeight)?s.minHeight:0,maxHeight:this._isNumber(s.maxHeight)?s.maxHeight:1/0},(this._aspectRatio||e)&&(t=o.minHeight*this.aspectRatio,r=o.minWidth/this.aspectRatio,n=o.maxHeight*this.aspectRatio,i=o.maxWidth/this.aspectRatio,t>o.minWidth&&(o.minWidth=t),r>o.minHeight&&(o.minHeight=r),n<o.maxWidth&&(o.maxWidth=n),i<o.maxHeight&&(o.maxHeight=i)),this._vBoundaries=o},_updateCache:function(e){this.offset=this.helper.offset(),this._isNumber(e.left)&&(this.position.left=e.left),this._isNumber(e.top)&&(this.position.top=e.top),this._isNumber(e.height)&&(this.size.height=e.height),this._isNumber(e.width)&&(this.size.width=e.width)},_updateRatio:function(e){var t=this.position,n=this.size,r=this.axis;return this._isNumber(e.height)?e.width=e.height*this.aspectRatio:this._isNumber(e.width)&&(e.height=e.width/this.aspectRatio),"sw"===r&&(e.left=t.left+(n.width-e.width),e.top=null),"nw"===r&&(e.top=t.top+(n.height-e.height),e.left=t.left+(n.width-e.width)),e},_respectSize:function(e){var t=this._vBoundaries,n=this.axis,r=this._isNumber(e.width)&&t.maxWidth&&t.maxWidth<e.width,i=this._isNumber(e.height)&&t.maxHeight&&t.maxHeight<e.height,o=this._isNumber(e.width)&&t.minWidth&&t.minWidth>e.width,s=this._isNumber(e.height)&&t.minHeight&&t.minHeight>e.height,a=this.originalPosition.left+this.originalSize.width,u=this.originalPosition.top+this.originalSize.height,l=/sw|nw|w/.test(n),c=/nw|ne|n/.test(n);return o&&(e.width=t.minWidth),s&&(e.height=t.minHeight),r&&(e.width=t.maxWidth),i&&(e.height=t.maxHeight),o&&l&&(e.left=a-t.minWidth),r&&l&&(e.left=a-t.maxWidth),s&&c&&(e.top=u-t.minHeight),i&&c&&(e.top=u-t.maxHeight),e.width||e.height||e.left||!e.top?e.width||e.height||e.top||!e.left||(e.left=null):e.top=null,e},_getPaddingPlusBorderDimensions:function(e){for(var t=0,n=[],r=[e.css("borderTopWidth"),e.css("borderRightWidth"),e.css("borderBottomWidth"),e.css("borderLeftWidth")],i=[e.css("paddingTop"),e.css("paddingRight"),e.css("paddingBottom"),e.css("paddingLeft")];t<4;t++)n[t]=parseFloat(r[t])||0,n[t]+=parseFloat(i[t])||0;return{height:n[0]+n[2],width:n[1]+n[3]}},_proportionallyResize:function(){if(this._proportionallyResizeElements.length)for(var e,t=0,n=this.helper||this.element;t<this._proportionallyResizeElements.length;t++)e=this._proportionallyResizeElements[t],this.outerDimensions||(this.outerDimensions=this._getPaddingPlusBorderDimensions(e)),e.css({height:n.height()-this.outerDimensions.height||0,width:n.width()-this.outerDimensions.width||0})},_renderProxy:function(){var t=this.element,n=this.options;this.elementOffset=t.offset(),this._helper?(this.helper=this.helper||e("<div style='overflow:hidden;'></div>"),this._addClass(this.helper,this._helper),this.helper.css({width:this.element.outerWidth(),height:this.element.outerHeight(),position:"absolute",left:this.elementOffset.left+"px",top:this.elementOffset.top+"px",zIndex:++n.zIndex}),this.helper.appendTo("body").disableSelection()):this.helper=this.element},_change:{e:function(e,t){return{width:this.originalSize.width+t}},w:function(e,t){var n=this.originalSize;return{left:this.originalPosition.left+t,width:n.width-t}},n:function(e,t,n){var r=this.originalSize;return{top:this.originalPosition.top+n,height:r.height-n}},s:function(e,t,n){return{height:this.originalSize.height+n}},se:function(t,n,r){return e.extend(this._change.s.apply(this,arguments),this._change.e.apply(this,[t,n,r]))},sw:function(t,n,r){return e.extend(this._change.s.apply(this,arguments),this._change.w.apply(this,[t,n,r]))},ne:function(t,n,r){return e.extend(this._change.n.apply(this,arguments),this._change.e.apply(this,[t,n,r]))},nw:function(t,n,r){return e.extend(this._change.n.apply(this,arguments),this._change.w.apply(this,[t,n,r]))}},_propagate:function(t,n){e.ui.plugin.call(this,t,[n,this.ui()]),"resize"!==t&&this._trigger(t,n,this.ui())},plugins:{},ui:function(){return{originalElement:this.originalElement,element:this.element,helper:this.helper,position:this.position,size:this.size,originalSize:this.originalSize,originalPosition:this.originalPosition}}}),e.ui.plugin.add("resizable","animate",{stop:function(t){var n=e(this).resizable("instance"),r=n.options,i=n._proportionallyResizeElements,o=i.length&&/textarea/i.test(i[0].nodeName),s=o&&n._hasScroll(i[0],"left")?0:n.sizeDiff.height,a=o?0:n.sizeDiff.width,u={width:n.size.width-a,height:n.size.height-s},l=parseFloat(n.element.css("left"))+(n.position.left-n.originalPosition.left)||null,c=parseFloat(n.element.css("top"))+(n.position.top-n.originalPosition.top)||null;n.element.animate(e.extend(u,c&&l?{top:c,left:l}:{}),{duration:r.animateDuration,easing:r.animateEasing,step:function(){var r={width:parseFloat(n.element.css("width")),height:parseFloat(n.element.css("height")),top:parseFloat(n.element.css("top")),left:parseFloat(n.element.css("left"))};i&&i.length&&e(i[0]).css({width:r.width,height:r.height}),n._updateCache(r),n._propagate("resize",t)}})}}),e.ui.plugin.add("resizable","containment",{start:function(){var t,n,r,i,o,s,a,u=e(this).resizable("instance"),l=u.options,c=u.element,f=l.containment,h=f instanceof e?f.get(0):/parent/.test(f)?c.parent().get(0):f;h&&(u.containerElement=e(h),/document/.test(f)||f===document?(u.containerOffset={left:0,top:0},u.containerPosition={left:0,top:0},u.parentData={element:e(document),left:0,top:0,width:e(document).width(),height:e(document).height()||document.body.parentNode.scrollHeight}):(t=e(h),n=[],e(["Top","Right","Left","Bottom"]).each((function(e,r){n[e]=u._num(t.css("padding"+r))})),u.containerOffset=t.offset(),u.containerPosition=t.position(),u.containerSize={height:t.innerHeight()-n[3],width:t.innerWidth()-n[1]},r=u.containerOffset,i=u.containerSize.height,o=u.containerSize.width,s=u._hasScroll(h,"left")?h.scrollWidth:o,a=u._hasScroll(h)?h.scrollHeight:i,u.parentData={element:h,left:r.left,top:r.top,width:s,height:a}))},resize:function(t){var n,r,i,o,s=e(this).resizable("instance"),a=s.options,u=s.containerOffset,l=s.position,c=s._aspectRatio||t.shiftKey,f={top:0,left:0},h=s.containerElement,p=!0;h[0]!==document&&/static/.test(h.css("position"))&&(f=u),l.left<(s._helper?u.left:0)&&(s.size.width=s.size.width+(s._helper?s.position.left-u.left:s.position.left-f.left),c&&(s.size.height=s.size.width/s.aspectRatio,p=!1),s.position.left=a.helper?u.left:0),l.top<(s._helper?u.top:0)&&(s.size.height=s.size.height+(s._helper?s.position.top-u.top:s.position.top),c&&(s.size.width=s.size.height*s.aspectRatio,p=!1),s.position.top=s._helper?u.top:0),i=s.containerElement.get(0)===s.element.parent().get(0),o=/relative|absolute/.test(s.containerElement.css("position")),i&&o?(s.offset.left=s.parentData.left+s.position.left,s.offset.top=s.parentData.top+s.position.top):(s.offset.left=s.element.offset().left,s.offset.top=s.element.offset().top),n=Math.abs(s.sizeDiff.width+(s._helper?s.offset.left-f.left:s.offset.left-u.left)),r=Math.abs(s.sizeDiff.height+(s._helper?s.offset.top-f.top:s.offset.top-u.top)),n+s.size.width>=s.parentData.width&&(s.size.width=s.parentData.width-n,c&&(s.size.height=s.size.width/s.aspectRatio,p=!1)),r+s.size.height>=s.parentData.height&&(s.size.height=s.parentData.height-r,c&&(s.size.width=s.size.height*s.aspectRatio,p=!1)),p||(s.position.left=s.prevPosition.left,s.position.top=s.prevPosition.top,s.size.width=s.prevSize.width,s.size.height=s.prevSize.height)},stop:function(){var t=e(this).resizable("instance"),n=t.options,r=t.containerOffset,i=t.containerPosition,o=t.containerElement,s=e(t.helper),a=s.offset(),u=s.outerWidth()-t.sizeDiff.width,l=s.outerHeight()-t.sizeDiff.height;t._helper&&!n.animate&&/relative/.test(o.css("position"))&&e(this).css({left:a.left-i.left-r.left,width:u,height:l}),t._helper&&!n.animate&&/static/.test(o.css("position"))&&e(this).css({left:a.left-i.left-r.left,width:u,height:l})}}),e.ui.plugin.add("resizable","alsoResize",{start:function(){var t=e(this).resizable("instance").options;e(t.alsoResize).each((function(){var t=e(this);t.data("ui-resizable-alsoresize",{width:parseFloat(t.width()),height:parseFloat(t.height()),left:parseFloat(t.css("left")),top:parseFloat(t.css("top"))})}))},resize:function(t,n){var r=e(this).resizable("instance"),i=r.options,o=r.originalSize,s=r.originalPosition,a={height:r.size.height-o.height||0,width:r.size.width-o.width||0,top:r.position.top-s.top||0,left:r.position.left-s.left||0};e(i.alsoResize).each((function(){var t=e(this),r=e(this).data("ui-resizable-alsoresize"),i={},o=t.parents(n.originalElement[0]).length?["width","height"]:["width","height","top","left"];e.each(o,(function(e,t){var n=(r[t]||0)+(a[t]||0);n&&n>=0&&(i[t]=n||null)})),t.css(i)}))},stop:function(){e(this).removeData("ui-resizable-alsoresize")}}),e.ui.plugin.add("resizable","ghost",{start:function(){var t=e(this).resizable("instance"),n=t.size;t.ghost=t.originalElement.clone(),t.ghost.css({opacity:.25,display:"block",position:"relative",height:n.height,width:n.width,margin:0,left:0,top:0}),t._addClass(t.ghost,"ui-resizable-ghost"),!1!==e.uiBackCompat&&"string"==typeof t.options.ghost&&t.ghost.addClass(this.options.ghost),t.ghost.appendTo(t.helper)},resize:function(){var t=e(this).resizable("instance");t.ghost&&t.ghost.css({position:"relative",height:t.size.height,width:t.size.width})},stop:function(){var t=e(this).resizable("instance");t.ghost&&t.helper&&t.helper.get(0).removeChild(t.ghost.get(0))}}),e.ui.plugin.add("resizable","grid",{resize:function(){var t,n=e(this).resizable("instance"),r=n.options,i=n.size,o=n.originalSize,s=n.originalPosition,a=n.axis,u="number"==typeof r.grid?[r.grid,r.grid]:r.grid,l=u[0]||1,c=u[1]||1,f=Math.round((i.width-o.width)/l)*l,h=Math.round((i.height-o.height)/c)*c,p=o.width+f,d=o.height+h,g=r.maxWidth&&r.maxWidth<p,v=r.maxHeight&&r.maxHeight<d,m=r.minWidth&&r.minWidth>p,y=r.minHeight&&r.minHeight>d;r.grid=u,m&&(p+=l),y&&(d+=c),g&&(p-=l),v&&(d-=c),/^(se|s|e)$/.test(a)?(n.size.width=p,n.size.height=d):/^(ne)$/.test(a)?(n.size.width=p,n.size.height=d,n.position.top=s.top-h):/^(sw)$/.test(a)?(n.size.width=p,n.size.height=d,n.position.left=s.left-f):((d-c<=0||p-l<=0)&&(t=n._getPaddingPlusBorderDimensions(this)),d-c>0?(n.size.height=d,n.position.top=s.top-h):(d=c-t.height,n.size.height=d,n.position.top=s.top+o.height-d),p-l>0?(n.size.width=p,n.position.left=s.left-f):(p=l-t.width,n.size.width=p,n.position.left=s.left+o.width-p))}}),e.ui.resizable})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Mouse 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(0),n(19),n(1),n(3)],void 0===(o="function"==typeof(r=function(e){var t=!1;return e(document).on("mouseup",(function(){t=!1})),e.widget("ui.mouse",{version:"1.12.1",options:{cancel:"input, textarea, button, select, option",distance:1,delay:0},_mouseInit:function(){var t=this;this.element.on("mousedown."+this.widgetName,(function(e){return t._mouseDown(e)})).on("click."+this.widgetName,(function(n){if(!0===e.data(n.target,t.widgetName+".preventClickEvent"))return e.removeData(n.target,t.widgetName+".preventClickEvent"),n.stopImmediatePropagation(),!1})),this.started=!1},_mouseDestroy:function(){this.element.off("."+this.widgetName),this._mouseMoveDelegate&&this.document.off("mousemove."+this.widgetName,this._mouseMoveDelegate).off("mouseup."+this.widgetName,this._mouseUpDelegate)},_mouseDown:function(n){if(!t){this._mouseMoved=!1,this._mouseStarted&&this._mouseUp(n),this._mouseDownEvent=n;var r=this,i=1===n.which,o=!("string"!=typeof this.options.cancel||!n.target.nodeName)&&e(n.target).closest(this.options.cancel).length;return!(i&&!o&&this._mouseCapture(n)&&(this.mouseDelayMet=!this.options.delay,this.mouseDelayMet||(this._mouseDelayTimer=setTimeout((function(){r.mouseDelayMet=!0}),this.options.delay)),this._mouseDistanceMet(n)&&this._mouseDelayMet(n)&&(this._mouseStarted=!1!==this._mouseStart(n),!this._mouseStarted)?(n.preventDefault(),0):(!0===e.data(n.target,this.widgetName+".preventClickEvent")&&e.removeData(n.target,this.widgetName+".preventClickEvent"),this._mouseMoveDelegate=function(e){return r._mouseMove(e)},this._mouseUpDelegate=function(e){return r._mouseUp(e)},this.document.on("mousemove."+this.widgetName,this._mouseMoveDelegate).on("mouseup."+this.widgetName,this._mouseUpDelegate),n.preventDefault(),t=!0,0)))}},_mouseMove:function(t){if(this._mouseMoved){if(e.ui.ie&&(!document.documentMode||document.documentMode<9)&&!t.button)return this._mouseUp(t);if(!t.which)if(t.originalEvent.altKey||t.originalEvent.ctrlKey||t.originalEvent.metaKey||t.originalEvent.shiftKey)this.ignoreMissingWhich=!0;else if(!this.ignoreMissingWhich)return this._mouseUp(t)}return(t.which||t.button)&&(this._mouseMoved=!0),this._mouseStarted?(this._mouseDrag(t),t.preventDefault()):(this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=!1!==this._mouseStart(this._mouseDownEvent,t),this._mouseStarted?this._mouseDrag(t):this._mouseUp(t)),!this._mouseStarted)},_mouseUp:function(n){this.document.off("mousemove."+this.widgetName,this._mouseMoveDelegate).off("mouseup."+this.widgetName,this._mouseUpDelegate),this._mouseStarted&&(this._mouseStarted=!1,n.target===this._mouseDownEvent.target&&e.data(n.target,this.widgetName+".preventClickEvent",!0),this._mouseStop(n)),this._mouseDelayTimer&&(clearTimeout(this._mouseDelayTimer),delete this._mouseDelayTimer),this.ignoreMissingWhich=!1,t=!1,n.preventDefault()},_mouseDistanceMet:function(e){return Math.max(Math.abs(this._mouseDownEvent.pageX-e.pageX),Math.abs(this._mouseDownEvent.pageY-e.pageY))>=this.options.distance},_mouseDelayMet:function(){return this.mouseDelayMet},_mouseStart:function(){},_mouseDrag:function(){},_mouseStop:function(){},_mouseCapture:function(){return!0}})})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;i=[n(0),n(1)],void 0===(o="function"==typeof(r=function(e){return e.ui.ie=!!/msie [\w.]+/.exec(navigator.userAgent.toLowerCase())})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;
/*!
 * jQuery UI Disable Selection 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */i=[n(0),n(1)],void 0===(o="function"==typeof(r=function(e){return e.fn.extend({disableSelection:(t="onselectstart"in document.createElement("div")?"selectstart":"mousedown",function(){return this.on(t+".ui-disableSelection",(function(e){e.preventDefault()}))}),enableSelection:function(){return this.off(".ui-disableSelection")}});var t})?r.apply(t,i):r)||(e.exports=o)},function(e,t,n){var r,i,o;i=[n(0),n(1)],void 0===(o="function"==typeof(r=function(e){return e.ui.plugin={add:function(t,n,r){var i,o=e.ui[t].prototype;for(i in r)o.plugins[i]=o.plugins[i]||[],o.plugins[i].push([n,r[i]])},call:function(e,t,n,r){var i,o=e.plugins[t];if(o&&(r||e.element[0].parentNode&&11!==e.element[0].parentNode.nodeType))for(i=0;i<o.length;i++)e.options[o[i][0]]&&o[i][1].apply(e.element,n)}}})?r.apply(t,i):r)||(e.exports=o)},function(e){e.exports=JSON.parse('{"name":"vispy","version":"0.2.2","description":"A Custom Jupyter Widget Library using WebGL for the VisPy Python Library","author":"Vispy Development Team","license":"BSD-3-Clause","main":"lib/index.js","bugs":{"url":"https://github.com/vispy/vispy.js/issues"},"repository":{"type":"git","url":"https://github.com/vispy/vispy.git"},"homepage":"https://github.com/vispy/vispy","keywords":["jupyter","widgets","ipython","ipywidgets","jupyterlab-extension","vispy","webgl","opengl"],"files":["lib/**/*.js","dist/*.js"],"scripts":{"clean":"rimraf dist/","prepare":"webpack --mode=production","test":"echo \\"Error: no test specified\\" && exit 1","watch":"npm-run-all -p watch:*","watch:nbextension":"webpack --watch --mode=development"},"devDependencies":{"npm-run-all":"^4.1.5","npm-watch":"^0.6.0","raw-loader":"^2.0.0","rimraf":"^2.6.3","source-map-loader":"^0.2.4","webpack":"^4.29.6","webpack-cli":"^3.3.0"},"dependencies":{"@jupyter-widgets/base":"^1.0.0 || ^2.0.0","jquery":"^3.4.1","jquery-ui":"^1.12.1","lodash":"^4.17.15","screenfull":"^4.1.0"},"jupyterlab":{"extension":"lib/labplugin"}}')}])}));
//# sourceMappingURL=index.js.map